#ifndef _ParameterHandleValuePairSetImpl_H_
#define _ParameterHandleValuePairSetImpl_H_

#ifndef _RTI_HH_
#define _RTI_HH_
#include <RTI.hh>
#endif

// typedef HandleValuePairSet ParameterHandleValuePairSet;
struct ParameterHandleValuePair
{
   RTI::Handle handle;
   char *buf;
   RTI::ULong size;
};

class ParameterHandleValuePairSetImpl : public RTI::ParameterHandleValuePairSet 

{

// Instances of class HandleValuePairSet are the containers used to pass
// object attribute values and interaction parameter values between the
// Fedrate and the RTI.  These containers hold sets of attribute/parameter
// values indexed by their attribute/parameter handle.  Instances of this
// class are provided to the RTI in the Update Attribute Values and Send
// Interaction service invocations.  Instances of this class are provided
// to the Federate in the Reflect Attribute Values and Receive Interaction
// service invocations.  When instances of HandleValuePairSet are provided
// to the Federate by the RTI, the memory used to store attribute/parameter
// values is valid for use by the federate only within the scope of the
// Reflect Attribute Values or Receive Interaction service invocation.
// Symmetrically, for instances of HandleValuePairSet provided by the Federate
// to the RTI, the memory used to store attribute/parameter values is valid for
// use by the RTI only within the scope of the Update Attribute Values or Send
// Interaction service invocation.  

private:
      RTI::ULong _size;
      RTI::ULong number_of_elements;
      ParameterHandleValuePair *eles;
      //mySet<ParameterHandleValuePair,RTI::ULong> *st;

public:

  ParameterHandleValuePairSetImpl(RTI::ULong _size) ;

  ~ParameterHandleValuePairSetImpl() ;

  RTI::ULong size() const ;
  
  RTI::Handle getHandle( RTI::ULong i) const throw ( RTI::ArrayIndexOutOfBounds);
 
  RTI::ULong 
  getValueLength( RTI::ULong i) const throw ( RTI::ArrayIndexOutOfBounds) ;
  
  void 
  getValue( RTI::ULong i, char* buff, RTI::ULong& valueLength) const
            throw ( RTI::ArrayIndexOutOfBounds) ;
  
  char * getValuePointer( RTI::ULong i, RTI::ULong& valueLength) const
         throw ( RTI::ArrayIndexOutOfBounds) ;
  
  RTI::TransportType getTransportType(void) const throw ( RTI::ArrayIndexOutOfBounds) ;
   
  RTI::OrderType getOrderType(void) const throw ( RTI::ArrayIndexOutOfBounds) ;
   
  RTI::Region* getRegion(void) const throw ( RTI::ArrayIndexOutOfBounds) ;
  
  void add( RTI::Handle h, const char* buff, RTI::ULong valueLength)
       throw ( RTI::ValueLengthExceeded, RTI::ValueCountExceeded) ;
  
  // not guaranteed safe while iterating
  void remove( RTI::Handle h) throw ( RTI::ArrayIndexOutOfBounds ) ;
  
  void moveFrom( const RTI::ParameterHandleValuePairSet& phvps, RTI::ULong& i)
       throw ( RTI::ValueCountExceeded, RTI::ArrayIndexOutOfBounds) ;
  
  // Empty the Set without deallocating space.
  void empty() ;

  inline RTI::ULong start() const ;
  inline RTI::ULong valid(RTI::ULong i) const ;
  inline RTI::ULong next(RTI::ULong i) const ;

   findIndexOf(ParameterHandleValuePair _sele, RTI::ULong *index) const;
  void freeUpElements();
  void rmAt(RTI::ULong index) throw (RTI::ArrayIndexOutOfBounds);
  void put(ParameterHandleValuePair _newele) throw (RTI::ArrayIndexOutOfBounds);
  ParameterHandleValuePair getElementAt( RTI::ULong index) const
    throw ( RTI::ArrayIndexOutOfBounds);

};

#endif
