//File RTIambServices.hh
//Included in RTI.hh


//                RTI Parameter Passing Memory Conventions
//
// C1  In parameter by value.
// C2  Out parameter by pointer value.
// C3  Function return by value.
// C4  In parameter by const pointer value.  Caller provides memory.
//     Caller may free memory or overwrite it upon completion of
//     the call.  Callee must copy during the call anything it
//     wishes to save beyond completion of the call.  Parameter
//     type must define const accessor methods.
// C5  Out parameter by pointer value.  Caller provides reference to object.
//     Callee constructs an instance on the heap (new) and returns.
//     The caller destroys the instance (delete) at its leisure.
// C6  Function return by pointer value.  Callee constructs an instance on
//     the heap (new) and returns a reference.  The caller destroys the
//     instance (delete) at its leisure.
//

//typedef FederateAmbassador *FederateAmbassadorPtr;

#include <stdio.h>
#include <stdlib.h>

#ifndef _RTI_HH_
#define _RTI_HH_
#include "RTI.hh"
#endif

#include <fedtime.hh>

#ifndef _RTIcap_HH_
#define _RTIcap_HH_
#include "RTIcap.hh"
#endif

#ifndef _NPACRTI_HH_
#define _NPACRTI_HH_
#include "npacrti.hh"
#endif

#include <capFederateAmbassadorImpl.h>
#include "debug.h"
#include "Locator.h"

typedef capFederateAmbassadorImpl *capFederateAmbassadorImplPtr;

struct RTIambPrivateRefs {
   RTIcap::RTIambassador_var rtiCapAmb;
   double time ;
   char   myErrBuffer[400];
};

#define R_CALL(body) \
   char buf[400]; \
   try { \
     body \
   } catch(RTIcap::FederationExecutionAlreadyExists feae)  \
     { sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1); \
        throw RTI::FederationExecutionAlreadyExists(""); \
     } \
     catch(CORBA::COMM_FAILURE& ex) \
     { cout << " CORBA::COMM_FAILURE " << endl; \
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1); \
        throw RTI::RTIinternalError(buf); \
     } \
     catch(omniORB::fatalException& ex) \
     { cout << " omniORB::fatalException " << endl; \
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1); \
        throw RTI::RTIinternalError(buf); \
     } \
     catch(RTIcap::RTIinternalError rie)  \
     { throw RTI::RTIinternalError();  } \
     catch(...) \
     {  sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1); \
        throw RTI::RTIinternalError(buf); \
     }

/**
* Get the handle of RTIKernel object
*/
DLL_SYMBOL RTI::RTIambassador::RTIambassador()
throw ( RTI::MemoryExhausted, RTI::RTIinternalError)
{
  int  argv = 0;
  char **args = NULL;
  CORBA::ORB_ptr orb = CORBA::ORB_init(argv,args,"omniORB2");
  CORBA::BOA_ptr boa = orb->BOA_init(argv,args,"omniORB2_BOA");
  npacrti::RTIKernelInterface_var rtiKernel ;
  CORBA::Object_var obj ;

  try
  {
      /*ifstream in("IOR");
      char *buf = new char[400];
      in.getline(buf,400);
      if(DEBUG3) cout << ">" << buf<< "<" << endl;
      in.close();
      */

      char *buf = new char[1024];
      Locator loc;
      if(loc.getIOR(buf)) 
      {
         if(DEBUG3) cout << ">" << buf<< "<" << endl;
      }
      else 
      { cerr << "I could not find any object on any server"<< endl;
             exit(1); 
      }

      obj = orb->string_to_object(buf);
      rtiKernel = npacrti::RTIKernelInterface::_narrow(obj);
      if(DEBUG3) cout << " version " << rtiKernel->getVersion() << endl;

  } catch(CORBA::COMM_FAILURE& ex)
    {
       cerr << "Caught system exception COMM_FAILURE, unable to contact the "
            << "object." << endl;
       exit(1);
    }
    catch(omniORB::fatalException& ex)
    {
       cerr << "Caught omniORB2 fatalException. This indicates a bug is caught "
            << "within omniORB2.\nPlease send a bug report.\n"
            << "The exception was thrown in file: " << ex.file() << "\n"
            << "                            line: " << ex.line() << "\n"
            << "The error message is: " << ex.errmsg() << endl;
       exit(1);
    }
    catch(...)
    {
       cerr << "Caught a system exception." << endl;
       exit(1);
    }

  privateRefs = new RTIambPrivateRefs;
  if(DEBUG3) cout << " Let's check the object pointer " << endl;
  //RTIcap::RTIambassador_var rtiCapAmb1 = rtiKernel->createRTIAmbassador();
  privateRefs->rtiCapAmb  = rtiKernel->createRTIAmbassador();
  privateRefs->time  = 0;
  if(DEBUG3) cout << " I got the object pointer " << endl;

  if(CORBA::is_nil(privateRefs->rtiCapAmb) )
  {
     cerr << " Object has NIL reference! " << endl;
     exit(1);

  }// end of if
  else if(DEBUG3) cout << " Object IS VALID reference! " << endl;


}// end of CONSTRUCTOR

DLL_SYMBOL RTI::RTIambassador::~RTIambassador()
throw (RTI::RTIinternalError)
{
}

////////////////////////////////////
// Federation Management Services //
////////////////////////////////////

/**
// 4.2
*/
DLL_SYMBOL void RTI::RTIambassador::createFederationExecution (
  const char *executionName, // supplied C4
  const char *FED)           // supplied C4      
throw (
  RTI::FederationExecutionAlreadyExists,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{

   char buf[400];
   try {
     privateRefs->rtiCapAmb->createFederationExecution(executionName,FED);

   } catch(RTIcap::FederationExecutionAlreadyExists feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationExecutionAlreadyExists(""); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); 
     }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

}// end of createFederationExecution()

/**
// 4.3
*/
void RTI::RTIambassador::destroyFederationExecution (
  const char *executionName) // supplied C4
throw (
  RTI::FederatesCurrentlyJoined,
  RTI::FederationExecutionDoesNotExist, 
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char buf[400];

   try
   {
     privateRefs->rtiCapAmb->destroyFederationExecution(executionName);

   } catch(RTIcap::FederatesCurrentlyJoined feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederatesCurrentlyJoined(""); 
     }
     catch(RTIcap::FederationExecutionDoesNotExist feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationExecutionDoesNotExist(""); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

}// end of destroyFederationExecution()

/**
// 4.4
   Defines capFederateAmbassador for this federateAmbassador.
   Registers this ambassador to the CORBA version
   Gets the handle for this federate.
   Returns this info.
*/
RTI::FederateHandle                                               // returned C3
RTI::RTIambassador::joinFederationExecution (
  const char                   *yourName,                    // supplied C4
  const char                   *executionName,               // supplied C4
  RTI::FederateAmbassadorPtr   federateAmbassadorReference) // supplied C1
throw (
  RTI::FederateAlreadyExecutionMember,
  RTI::FederationExecutionDoesNotExist,
  RTI::CouldNotOpenFED,
  RTI::ErrorReadingFED,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   char buf[400];
   RTI::FederateHandle ret = -1;
   capFederateAmbassadorImplPtr fed = new capFederateAmbassadorImpl(federateAmbassadorReference);

   try
   {
     ret = privateRefs->rtiCapAmb->joinFederationExecution(yourName,executionName,fed);

   } catch(RTIcap::FederateAlreadyExecutionMember feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateAlreadyExecutionMember(buf); 
     }
     catch(RTIcap::FederationExecutionDoesNotExist feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationExecutionDoesNotExist(buf); 
     }
     catch(RTIcap::CouldNotOpenFED feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::CouldNotOpenFED(buf); 
     }
     catch(RTIcap::ErrorReadingFED feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ErrorReadingFED(buf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(buf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(buf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

   return ret;

}// end of joinFederationExecution()

/**
// 4.5
*/
void RTI::RTIambassador::resignFederationExecution (
  RTI::ResignAction theAction) // supplied C1
throw (
  RTI::FederateOwnsAttributes,
  RTI::FederateNotExecutionMember,
  RTI::InvalidResignAction,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
#ifdef WIN32
   RTIcap::ResignAction act = RTIcap::ResignAction::RELEASE_ATTRIBUTES;
   
   switch(theAction)
   {
      case RTI::ResignAction::RELEASE_ATTRIBUTES :
	       act = RTIcap::ResignAction::RELEASE_ATTRIBUTES; break;
      case RTI::ResignAction::DELETE_OBJECTS :
	       act = RTIcap::ResignAction::DELETE_OBJECTS; break;
      case RTI::ResignAction::DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES:
	       act = RTIcap::ResignAction::DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES; break;
      case RTI::ResignAction::NO_ACTION:
	       act = RTIcap::ResignAction::NO_ACTION; break; 
   }; 
#else
   RTIcap::ResignAction act = theAction;
#endif
   //RTIcap::ResignAction act  = theAction;
   char buf[400];

   try
   {
     privateRefs->rtiCapAmb->resignFederationExecution(act);

   } catch(RTIcap::FederateOwnsAttributes feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateOwnsAttributes(buf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(buf); 
     }
     catch(RTIcap::InvalidResignAction feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidResignAction(buf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

}// end of resignFederationExecution()

/**
// 4.6
*/
void RTI::RTIambassador::registerFederationSynchronizationPoint (       
  const char *label,  // supplied C4
  const char *theTag) // supplied C4
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

void RTI::RTIambassador::registerFederationSynchronizationPoint (       
  const char                *label,    // supplied C4
  const char                *theTag,   // supplied C4
  const RTI::FederateHandleSet&   syncSet)  // supplied C4      
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.9
void RTI::RTIambassador::synchronizationPointAchieved (      
  const char *label) // supplied C4
throw (
  RTI::SynchronizationPointLabelWasNotAnnounced,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.11
void RTI::RTIambassador::requestFederationSave (    
  const char     *label,   // supplied C4
  const RTI::FedTime&  theTime) // supplied C4
throw (
  RTI::FederationTimeAlreadyPassed, 
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

void RTI::RTIambassador::requestFederationSave ( 
  const char *label)     // supplied C4
  throw (
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{}

// 4.13
void RTI::RTIambassador::federateSaveBegun ()
throw (
  RTI::SaveNotInitiated,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.14
void RTI::RTIambassador::federateSaveComplete ()
throw (
  RTI::SaveNotInitiated, RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted, RTI::RestoreInProgress, 
  RTI::RTIinternalError)
{}

void RTI::RTIambassador::federateSaveNotComplete ()
throw (
  RTI::SaveNotInitiated, RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted, RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.16
void RTI::RTIambassador::requestFederationRestore (    
  const char *label) // supplied C4
throw (
  RTI::FederateNotExecutionMember, RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress, RTI::RestoreInProgress, RTI::RTIinternalError)
{}

// 4.20
void RTI::RTIambassador::federateRestoreComplete ()
  throw (
    RTI::RestoreNotRequested, RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted, RTI::RTIinternalError)
{}

void RTI::RTIambassador::federateRestoreNotComplete ()
throw (
  RTI::RestoreNotRequested, RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted, RTI::RTIinternalError)
{}

/////////////////////////////////////
// Declaration Management Services //
/////////////////////////////////////

/**
// 5.2
*/
void RTI::RTIambassador::publishObjectClass (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet& attributeList) // supplied C4
throw (
  RTI::ObjectClassNotDefined, RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember, RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress, RTI::RestoreInProgress, RTI::RTIinternalError)
{

   if(DEBUG2) cout << " in RTI::RTIambassador::publishObjectClass() " << endl;
   int lngth = attributeList.size();
   RTIcap::AttributeHandle *buf = new RTIcap::AttributeHandle[lngth]; 

   for(int i=0;i<attributeList.size();i++) 
       buf[i] = (RTIcap::AttributeHandle) attributeList.getHandle(i);

   RTIcap::AttributeHandleSet att(lngth,lngth,buf);

   privateRefs->rtiCapAmb->publishObjectClass(theClass,att);

   if(DEBUG2) 
      cout << " out of RTI::RTIambassador::publishObjectClass() " << endl;

}// end of publishObjectClass()

/**
// 5.3
*/
void RTI::RTIambassador::unpublishObjectClass (
  RTI::ObjectClassHandle theClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined, 
  RTI::ObjectClassNotPublished,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   char buf[400];
   try
   {
     privateRefs->rtiCapAmb->unpublishObjectClass(theClass);

   } catch(RTIcap::ObjectClassNotDefined feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotDefined(buf); 
     }
     catch(RTIcap::ObjectClassNotPublished feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotPublished(buf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(buf); 
     }
     catch(RTIcap::ConcurrentAccessAttempted feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ConcurrentAccessAttempted(buf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(buf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(buf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

}// end of unpublishObjectClass()

/**
// 5.4
*/
void RTI::RTIambassador::publishInteractionClass (
  RTI::InteractionClassHandle theInteraction) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   char buf[400];

   try
   {
     privateRefs->rtiCapAmb->publishInteractionClass(theInteraction);

   } catch(RTIcap::InteractionClassNotDefined feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InteractionClassNotDefined(buf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(buf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(buf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(buf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

}// end of publishInteractionClass()

/**
// 5.5
*/
void RTI::RTIambassador::unpublishInteractionClass (
  RTI::InteractionClassHandle theInteraction) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   char buf[400];
   try
   {
     privateRefs->rtiCapAmb->unpublishInteractionClass(theInteraction);

   } catch(RTIcap::InteractionClassNotDefined feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InteractionClassNotDefined(buf); 
     }
     catch(RTIcap::InteractionClassNotPublished feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InteractionClassNotPublished(buf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(buf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(buf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(buf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(buf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(buf);
     }

}// end of unpublishInteractionClass()

/**
// 5.6
*/
void RTI::RTIambassador::subscribeObjectClassAttributes (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet& attributeList, // supplied C4
        RTI::Boolean        active )
throw (
  RTI::ObjectClassNotDefined, 
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
    if(DEBUG2) 
       cout << " In RTI::RTIambassador::subscribeObjectClassAttributes " << endl;
    int lngth = attributeList.size();

    if(DEBUG2) cout << " lngth "  << lngth << endl;

    RTIcap::AttributeHandle *buf = new RTIcap::AttributeHandle[lngth];
    for(int i=0;i<attributeList.size();i++) 
         buf[i]=(RTIcap::AttributeHandle)attributeList.getHandle(i);
    if(DEBUG2) cout << " Attribute list is managed " << endl;

    RTIcap::AttributeHandleSet att(lngth,lngth,buf,0);
    if(DEBUG2) cout << " after definition " << endl;

#ifdef WIN32
    RTIcap::Boolean act = RTIcap::Boolean::RTI_TRUE;
	switch(active) 
	{ 
	  case RTI::Boolean::RTI_TRUE  : act = RTIcap::Boolean::RTI_TRUE ; break;
	  case RTI::Boolean::RTI_FALSE : act = RTIcap::Boolean::RTI_FALSE; break;
	}
#else
    RTIcap::Boolean act = active;
#endif


    char *errBuf = privateRefs->myErrBuffer;
    try
    {
       privateRefs->rtiCapAmb->subscribeObjectClassAttributes(theClass,att,act);

       if(DEBUG2) 
          cout << " After privateRefs->rtiCapAmb->subscribeObjectClassAttribute1() " << endl;

     } catch(RTIcap::ObjectClassNotDefined feae) 
       { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotDefined(errBuf); 
     }
     catch(RTIcap::AttributeNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::AttributeNotDefined(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of subscribeObjectClassAttributes()

/**
// 5.7
*/
void RTI::RTIambassador::unsubscribeObjectClass (
  RTI::ObjectClassHandle theClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotSubscribed,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   char *errBuf = privateRefs->myErrBuffer;
   try
   {
     privateRefs->rtiCapAmb->unsubscribeObjectClass(theClass);

   } catch(RTIcap::ObjectClassNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotDefined(errBuf); 
     }
     catch(RTIcap::ObjectClassNotSubscribed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotSubscribed(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of unsubscribeObjectClass()

/**
// 5.8
*/
void RTI::RTIambassador::subscribeInteractionClass (
  RTI::InteractionClassHandle theClass, // supplied C1
  RTI::Boolean           active ) 
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::FederateLoggingServiceCalls,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

#ifdef WIN32
   RTIcap::Boolean act = RTIcap::Boolean::RTI_TRUE;
   switch(active) 
   { 
	  case RTI::Boolean::RTI_TRUE  : act = RTIcap::Boolean::RTI_TRUE ; break;
	  case RTI::Boolean::RTI_FALSE : act = RTIcap::Boolean::RTI_FALSE; break;
   }
#else
   RTIcap::Boolean act = active;
#endif

   char *errBuf = privateRefs->myErrBuffer;
   try
   {
     privateRefs->rtiCapAmb->subscribeInteractionClass(theClass,act);

   } catch(RTIcap::InteractionClassNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InteractionClassNotDefined(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::FederateLoggingServiceCalls feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateLoggingServiceCalls(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of subscribeInteractionClass()

/**
// 5.9
*/
void RTI::RTIambassador::unsubscribeInteractionClass (
  RTI::InteractionClassHandle theClass) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotSubscribed,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   char *errBuf = privateRefs->myErrBuffer;
   try
   {
     privateRefs->rtiCapAmb->unsubscribeInteractionClass(theClass);

   } catch(RTIcap::InteractionClassNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InteractionClassNotDefined(errBuf); 
     }
     catch(RTIcap::InteractionClassNotSubscribed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InteractionClassNotSubscribed(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of unsubscribeInteractionClass()

////////////////////////////////
// Object Management Services //
////////////////////////////////

/**
// 6.2
*/
RTI::ObjectHandle                          // returned C3
RTI::RTIambassador::registerObjectInstance (
        RTI::ObjectClassHandle  theClass,  // supplied C1
  const char              *theObject) // supplied C4
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::ObjectAlreadyRegistered,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTIcap::ObjectID objId;
   char *errBuf = privateRefs->myErrBuffer;

   try
   {
     objId = privateRefs->rtiCapAmb->registerObjectInstanceWithName(theClass,theObject);

   } catch(RTIcap::ObjectClassNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotDefined(errBuf); 
     }
     catch(RTIcap::ObjectClassNotPublished feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotPublished(errBuf); 
     }
     catch(RTIcap::ObjectAlreadyRegistered feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectAlreadyRegistered(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

   return objId;

}// end of registerObjectInstance()

RTI::ObjectHandle                         // returned C3
RTI::RTIambassador::registerObjectInstance (
        RTI::ObjectClassHandle theClass)  // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTIcap::ObjectID objId;
   char *errBuf = privateRefs->myErrBuffer;

   try
   {
     objId = privateRefs->rtiCapAmb->registerObjectInstance(theClass);

   } catch(RTIcap::ObjectClassNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotDefined(errBuf); 
     }
     catch(RTIcap::ObjectClassNotPublished feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectClassNotPublished(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

   return objId;

}// end of registerObjectInstance()

/**
// 6.4
*/
RTI::EventRetractionHandle                               // returned C3
RTI::RTIambassador::updateAttributeValues (
        RTI::ObjectHandle                 theObject,     // supplied C1
  const RTI::AttributeHandleValuePairSet& theAttributes, // supplied C4
  const RTI::FedTime&                     theTime,       // supplied C4
  const char                        *theTag)        // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

  int  i, j; 
  unsigned long valLngth;
  int lngth = theAttributes.size();

  RTIcap::AttributeHandleValuePair *buff = 
                   new RTIcap::AttributeHandleValuePair[lngth];

  RTIcap::AttributeValue *attValue;
  CORBA::Octet *buf1;
  if(DEBUG2)
     cout << " In updateAttributeValuesWithTime for " << theObject  
          << " length : " << lngth << endl;

  for(i=0;i<lngth;i++)
  {
     buff[i].handle = theAttributes.getHandle(i);
     valLngth = theAttributes.getValueLength(i);

     buf1 = new CORBA::Octet[valLngth];
     theAttributes.getValue(i,(char*)buf1,valLngth);
     attValue = new RTIcap::AttributeValue(valLngth,valLngth,buf1);
     buff[i].value = *attValue; 
     delete[] buf1;

  }// end of for

  RTIcap::AttributeHandleValuePairSet att(lngth,lngth,buff);

  RTI::EventRetractionHandle ret;

  RTIcap::EventRetractionHandle ret1 ;

   char *errBuf = privateRefs->myErrBuffer;

  try
  {
     ret1 = privateRefs->rtiCapAmb->updateAttributeValuesWithTime(theObject,att,0.0,theTag);

   } catch(RTIcap::ObjectNotKnown feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::ObjectNotKnown(errBuf); 
     }
     catch(RTIcap::AttributeNotDefined feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::AttributeNotDefined(errBuf); 
     }
     catch(RTIcap::AttributeNotOwned feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::AttributeNotOwned(errBuf); 
     }
     catch(RTIcap::InvalidFederationTime feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidFederationTime(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

  ret.theSerialNumber = ret1.theSerialNumber;
  ret.sendingFederate = ret1.sendingFederate;

  //delete[] buff;

  return ret;

}// end of updateAttributeValues()

/**
*/
void RTI::RTIambassador::updateAttributeValues (
        RTI::ObjectHandle                 theObject,     // supplied C1
  const RTI::AttributeHandleValuePairSet& theAttributes, // supplied C4
  const char                        *theTag)        // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

  int  i, j; 
  unsigned long valLngth;
  int lngth = theAttributes.size();

  RTIcap::AttributeHandleValuePair *buff = 
                   new RTIcap::AttributeHandleValuePair[lngth];

  RTIcap::AttributeValue *attValue;
  CORBA::Octet *buf1;

  for(i=0;i<lngth;i++)
  {
     buff[i].handle = theAttributes.getHandle(i);
     valLngth = theAttributes.getValueLength(i);

     buf1 = new CORBA::Octet[valLngth];
     theAttributes.getValue(i,(char*)buf1,valLngth);
     attValue = new RTIcap::AttributeValue(valLngth,valLngth,buf1);
     buff[i].value = *attValue; 

  }// end of for

  RTIcap::AttributeHandleValuePairSet att(lngth,lngth,buff);


  RTI::EventRetractionHandle ret;
   char *errBuf = privateRefs->myErrBuffer;

  try
  {
     privateRefs->rtiCapAmb->updateAttributeValues(theObject,att,theTag);

  } catch(RTIcap::ObjectNotKnown feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::ObjectNotKnown(errBuf); 
    }
    catch(RTIcap::AttributeNotDefined feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::AttributeNotDefined(errBuf); 
    }
    catch(RTIcap::AttributeNotOwned feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::AttributeNotOwned(errBuf); 
    }
    catch(RTIcap::FederateNotExecutionMember feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::FederateNotExecutionMember(errBuf); 
    }
    catch(RTIcap::SaveInProgress feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::SaveInProgress(errBuf); 
    }
    catch(RTIcap::RestoreInProgress feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RestoreInProgress(errBuf); 
    }
    catch(CORBA::COMM_FAILURE& ex)
    { 
       cout << " CORBA::COMM_FAILURE " << endl;
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RTIinternalError(errBuf); 
    }
    catch(omniORB::fatalException& ex)
    { 
       cout << " omniORB::fatalException " << endl;
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RTIinternalError(errBuf);
    }
    catch(RTIcap::RTIinternalError rie) 
    { throw RTI::RTIinternalError(""); }
    catch(...)
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RTIinternalError(errBuf);
    }

  delete[] buff;

}// end of updateAttributeValues()

/**
// 6.6
*/
RTI::EventRetractionHandle                                // returned C3
RTI::RTIambassador::sendInteraction (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet& theParameters,  // supplied C4
  const RTI::FedTime&                     theTime,        // supplied C4
  const char                        *theTag)         // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

  int  i, j;
  unsigned long valLngth; 

  int lngth = theParameters.size();
  RTIcap::ParameterValue *ele;
  RTIcap::ParameterHandleValuePair *buff = new RTIcap::ParameterHandleValuePair[lngth];

  RTI::EventRetractionHandle ret;

  CORBA::Octet *buf1;

  for(i=0;i<lngth;i++) 
  {
     buff[i].handle = theParameters.getHandle(i);
     valLngth = theParameters.getValueLength(i);
     buf1 = new CORBA::Octet[valLngth];
     theParameters.getValue(i,(char *)buf1,valLngth);
     ele = new RTIcap::ParameterValue(valLngth,valLngth,buf1);
     buff[i].value = *ele;
     
  }// end of for

  RTIcap::ParameterHandleValuePairSet att(lngth,lngth,buff);

  RTIcap::EventRetractionHandle ret1 ;
   char *errBuf = privateRefs->myErrBuffer;

  try
  {
    ret1 = privateRefs->rtiCapAmb->sendInteractionWithTime(theInteraction,att,0.0,theTag);

  } catch(RTIcap::InteractionClassNotDefined feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::InteractionClassNotDefined(errBuf); 
    }
    catch(RTIcap::InteractionClassNotPublished feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::InteractionClassNotPublished(errBuf); 
    }
    catch(RTIcap::InteractionParameterNotDefined feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::InteractionParameterNotDefined(errBuf); 
    }
    catch(RTIcap::InvalidFederationTime feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::InvalidFederationTime(errBuf); 
    }
    catch(RTIcap::FederateNotExecutionMember feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::FederateNotExecutionMember(errBuf); 
    }
    catch(RTIcap::SaveInProgress feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::SaveInProgress(errBuf); 
    }
    catch(RTIcap::RestoreInProgress feae) 
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RestoreInProgress(errBuf); 
    }
    catch(CORBA::COMM_FAILURE& ex)
    { 
       cout << " CORBA::COMM_FAILURE " << endl;
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RTIinternalError(errBuf); 
    }
    catch(omniORB::fatalException& ex)
    { 
       cout << " omniORB::fatalException " << endl;
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RTIinternalError(errBuf);
    }
    catch(RTIcap::RTIinternalError rie) 
    { throw RTI::RTIinternalError(""); }
    catch(...)
    { 
       sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
       throw RTI::RTIinternalError(errBuf);
    }

  ret.theSerialNumber = ret1.theSerialNumber;
  ret.sendingFederate = ret1.sendingFederate;

  delete[] buff;

  return ret;

}// end of sendInteraction()

void RTI::RTIambassador::sendInteraction (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet& theParameters,  // supplied C4
  const char                        *theTag)         // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

  int  i, j ; 
  unsigned long valLngth;

  int lngth = theParameters.size();
  RTIcap::ParameterValue *ele;
  RTIcap::ParameterHandleValuePair *buff = new RTIcap::ParameterHandleValuePair[lngth];

  RTI::EventRetractionHandle ret;

  CORBA::Octet *buf1;

  for(i=0;i<lngth;i++) 
  {
     buff[i].handle = theParameters.getHandle(i);
     valLngth = theParameters.getValueLength(i);
     buf1 = new CORBA::Octet[valLngth];
     theParameters.getValue(i,(char *)buf1,valLngth);
     ele = new RTIcap::ParameterValue(valLngth,valLngth,buf1);
     buff[i].value = *ele;
     
  }// end of for

  RTIcap::ParameterHandleValuePairSet att(lngth,lngth,buff);

  privateRefs->rtiCapAmb->sendInteraction(theInteraction,att,theTag);

  delete[] buff;

}// end of sendInteraction() 

// 6.8
RTI::EventRetractionHandle                 // returned C3
RTI::RTIambassador::deleteObjectInstance (
        RTI::ObjectHandle    ObjectHandle, // supplied C1
  const RTI::FedTime&        theTime,      // supplied C4
  const char           *theTag)       // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::DeletePrivilegeNotHeld,
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   RTI::EventRetractionHandle ret;
   RTIcap::EventRetractionHandle ret1 = privateRefs->rtiCapAmb->deleteObjectInstanceWithTime(ObjectHandle,0.0,theTag);
   ret.theSerialNumber = ret1.theSerialNumber;
   ret.sendingFederate = ret1.sendingFederate;

   return ret;

}// end of deleteObjectInstance()

/**
*/
void RTI::RTIambassador::deleteObjectInstance (
        RTI::ObjectHandle    ObjectHandle, // supplied C1
  const char           *theTag)       // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::DeletePrivilegeNotHeld,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   privateRefs->rtiCapAmb->deleteObjectInstance(ObjectHandle,theTag);

}// end of deleteObjectInstance()

// 6.10
void RTI::RTIambassador::localDeleteObjectInstance (
  RTI::ObjectHandle    ObjectHandle) // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::FederateOwnsAttributes,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   //privateRefs->rtiCapAmb->localDeleteObjectInstance(ObjectHandle);
}// end of localDeleteObjectInstance()

// 6.11
void RTI::RTIambassador::changeAttributeTransportationType (
        RTI::ObjectHandle             theObject,     // supplied C1
  const RTI::AttributeHandleSet&      theAttributes, // supplied C4
        RTI::TransportationHandle     theType)       // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::InvalidTransportationHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
}// end of changeAttributeTransportationType()

// 6.12
void RTI::RTIambassador::changeInteractionTransportationType (
  RTI::InteractionClassHandle theClass, // supplied C1
  RTI::TransportationHandle   theType)  // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InvalidTransportationHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
#ifdef WIN32
   RTIcap::TransportType aType = RTIcap::TransportType::BEST_EFFORT;
#else
   RTIcap::TransportType aType = theType;
#endif
   privateRefs->rtiCapAmb->changeInteractionTransportType(theClass,aType);

}// end of changeInteractionTransportationType()

// 6.15
void RTI::RTIambassador::requestObjectAttributeValueUpdate (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   if(DEBUG2) cout << " In requestObjectAttributeValueUpdate() " << endl;

   int lngth = theAttributes.size();
   RTIcap::AttributeHandle *buf = new RTIcap::AttributeHandle[lngth];
   for(int i=0;i<theAttributes.size();i++) buf[i] = theAttributes.getHandle(i);
   RTIcap::AttributeHandleSet att(lngth,lngth,buf);

   if(DEBUG2)
      cout << " before rtiCapAmb->requestObjectAttributeValueUpdate1() " << endl;
 
   privateRefs->rtiCapAmb->requestObjectAttributeValueUpdate(theObject,att);

   if(DEBUG2)
      cout << " after rtiCapAmb->requestObjectAttributeValueUpdate1() " << endl;

}// end of requestObjectAttributeValueUpdate()

void RTI::RTIambassador::requestClassAttributeValueUpdate (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectClassNotDefined, 
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   int lngth = theAttributes.size();
   RTIcap::AttributeHandle *buf = new RTIcap::AttributeHandle[lngth];
   for(int i=0;i<theAttributes.size();i++) buf[i] = theAttributes.getHandle(i);
   RTIcap::AttributeHandleSet att(lngth,lngth,buf);

   if(DEBUG2)
      cout << " before rtiCapAmb->requestObjectAttributeValueUpdate1() " << endl;

   privateRefs->rtiCapAmb->requestClassAttributeValueUpdate(theClass,att);

   if(DEBUG2)
      cout << " after rtiCapAmb->requestObjectAttributeValueUpdate1() " << endl;

}// end of requestClassAttributeValueUpdate()

///////////////////////////////////
// Ownership Management Services //
///////////////////////////////////

// 7.2
void RTI::RTIambassador::unconditionalAttributeOwnershipDivestiture (
        RTI::ObjectHandle                  theObject,     // supplied C1
  const RTI::AttributeHandleSet&           theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown, 
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.3
void RTI::RTIambassador::negotiatedAttributeOwnershipDivestiture (
        RTI::ObjectHandle                  theObject,     // supplied C1
  const RTI::AttributeHandleSet&           theAttributes, // supplied C4
  const char                         *theTag)        // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::AttributeAlreadyBeingDivested,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.7
void RTI::RTIambassador::attributeOwnershipAcquisition (
        RTI::ObjectHandle        theObject,         // supplied C1
  const RTI::AttributeHandleSet& desiredAttributes, // supplied C4
  const char               *theTag)            // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::ObjectClassNotPublished,
  RTI::AttributeNotDefined,
  RTI::AttributeNotPublished,
  RTI::FederateOwnsAttributes,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.8
void RTI::RTIambassador::attributeOwnershipAcquisitionIfAvailable (
        RTI::ObjectHandle        theObject,         // supplied C1
  const RTI::AttributeHandleSet& desiredAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::ObjectClassNotPublished,
  RTI::AttributeNotDefined,
  RTI::AttributeNotPublished,
  RTI::FederateOwnsAttributes,
  RTI::AttributeAlreadyBeingAcquired,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.11
RTI::AttributeHandleSet*                        // returned C6
RTI::RTIambassador::attributeOwnershipReleaseResponse (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::FederateWasNotAskedToReleaseAttribute,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
  RTI::AttributeHandleSet *ret;

   return ret;
}

// 7.12
void RTI::RTIambassador::cancelNegotiatedAttributeOwnershipDivestiture (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::AttributeDivestitureWasNotRequested,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.13
void RTI::RTIambassador::cancelAttributeOwnershipAcquisition (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeAlreadyOwned,
  RTI::AttributeAcquisitionWasNotRequested,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.15
void RTI::RTIambassador::queryAttributeOwnership (
  RTI::ObjectHandle    theObject,    // supplied C1
  RTI::AttributeHandle theAttribute) // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.17
RTI::Boolean                          // returned C3
RTI::RTIambassador::isAttributeOwnedByFederate (
  RTI::ObjectHandle    theObject,     // supplied C1
  RTI::AttributeHandle theAttribute)  // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::Boolean ret;

   return ret;
}

//////////////////////////////
// Time Management Services //
//////////////////////////////

// 8.2
void RTI::RTIambassador::enableTimeRegulation (
  const RTI::FedTime& theFederateTime,  // supplied C4
  const RTI::FedTime& theLookahead)     // supplied C4
throw (
  RTI::TimeRegulationAlreadyEnabled,
  RTI::EnableTimeRegulationPending,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::InvalidFederationTime,
  RTI::InvalidLookahead,
  RTI::ConcurrentAccessAttempted,
  RTI::FederateNotExecutionMember,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   if(DEBUG2)
      cout << " In enableTimeRegulation() " << endl;

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
     privateRefs->rtiCapAmb->enableTimeRegulation(((RTIfedTime&)theFederateTime).getTime(),((RTIfedTime&)theLookahead).getTime());

   } catch(RTIcap::FederationExecutionAlreadyExists feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationExecutionAlreadyExists(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); 
     }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

   if(DEBUG2) cout << " Out of enableTimeRegulation() " << endl;

}// end of enableTimeRegulation()

// 8.4
void RTI::RTIambassador::disableTimeRegulation ()
throw (
  RTI::TimeRegulationWasNotEnabled,
  RTI::ConcurrentAccessAttempted,
  RTI::FederateNotExecutionMember,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
     privateRefs->rtiCapAmb->disableTimeRegulation();

   } catch(RTIcap::TimeRegulationWasNotEnabled feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeRegulationWasNotEnabled(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of disableTimeRegulation()

// 8.5
void RTI::RTIambassador::enableTimeConstrained ()
throw (
  RTI::TimeConstrainedAlreadyEnabled,
  RTI::EnableTimeConstrainedPending,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
     privateRefs->rtiCapAmb->enableTimeConstrained();

   } catch(RTIcap::TimeConstrainedAlreadyEnabled feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeConstrainedAlreadyEnabled(errBuf); 
     }
     catch(RTIcap::EnableTimeConstrainedPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeConstrainedPending(errBuf); 
     }
     catch(RTIcap::TimeAdvanceAlreadyInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeAdvanceAlreadyInProgress(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of enableTimeConstrained()

// 8.7
void RTI::RTIambassador::disableTimeConstrained ()
throw (
  RTI::TimeConstrainedWasNotEnabled,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
     privateRefs->rtiCapAmb->disableTimeConstrained();

   } catch(RTIcap::TimeConstrainedWasNotEnabled feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeConstrainedWasNotEnabled(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of disableTimeConstrained()

/**
// 8.8
*/
void RTI::RTIambassador::timeAdvanceRequest (
 const  RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->timeAdvanceRequest(((RTIfedTime&)theTime).getTime());

   } catch(RTIcap::InvalidFederationTime feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidFederationTime(errBuf); 
     }
     catch(RTIcap::FederationTimeAlreadyPassed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationTimeAlreadyPassed(errBuf); 
     }
     catch(RTIcap::TimeAdvanceAlreadyInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeAdvanceAlreadyInProgress(errBuf); 
     }
     catch(RTIcap::EnableTimeRegulationPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeRegulationPending(errBuf); 
     }
     catch(RTIcap::EnableTimeConstrainedPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeConstrainedPending(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(""); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of timeAdvanceRequest()

/**
// 8.9
*/
void RTI::RTIambassador::timeAdvanceRequestAvailable (
const RTI::FedTime& theTime) // supplied C4
  throw (
    RTI::InvalidFederationTime,
    RTI::FederationTimeAlreadyPassed,
    RTI::TimeAdvanceAlreadyInProgress,
    RTI::EnableTimeRegulationPending,
    RTI::EnableTimeConstrainedPending,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->timeAdvanceRequestAvailable(((RTIfedTime&)theTime).getTime());

   } catch(RTIcap::InvalidFederationTime feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidFederationTime(errBuf); 
     }
     catch(RTIcap::FederationTimeAlreadyPassed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationTimeAlreadyPassed(errBuf); 
     }
     catch(RTIcap::TimeAdvanceAlreadyInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeAdvanceAlreadyInProgress(errBuf); 
     }
     catch(RTIcap::EnableTimeRegulationPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeRegulationPending(errBuf); 
     }
     catch(RTIcap::EnableTimeConstrainedPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeConstrainedPending(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of timeAdvanceRequestAvailable()

/**
// 8.10
*/
void RTI::RTIambassador::nextEventRequest (
  const RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->nextEventRequest(((RTIfedTime&)theTime).getTime());

   } catch(RTIcap::InvalidFederationTime feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidFederationTime(errBuf); 
     }
     catch(RTIcap::FederationTimeAlreadyPassed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationTimeAlreadyPassed(errBuf); 
     }
     catch(RTIcap::TimeAdvanceAlreadyInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeAdvanceAlreadyInProgress(errBuf); 
     }
     catch(RTIcap::EnableTimeRegulationPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeRegulationPending(errBuf); 
     }
     catch(RTIcap::EnableTimeConstrainedPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeConstrainedPending(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of nextEventRequest()

/**
// 8.11
*/
void RTI::RTIambassador::nextEventRequestAvailable (
  const RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,  
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->nextEventRequestAvailable(((RTIfedTime&)theTime).getTime());

   } catch(RTIcap::InvalidFederationTime feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidFederationTime(errBuf); 
     }
     catch(RTIcap::FederationTimeAlreadyPassed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationTimeAlreadyPassed(errBuf); 
     }
     catch(RTIcap::TimeAdvanceAlreadyInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeAdvanceAlreadyInProgress(errBuf); 
     }
     catch(RTIcap::EnableTimeRegulationPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeRegulationPending(errBuf); 
     }
     catch(RTIcap::EnableTimeConstrainedPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeConstrainedPending(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of nextEventRequestAvailable()

/**
// 8.12
*/
void RTI::RTIambassador::flushQueueRequest (
  const RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,  
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->flushQueueRequest(((RTIfedTime&)theTime).getTime());

   } catch(RTIcap::InvalidFederationTime feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidFederationTime(errBuf); 
     }
     catch(RTIcap::FederationTimeAlreadyPassed feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederationTimeAlreadyPassed(errBuf); 
     }
     catch(RTIcap::TimeAdvanceAlreadyInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::TimeAdvanceAlreadyInProgress(errBuf); 
     }
     catch(RTIcap::EnableTimeRegulationPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeRegulationPending(errBuf); 
     }
     catch(RTIcap::EnableTimeConstrainedPending feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::EnableTimeConstrainedPending(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of flushQueueRequest()

/**
// 8.14
*/
void RTI::RTIambassador::enableAsynchronousDelivery()
  throw (
    RTI::AsynchronousDeliveryAlreadyEnabled,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->enableAsynchronousDelivery();

   } catch(RTIcap::AsynchronousDeliveryAlreadyEnabled feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::AsynchronousDeliveryAlreadyEnabled(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of enableAsynchronousDelivery()

/**
// 8.15
*/
void RTI::RTIambassador::disableAsynchronousDelivery()
  throw (
    RTI::AsynchronousDeliveryAlreadyDisabled,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->disableAsynchronousDelivery();

   } catch(RTIcap::AsynchronousDeliveryAlreadyDisabled feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::AsynchronousDeliveryAlreadyDisabled(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of disableAsynchronousDelivery()

/*******************************************************************/
// 8.16
/*******************************************************************/
void RTI::RTIambassador::queryLBTS (
  RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      double lbts = privateRefs->rtiCapAmb->queryLBTS();
      RTI::FedTime *tm = new RTIfedTime(lbts); 
      theTime = *tm;
      delete tm;

   } catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of queryLBTS()

/*******************************************************************/
// 8.17
/*******************************************************************/
void RTI::RTIambassador::queryFederateTime (
  RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      double lbts = privateRefs->rtiCapAmb->queryFederateTime();
      RTI::FedTime *tm = new RTIfedTime(lbts); 
      theTime = *tm;
      delete tm;

   } catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of queryFederateTime()

/*******************************************************************/
// 8.18
/*******************************************************************/
void RTI::RTIambassador::queryMinNextEventTime (
  RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      double lbts = privateRefs->rtiCapAmb->queryMinNextEventTime();
      RTI::FedTime *tm = new RTIfedTime(lbts); 
      theTime = *tm;
      delete tm;

   } catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of queryMinNextEventTime()

/*******************************************************************/
// 8.19
/*******************************************************************/
void RTI::RTIambassador::modifyLookahead (
  const RTI::FedTime& theLookahead) // supplied C4
throw (
  RTI::InvalidLookahead,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      privateRefs->rtiCapAmb->modifyLookahead(((RTIfedTime&)theLookahead).getTime());

   } catch(RTIcap::InvalidLookahead feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::InvalidLookahead(errBuf); 
     }
     catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of modifyLookahead()

/*******************************************************************/
// 8.20
/*******************************************************************/
void RTI::RTIambassador::queryLookahead (
   RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{

   char *errBuf = privateRefs->myErrBuffer;

   try 
   {
      double lbts = privateRefs->rtiCapAmb->queryLookahead();
      RTI::FedTime *tm = new RTIfedTime(lbts);
      theTime = *tm; 

   } catch(RTIcap::FederateNotExecutionMember feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::FederateNotExecutionMember(errBuf); 
     }
     catch(RTIcap::SaveInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::SaveInProgress(errBuf); 
     }
     catch(RTIcap::RestoreInProgress feae) 
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RestoreInProgress(errBuf); 
     }
     catch(CORBA::COMM_FAILURE& ex)
     { 
        cout << " CORBA::COMM_FAILURE " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf); 
     }
     catch(omniORB::fatalException& ex)
     { 
        cout << " omniORB::fatalException " << endl;
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }
     catch(RTIcap::RTIinternalError rie) 
     { throw RTI::RTIinternalError(errBuf); }
     catch(...)
     { 
        sprintf(errBuf,"File %s at line %d.",__FILE__,__LINE__+1);
        throw RTI::RTIinternalError(errBuf);
     }

}// end of queryLookahead()

/*******************************************************************/
// 8.21
/*******************************************************************/
void RTI::RTIambassador::retract (
  RTI::EventRetractionHandle theHandle) // supplied C1
throw (
  RTI::InvalidRetractionHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
}// end of retract()

// 8.23
void RTI::RTIambassador::changeAttributeOrderType (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes, // supplied C4
        RTI::OrderingHandle      theType)       // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::InvalidOrderingHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
}// end of changeAttributeOrderType()

// 8.24
void RTI::RTIambassador::changeInteractionOrderType (
  RTI::InteractionClassHandle theClass, // supplied C1
  RTI::OrderingHandle         theType)  // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InvalidOrderingHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
}// end of changeInteractionOrderType()

//////////////////////////////////
// Data Distribution Management //
//////////////////////////////////

// 9.2
RTI::Region*                           // returned C6
RTI::RTIambassador::createRegion (
  RTI::SpaceHandle theSpace,           // supplied C1
  RTI::ULong       numberOfExtents)    // supplied C1
throw (
  RTI::SpaceNotDefined,
  RTI::InvalidExtents,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::Region* ret;

   return ret;
}

// 9.3
void RTI::RTIambassador::notifyAboutRegionModification (
  RTI::Region &theRegion)  // supplied C4
throw (
  RTI::RegionNotKnown,
  RTI::InvalidExtents,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.4
void RTI::RTIambassador::deleteRegion (
  RTI::Region *theRegion) // supplied C1
throw (
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.5
RTI::ObjectHandle                                  // returned C3
RTI::RTIambassador::registerObjectInstanceWithRegion (
        RTI::ObjectClassHandle theClass,           // supplied C1
  const char             *theObject,          // supplied C4
        RTI::AttributeHandle   theAttributes[],    // supplied C4
        RTI::Region           *theRegions[],       // supplied C4
        RTI::ULong             theNumberOfHandles) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::ObjectAlreadyRegistered,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::ObjectHandle ret;

   return ret;

}// end of registerObjectInstanceWithRegion()


RTI::ObjectHandle                              // returned C3
RTI::RTIambassador::registerObjectInstanceWithRegion (
  RTI::ObjectClassHandle theClass,             // supplied C1
  RTI::AttributeHandle   theAttributes[],      // supplied C4
  RTI::Region           *theRegions[],         // supplied C4
  RTI::ULong             theNumberOfHandles)   // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::ObjectHandle ret;

   return ret;
}

// 9.6
void RTI::RTIambassador::associateRegionForUpdates (
        RTI::Region             &theRegion,     // supplied C4
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet &theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::InvalidRegionContext,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.7
void RTI::RTIambassador::unassociateRegionForUpdates (
  RTI::Region       &theRegion,     // supplied C4
  RTI::ObjectHandle  theObject)     // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::InvalidRegionContext,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.8
void RTI::RTIambassador::subscribeObjectClassAttributesWithRegion (
        RTI::ObjectClassHandle   theClass,      // supplied C1
        RTI::Region             &theRegion,     // supplied C4
  const RTI::AttributeHandleSet &attributeList, // supplied C4
        RTI::Boolean        active )
throw (
  RTI::ObjectClassNotDefined,
  RTI::AttributeNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.9
void RTI::RTIambassador::unsubscribeObjectClassWithRegion (
  RTI::ObjectClassHandle theClass,          // supplied C1
  RTI::Region           &theRegion)         // supplied C4
throw (
  RTI::ObjectClassNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.10
void RTI::RTIambassador::subscribeInteractionClassWithRegion (
  RTI::InteractionClassHandle theClass,        // supplied C1
  RTI::Region                &theRegion,       // supplied C4
  RTI::Boolean           active )
throw (
  RTI::InteractionClassNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateLoggingServiceCalls,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.11
void RTI::RTIambassador::unsubscribeInteractionClassWithRegion (
  RTI::InteractionClassHandle theClass,  // supplied C1
  RTI::Region                &theRegion) // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotSubscribed,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.12
RTI::EventRetractionHandle                                // returned C3
RTI::RTIambassador::sendInteractionWithRegion (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet &theParameters,  // supplied C4
  const RTI::FedTime&                     theTime,        // supplied C4
  const char                        *theTag,         // supplied C4
  const RTI::Region                      &theRegion)      // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::InvalidFederationTime,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
  RTI::EventRetractionHandle ret;
  return ret;
}

void RTI::RTIambassador::sendInteractionWithRegion (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet &theParameters,  // supplied C4
  const char                        *theTag,         // supplied C4
  const RTI::Region                      &theRegion)      // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.13
void RTI::RTIambassador::requestClassAttributeValueUpdateWithRegion (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet &theAttributes, // supplied C4
  const RTI::Region             &theRegion)     // supplied C4
throw (
  RTI::ObjectClassNotDefined, 
  RTI::AttributeNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

//////////////////////////
// RTI Support Services //
//////////////////////////

// 10.2
RTI::ObjectClassHandle      // returned C3
RTI::RTIambassador::getObjectClassHandle (
  const char *theName) // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::ObjectClassHandle ret ;
   try
   {

      if(DEBUG2)
         cout << " Before rtiCapAmb->getObjectClassHandle( " 
              << theName << endl;

      ret = privateRefs->rtiCapAmb->getObjectClassHandle(theName);

      if(DEBUG2)
         cout << " After rtiCapAmb->getObjectClassHandle( " 
              << theName << ") = " << ret << endl;

   } catch(RTIcap::NameNotFound &r1)
     { 
        cout << " RTIcap::NameNotFound " << endl;
        throw RTI::NameNotFound(""); 
     }
     catch(RTIcap::FederateNotExecutionMember &e2)
     { 
        cout << " RTIcap::FederateNotExecutionMember " << endl;
        throw RTI::NameNotFound(""); 
     }
     catch(RTIcap::ConcurrentAccessAttempted &e3)
     { 
        cout << " RTIcap::ConcurrentAccessAttempted " << endl;
        throw RTI::NameNotFound(""); 
     }
     catch(RTIcap::RTIinternalError &e4)
     { 
        cout << " RTIcap::RTIinternalError " << endl;
        throw RTI::NameNotFound(""); 
     }
     catch(...)
     { 
        cout << " RTIcap::RTIinternalError " << endl;
        throw RTI::NameNotFound(""); 
     }

   return ret;

}// end of getObjectClassHandle()

// 10.3
char *                         // returned C6    
RTI::RTIambassador::getObjectClassName (
  RTI::ObjectClassHandle theHandle) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret ;
   ret = privateRefs->rtiCapAmb->getObjectClassName(theHandle);
   return ret;

}// end of getObjectClassName()

// 10.4
RTI::AttributeHandle                       // returned C3
RTI::RTIambassador::getAttributeHandle (
  const char             *theName,    // supplied C4
        RTI::ObjectClassHandle whichClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::AttributeHandle ret;

   if(DEBUG2)
      cout << " Looking for  " << theName << " in " << whichClass << endl;

   ret = privateRefs->rtiCapAmb->getAttributeHandle(theName,whichClass);

   if(DEBUG2)
      cout << " Handle for  " << theName << " in " << whichClass 
           << " is " << ret << endl;

   return ret;

}// end of getAttributeHandle()

// 10.5
char *                          // returned C6 
RTI::RTIambassador::getAttributeName (
  RTI::AttributeHandle   theHandle,  // supplied C1
  RTI::ObjectClassHandle whichClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret ;
   ret = privateRefs->rtiCapAmb->getAttributeName(theHandle,whichClass);
   return ret;

}// end of getAttributeName()

// 10.6
RTI::InteractionClassHandle      // returned C3
RTI::RTIambassador::getInteractionClassHandle (
  const char *theName)      // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::InteractionClassHandle ret;

   if(DEBUG2)
      cout << " In RTIambassador::getInteractionClassHandle( " << theName 
           << " ) " << endl;

   ret = privateRefs->rtiCapAmb->getInteractionClassHandle(theName);

   return ret;

}// end of getInteractionClassHandle()

// 10.7
char *                              // returned C6 
RTI::RTIambassador::getInteractionClassName (
  RTI::InteractionClassHandle theHandle) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret ;
   ret = privateRefs->rtiCapAmb->getInteractionClassName(theHandle);
   return ret;

}// end of getInteractionClassName()

// 10.8
RTI::ParameterHandle                            // returned C3
RTI::RTIambassador::getParameterHandle (
  const char *theName,                     // supplied C4
        RTI::InteractionClassHandle whichClass) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::ParameterHandle ret;

   if(DEBUG2)
      cout << " ask parameter handle for " << theName << " in class " 
           << whichClass << endl; 

   ret = privateRefs->rtiCapAmb->getParameterHandle(theName,whichClass);

   if(DEBUG2)
      cout << " parameter handle for " << theName << " in class " 
           << whichClass << " is " << ret << endl; 

   return ret;

}// end of getParameterHandle()
    
// 10.9
char *                               // returned C6
RTI::RTIambassador::getParameterName (
  RTI::ParameterHandle        theHandle,  // supplied C1
  RTI::InteractionClassHandle whichClass) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionParameterNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret ;
   ret = privateRefs->rtiCapAmb->getParameterName(theHandle,whichClass);
   return ret; 

}// end of getParameterName()

// 10.10
RTI::ObjectHandle                 // returned C3
RTI::RTIambassador::getObjectInstanceHandle (
  const char *theName)       // supplied C4
throw (
    RTI::ObjectNotKnown,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::RTIinternalError)
{
   RTI::ObjectHandle ret;
   //ret = privateRefs->rtiCapAmb->getObjectInstanceHandle(theName);
   return ret;

}// end of getObjectInstanceHandle()

// 10.11
char *                     // returned C6  
RTI::RTIambassador::getObjectInstanceName (
  RTI::ObjectHandle theHandle)  // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret ;
   //ret = privateRefs->rtiCapAmb->getObjectInstanceName(theHandle);
   return ret;

}// end of getObjectInstanceName()

// 10.12
RTI::SpaceHandle                // returned C3
RTI::RTIambassador::getRoutingSpaceHandle (
  const char *theName)     // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::SpaceHandle ret;

   return ret;

}// end of getRoutingSpaceHandle()

// 10.13
char *                         // returned C6
RTI::RTIambassador::getRoutingSpaceName (
  const RTI::SpaceHandle theHandle) // supplied C4
throw (
  RTI::SpaceNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret = new char[10];

   return ret;

}// end of getRoutingSpaceName()

// 10.14
RTI::DimensionHandle                   // returned C3
RTI::RTIambassador::getDimensionHandle (
  const char         *theName,    // supplied C4
        RTI::SpaceHandle   whichSpace) // supplied C1
throw (
  RTI::SpaceNotDefined,
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::DimensionHandle ret;

   return ret;
}
    
// 10.15
char *                        // returned C6
RTI::RTIambassador::getDimensionName (
  RTI::DimensionHandle theHandle,  // supplied C1
  RTI::SpaceHandle     whichSpace) // supplied C1
throw (
  RTI::SpaceNotDefined,
  RTI::DimensionNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;
}

// 10.16
RTI::SpaceHandle                      // returned C3
RTI::RTIambassador::getAttributeRoutingSpaceHandle (
  RTI::AttributeHandle   theHandle,   // supplied C1
  RTI::ObjectClassHandle whichClass)  // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::SpaceHandle ret;

   return ret;
}

// 10.17
RTI::ObjectClassHandle            // returned C3
RTI::RTIambassador::getObjectClass (
  RTI::ObjectHandle theObject)    // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::ObjectClassHandle ret;
   
   return ret;
}

// 10.18
RTI::SpaceHandle                             // returned C3
RTI::RTIambassador::getInteractionRoutingSpaceHandle (
  RTI::InteractionClassHandle   theHandle)   // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::SpaceHandle ret;

   return ret;
}

// 10.19
RTI::TransportationHandle      // returned C3
RTI::RTIambassador::getTransportationHandle (
  const char *theName)    // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::TransportationHandle ret;

   return ret;
}

// 10.20
char *                            // returned C6 
RTI::RTIambassador::getTransportationName (
  RTI::TransportationHandle theHandle) // supplied C1
throw (
  RTI::InvalidTransportationHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret = new char[10];

   return ret;
}

// 10.21
RTI::OrderingHandle         // returned C3
RTI::RTIambassador::getOrderingHandle (
  const char *theName) // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::OrderingHandle ret;

   return ret;

}// end of getOrderingHandle()

// 10.22
char *                      // returned C6 
RTI::RTIambassador::getOrderingName (
  RTI::OrderingHandle theHandle) // supplied C1
throw (
  RTI::InvalidOrderingHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;

}// end of getOrderingName()

// 10.23
void RTI::RTIambassador::enableClassRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.24
void RTI::RTIambassador::disableClassRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.25
void RTI::RTIambassador::enableAttributeRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.26
void RTI::RTIambassador::disableAttributeRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.27
void RTI::RTIambassador::enableAttributeScopeAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.28
void RTI::RTIambassador::disableAttributeScopeAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.29
void RTI::RTIambassador::enableInteractionRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.30
void RTI::RTIambassador::disableInteractionRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 
RTI::Boolean // returned C3
RTI::RTIambassador::tick ()
throw (
  RTI::SpecifiedSaveLabelDoesNotExist,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   if(DEBUG2)
      cout << " In RTI::RTIambassador::tick() " ;

   RTIcap::Boolean ret1 = privateRefs->rtiCapAmb->tick();
   RTI::Boolean ret = (ret1==RTIcap::RTI_TRUE?RTI::RTI_TRUE:RTI::RTI_FALSE);

   if(DEBUG2)
      cout << " received " << ( ret==RTI::RTI_TRUE?" TRUE ":" FALSE ");
    
   return ret;

}// end of tick()

RTI::Boolean             // returned C3
RTI::RTIambassador::tick (
  RTI::TickTime minimum, // supplied C1
  RTI::TickTime maximum) // supplied C1
throw (
  RTI::SpecifiedSaveLabelDoesNotExist,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   if(DEBUG2)
      cout << " In RTI::RTIambassador::tickMinMax() " << endl;

   RTIcap::Boolean ret1 = privateRefs->rtiCapAmb->tickMinMax(minimum,maximum);
   RTI::Boolean ret = (ret1==RTIcap::RTI_TRUE?RTI::RTI_TRUE:RTI::RTI_FALSE);

   if(DEBUG2)
      cout << " out of RTI::RTIambassador::tickMinMax() " 
           << " received " << ( ret==RTI::RTI_TRUE?" TRUE ":" FALSE ");

   return ret;

}// end of tick()

RTI::RegionToken
RTI::RTIambassador::getRegionToken(
  RTI::Region *)
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RegionNotKnown,
  RTI::RTIinternalError)
{
   RTI::RegionToken ret;
   return ret;
}

RTI::Region *
RTI::RTIambassador::getRegion(RTI::RegionToken tk)
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RegionNotKnown,
  RTI::RTIinternalError)
{
   RTI::Region *ret ;

   return ret;

}// end of getRegion()



