//File RTIambServices.hh
//Included in RTI.hh


//                RTI Parameter Passing Memory Conventions
//
// C1  In parameter by value.
// C2  Out parameter by pointer value.
// C3  Function return by value.
// C4  In parameter by const pointer value.  Caller provides memory.
//     Caller may free memory or overwrite it upon completion of
//     the call.  Callee must copy during the call anything it
//     wishes to save beyond completion of the call.  Parameter
//     type must define const accessor methods.
// C5  Out parameter by pointer value.  Caller provides reference to object.
//     Callee constructs an instance on the heap (new) and returns.
//     The caller destroys the instance (delete) at its leisure.
// C6  Function return by pointer value.  Callee constructs an instance on
//     the heap (new) and returns a reference.  The caller destroys the
//     instance (delete) at its leisure.
//

//typedef FederateAmbassador *FederateAmbassadorPtr;

#include "RTIambassadorImpl.h"

/**
* Get the handle of RTIKernel object
*/
RTIambassadorImpl::RTIambassadorImpl()
throw ( RTI::MemoryExhausted, RTI::RTIinternalError)
{
   

}// end of CONSTRUCTOR

RTIambassadorImpl::~RTIambassadorImpl()
throw (RTI::RTIinternalError)
{
}

////////////////////////////////////
// Federation Management Services //
////////////////////////////////////

// 4.2
void RTIambassadorImpl::createFederationExecution (
  const char *executionName, // supplied C4
  const char *FED)           // supplied C4      
throw (
  RTI::FederationExecutionAlreadyExists,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   //try {
     //rtiCapAmb->createFederationExecution(executionName,FED);
     rtiCapAmb->createFederationExecution(executionName);

   //} //catch(RTIcap::FederationExecutionAlreadyExists feae) 
     //{ throw new RTI::FederationExecutionAlreadyExists(); }
     //catch(RTIcap::ConcurrentAccessAttempted caa) 
     //{ throw new RTI::ConcurrentAccessAttempted(); }
     //catch(RTIcap::RTIinternalError rie) 
     //{ throw new RTI::RTIinternalError(); }

}// end of createFederationExecution()

// 4.3
void RTIambassadorImpl::destroyFederationExecution (
  const char *executionName) // supplied C4
throw (
  RTI::FederatesCurrentlyJoined,
  RTI::FederationExecutionDoesNotExist, 
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{}

/**
// 4.4
   Defines capFederateAmbassador for this federateAmbassador.
   Registers this ambassador to the CORBA version
   Gets the handle for this federate.
   Returns this info.
*/
RTI::FederateHandle                                               // returned C3
RTIambassadorImpl::joinFederationExecution (
  const char                   *yourName,                    // supplied C4
  const char                   *executionName,               // supplied C4
        RTI::FederateAmbassadorPtr   federateAmbassadorReference) // supplied C1
throw (
  RTI::FederateAlreadyExecutionMember,
  RTI::FederationExecutionDoesNotExist,
  RTI::CouldNotOpenFED,
  RTI::ErrorReadingFED,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::FederateHandle ret = -1;

   return ret;

}// end of joinFederationExecution()

// 4.5
void RTIambassadorImpl::resignFederationExecution (
  RTI::ResignAction theAction) // supplied C1
throw (
  RTI::FederateOwnsAttributes,
  RTI::FederateNotExecutionMember,
  RTI::InvalidResignAction,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{}

// 4.6
void RTIambassadorImpl::registerFederationSynchronizationPoint (       
  const char *label,  // supplied C4
  const char *theTag) // supplied C4
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

void RTIambassadorImpl::registerFederationSynchronizationPoint (       
  const char                *label,    // supplied C4
  const char                *theTag,   // supplied C4
  const RTI::FederateHandleSet&   syncSet)  // supplied C4      
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.9
void RTIambassadorImpl::synchronizationPointAchieved (      
  const char *label) // supplied C4
throw (
  RTI::SynchronizationPointLabelWasNotAnnounced,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.11
void RTIambassadorImpl::requestFederationSave (    
  const char     *label,   // supplied C4
  const RTI::FedTime&  theTime) // supplied C4
throw (
  RTI::FederationTimeAlreadyPassed, 
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

void RTIambassadorImpl::requestFederationSave ( 
  const char *label)     // supplied C4
  throw (
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{}

// 4.13
void RTIambassadorImpl::federateSaveBegun ()
throw (
  RTI::SaveNotInitiated,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.14
void RTIambassadorImpl::federateSaveComplete ()
throw (
  RTI::SaveNotInitiated, RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted, RTI::RestoreInProgress, 
  RTI::RTIinternalError)
{}

void RTIambassadorImpl::federateSaveNotComplete ()
throw (
  RTI::SaveNotInitiated, RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted, RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 4.16
void RTIambassadorImpl::requestFederationRestore (    
  const char *label) // supplied C4
throw (
  RTI::FederateNotExecutionMember, RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress, RTI::RestoreInProgress, RTI::RTIinternalError)
{}

// 4.20
void RTIambassadorImpl::federateRestoreComplete ()
  throw (
    RTI::RestoreNotRequested, RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted, RTI::RTIinternalError)
{}

void RTIambassadorImpl::federateRestoreNotComplete ()
throw (
  RTI::RestoreNotRequested, RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted, RTI::RTIinternalError)
{}

/////////////////////////////////////
// Declaration Management Services //
/////////////////////////////////////

// 5.2
void RTIambassadorImpl::publishObjectClass (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet& attributeList) // supplied C4
throw (
  RTI::ObjectClassNotDefined, RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember, RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress, RTI::RestoreInProgress, RTI::RTIinternalError)
{}

// 5.3
void RTIambassadorImpl::unpublishObjectClass (
  RTI::ObjectClassHandle theClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined, 
  RTI::ObjectClassNotPublished,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 5.4
void RTIambassadorImpl::publishInteractionClass (
  RTI::InteractionClassHandle theInteraction) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 5.5
void RTIambassadorImpl::unpublishInteractionClass (
  RTI::InteractionClassHandle theInteraction) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 5.6
void RTIambassadorImpl::subscribeObjectClassAttributes (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet& attributeList, // supplied C4
        RTI::Boolean        active )
throw (
  RTI::ObjectClassNotDefined, 
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
}

// 5.7
void RTIambassadorImpl::unsubscribeObjectClass (
  RTI::ObjectClassHandle theClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotSubscribed,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 5.8
void RTIambassadorImpl::subscribeInteractionClass (
  RTI::InteractionClassHandle theClass, // supplied C1
  RTI::Boolean           active ) 
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::FederateLoggingServiceCalls,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 5.9
void RTIambassadorImpl::unsubscribeInteractionClass (
  RTI::InteractionClassHandle theClass) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotSubscribed,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

////////////////////////////////
// Object Management Services //
////////////////////////////////

// 6.2
RTI::ObjectHandle                          // returned C3
RTIambassadorImpl::registerObjectInstance (
        RTI::ObjectClassHandle  theClass,  // supplied C1
  const char              *theObject) // supplied C4
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::ObjectAlreadyRegistered,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::ObjectHandle ret = -1;

   return ret;
}

RTI::ObjectHandle                         // returned C3
RTIambassadorImpl::registerObjectInstance (
        RTI::ObjectClassHandle theClass)  // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::ObjectHandle ret = -1;

   return ret;
}

// 6.4
RTI::EventRetractionHandle                               // returned C3
RTIambassadorImpl::updateAttributeValues (
        RTI::ObjectHandle                 theObject,     // supplied C1
  const RTI::AttributeHandleValuePairSet& theAttributes, // supplied C4
  const RTI::FedTime&                     theTime,       // supplied C4
  const char                        *theTag)        // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
  RTI::EventRetractionHandle ret;

  return ret;
}

void RTIambassadorImpl::updateAttributeValues (
        RTI::ObjectHandle                 theObject,     // supplied C1
  const RTI::AttributeHandleValuePairSet& theAttributes, // supplied C4
  const char                        *theTag)        // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 6.6
RTI::EventRetractionHandle                                // returned C3
RTIambassadorImpl::sendInteraction (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet& theParameters,  // supplied C4
  const RTI::FedTime&                     theTime,        // supplied C4
  const char                        *theTag)         // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::EventRetractionHandle ret;

   return ret;
}

void RTIambassadorImpl::sendInteraction (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet& theParameters,  // supplied C4
  const char                        *theTag)         // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 6.8
RTI::EventRetractionHandle                 // returned C3
RTIambassadorImpl::deleteObjectInstance (
        RTI::ObjectHandle    ObjectHandle, // supplied C1
  const RTI::FedTime&        theTime,      // supplied C4
  const char           *theTag)       // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::DeletePrivilegeNotHeld,
  RTI::InvalidFederationTime,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::EventRetractionHandle ret;

   return ret;
}

void RTIambassadorImpl::deleteObjectInstance (
        RTI::ObjectHandle    ObjectHandle, // supplied C1
  const char           *theTag)       // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::DeletePrivilegeNotHeld,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 6.10
void RTIambassadorImpl::localDeleteObjectInstance (
  RTI::ObjectHandle    ObjectHandle) // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::FederateOwnsAttributes,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 6.11
void RTIambassadorImpl::changeAttributeTransportationType (
        RTI::ObjectHandle             theObject,     // supplied C1
  const RTI::AttributeHandleSet&      theAttributes, // supplied C4
        RTI::TransportationHandle     theType)       // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::InvalidTransportationHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 6.12
void RTIambassadorImpl::changeInteractionTransportationType (
  RTI::InteractionClassHandle theClass, // supplied C1
  RTI::TransportationHandle   theType)  // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InvalidTransportationHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 6.15
void RTIambassadorImpl::requestObjectAttributeValueUpdate (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

void RTIambassadorImpl::requestClassAttributeValueUpdate (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectClassNotDefined, 
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

///////////////////////////////////
// Ownership Management Services //
///////////////////////////////////

// 7.2
void RTIambassadorImpl::unconditionalAttributeOwnershipDivestiture (
        RTI::ObjectHandle                  theObject,     // supplied C1
  const RTI::AttributeHandleSet&           theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown, 
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.3
void RTIambassadorImpl::negotiatedAttributeOwnershipDivestiture (
        RTI::ObjectHandle                  theObject,     // supplied C1
  const RTI::AttributeHandleSet&           theAttributes, // supplied C4
  const char                         *theTag)        // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::AttributeAlreadyBeingDivested,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.7
void RTIambassadorImpl::attributeOwnershipAcquisition (
        RTI::ObjectHandle        theObject,         // supplied C1
  const RTI::AttributeHandleSet& desiredAttributes, // supplied C4
  const char               *theTag)            // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::ObjectClassNotPublished,
  RTI::AttributeNotDefined,
  RTI::AttributeNotPublished,
  RTI::FederateOwnsAttributes,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.8
void RTIambassadorImpl::attributeOwnershipAcquisitionIfAvailable (
        RTI::ObjectHandle        theObject,         // supplied C1
  const RTI::AttributeHandleSet& desiredAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::ObjectClassNotPublished,
  RTI::AttributeNotDefined,
  RTI::AttributeNotPublished,
  RTI::FederateOwnsAttributes,
  RTI::AttributeAlreadyBeingAcquired,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.11
RTI::AttributeHandleSet*                        // returned C6
RTIambassadorImpl::attributeOwnershipReleaseResponse (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::FederateWasNotAskedToReleaseAttribute,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
  RTI::AttributeHandleSet *ret;

   return ret;
}

// 7.12
void RTIambassadorImpl::cancelNegotiatedAttributeOwnershipDivestiture (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::AttributeDivestitureWasNotRequested,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.13
void RTIambassadorImpl::cancelAttributeOwnershipAcquisition (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeAlreadyOwned,
  RTI::AttributeAcquisitionWasNotRequested,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.15
void RTIambassadorImpl::queryAttributeOwnership (
  RTI::ObjectHandle    theObject,    // supplied C1
  RTI::AttributeHandle theAttribute) // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 7.17
RTI::Boolean                          // returned C3
RTIambassadorImpl::isAttributeOwnedByFederate (
  RTI::ObjectHandle    theObject,     // supplied C1
  RTI::AttributeHandle theAttribute)  // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::Boolean ret;

   return ret;
}

//////////////////////////////
// Time Management Services //
//////////////////////////////

// 8.2
void RTIambassadorImpl::enableTimeRegulation (
  const RTI::FedTime& theFederateTime,  // supplied C4
  const RTI::FedTime& theLookahead)     // supplied C4
throw (
  RTI::TimeRegulationAlreadyEnabled,
  RTI::EnableTimeRegulationPending,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::InvalidFederationTime,
  RTI::InvalidLookahead,
  RTI::ConcurrentAccessAttempted,
  RTI::FederateNotExecutionMember,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.4
void RTIambassadorImpl::disableTimeRegulation ()
throw (
  RTI::TimeRegulationWasNotEnabled,
  RTI::ConcurrentAccessAttempted,
  RTI::FederateNotExecutionMember,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.5
void RTIambassadorImpl::enableTimeConstrained ()
throw (
  RTI::TimeConstrainedAlreadyEnabled,
  RTI::EnableTimeConstrainedPending,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.7
void RTIambassadorImpl::disableTimeConstrained ()
throw (
  RTI::TimeConstrainedWasNotEnabled,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.8
void RTIambassadorImpl::timeAdvanceRequest (
 const  RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.9
void RTIambassadorImpl::timeAdvanceRequestAvailable (
const RTI::FedTime& theTime) // supplied C4
  throw (
    RTI::InvalidFederationTime,
    RTI::FederationTimeAlreadyPassed,
    RTI::TimeAdvanceAlreadyInProgress,
    RTI::EnableTimeRegulationPending,
    RTI::EnableTimeConstrainedPending,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{}

// 8.10
void RTIambassadorImpl::nextEventRequest (
  const RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.11
void RTIambassadorImpl::nextEventRequestAvailable (
  const RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,  
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.12
void RTIambassadorImpl::flushQueueRequest (
  const RTI::FedTime& theTime) // supplied C4
throw (
  RTI::InvalidFederationTime,
  RTI::FederationTimeAlreadyPassed,
  RTI::TimeAdvanceAlreadyInProgress,
  RTI::EnableTimeRegulationPending,
  RTI::EnableTimeConstrainedPending,  
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.14
void RTIambassadorImpl::enableAsynchronousDelivery()
  throw (
    RTI::AsynchronousDeliveryAlreadyEnabled,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{}

// 8.15
void RTIambassadorImpl::disableAsynchronousDelivery()
  throw (
    RTI::AsynchronousDeliveryAlreadyDisabled,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::SaveInProgress,
    RTI::RestoreInProgress,
    RTI::RTIinternalError)
{}

// 8.16
void RTIambassadorImpl::queryLBTS (
  RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.17
void RTIambassadorImpl::queryFederateTime (
  RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.18
void RTIambassadorImpl::queryMinNextEventTime (
  RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.19
void RTIambassadorImpl::modifyLookahead (
  const RTI::FedTime& theLookahead) // supplied C4
throw (
  RTI::InvalidLookahead,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.20
void RTIambassadorImpl::queryLookahead (
   RTI::FedTime& theTime) // returned C5
throw (
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.21
void RTIambassadorImpl::retract (
  RTI::EventRetractionHandle theHandle) // supplied C1
throw (
  RTI::InvalidRetractionHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.23
void RTIambassadorImpl::changeAttributeOrderType (
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet& theAttributes, // supplied C4
        RTI::OrderingHandle      theType)       // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::AttributeNotOwned,
  RTI::InvalidOrderingHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 8.24
void RTIambassadorImpl::changeInteractionOrderType (
  RTI::InteractionClassHandle theClass, // supplied C1
  RTI::OrderingHandle         theType)  // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InvalidOrderingHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

//////////////////////////////////
// Data Distribution Management //
//////////////////////////////////

// 9.2
RTI::Region*                           // returned C6
RTIambassadorImpl::createRegion (
  RTI::SpaceHandle theSpace,           // supplied C1
  RTI::ULong       numberOfExtents)    // supplied C1
throw (
  RTI::SpaceNotDefined,
  RTI::InvalidExtents,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::Region* ret;

   return ret;
}

// 9.3
void RTIambassadorImpl::notifyAboutRegionModification (
  RTI::Region &theRegion)  // supplied C4
throw (
  RTI::RegionNotKnown,
  RTI::InvalidExtents,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.4
void RTIambassadorImpl::deleteRegion (
  RTI::Region *theRegion) // supplied C1
throw (
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.5
RTI::ObjectHandle                                  // returned C3
RTIambassadorImpl::registerObjectInstanceWithRegion (
        RTI::ObjectClassHandle theClass,           // supplied C1
  const char             *theObject,          // supplied C4
        RTI::AttributeHandle   theAttributes[],    // supplied C4
        RTI::Region           *theRegions[],       // supplied C4
        RTI::ULong             theNumberOfHandles) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::ObjectAlreadyRegistered,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::ObjectHandle ret;

   return ret;

}// end of registerObjectInstanceWithRegion()


RTI::ObjectHandle                              // returned C3
RTIambassadorImpl::registerObjectInstanceWithRegion (
  RTI::ObjectClassHandle theClass,             // supplied C1
  RTI::AttributeHandle   theAttributes[],      // supplied C4
  RTI::Region           *theRegions[],         // supplied C4
  RTI::ULong             theNumberOfHandles)   // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::ObjectClassNotPublished,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
   RTI::ObjectHandle ret;

   return ret;
}

// 9.6
void RTIambassadorImpl::associateRegionForUpdates (
        RTI::Region             &theRegion,     // supplied C4
        RTI::ObjectHandle        theObject,     // supplied C1
  const RTI::AttributeHandleSet &theAttributes) // supplied C4
throw (
  RTI::ObjectNotKnown,
  RTI::AttributeNotDefined,
  RTI::InvalidRegionContext,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.7
void RTIambassadorImpl::unassociateRegionForUpdates (
  RTI::Region       &theRegion,     // supplied C4
  RTI::ObjectHandle  theObject)     // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::InvalidRegionContext,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.8
void RTIambassadorImpl::subscribeObjectClassAttributesWithRegion (
        RTI::ObjectClassHandle   theClass,      // supplied C1
        RTI::Region             &theRegion,     // supplied C4
  const RTI::AttributeHandleSet &attributeList, // supplied C4
        RTI::Boolean        active )
throw (
  RTI::ObjectClassNotDefined,
  RTI::AttributeNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.9
void RTIambassadorImpl::unsubscribeObjectClassWithRegion (
  RTI::ObjectClassHandle theClass,          // supplied C1
  RTI::Region           &theRegion)         // supplied C4
throw (
  RTI::ObjectClassNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.10
void RTIambassadorImpl::subscribeInteractionClassWithRegion (
  RTI::InteractionClassHandle theClass,        // supplied C1
  RTI::Region                &theRegion,       // supplied C4
  RTI::Boolean           active )
throw (
  RTI::InteractionClassNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateLoggingServiceCalls,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.11
void RTIambassadorImpl::unsubscribeInteractionClassWithRegion (
  RTI::InteractionClassHandle theClass,  // supplied C1
  RTI::Region                &theRegion) // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotSubscribed,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.12
RTI::EventRetractionHandle                                // returned C3
RTIambassadorImpl::sendInteractionWithRegion (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet &theParameters,  // supplied C4
  const RTI::FedTime&                     theTime,        // supplied C4
  const char                        *theTag,         // supplied C4
  const RTI::Region                      &theRegion)      // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::InvalidFederationTime,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{
  RTI::EventRetractionHandle ret;
  return ret;
}

void RTIambassadorImpl::sendInteractionWithRegion (
        RTI::InteractionClassHandle       theInteraction, // supplied C1
  const RTI::ParameterHandleValuePairSet &theParameters,  // supplied C4
  const char                        *theTag,         // supplied C4
  const RTI::Region                      &theRegion)      // supplied C4
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionClassNotPublished,
  RTI::InteractionParameterNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 9.13
void RTIambassadorImpl::requestClassAttributeValueUpdateWithRegion (
        RTI::ObjectClassHandle   theClass,      // supplied C1
  const RTI::AttributeHandleSet &theAttributes, // supplied C4
  const RTI::Region             &theRegion)     // supplied C4
throw (
  RTI::ObjectClassNotDefined, 
  RTI::AttributeNotDefined,
  RTI::RegionNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

//////////////////////////
// RTI Support Services //
//////////////////////////

// 10.2
RTI::ObjectClassHandle      // returned C3
RTIambassadorImpl::getObjectClassHandle (
  const char *theName) // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::ObjectClassHandle ret;

   return ret;

}// end of getObjectClassHandle()

// 10.3
char *                         // returned C6    
RTIambassadorImpl::getObjectClassName (
  RTI::ObjectClassHandle theHandle) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;

}// end of getObjectClassName()

// 10.4
RTI::AttributeHandle                       // returned C3
RTIambassadorImpl::getAttributeHandle (
  const char             *theName,    // supplied C4
        RTI::ObjectClassHandle whichClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::AttributeHandle ret;

   return ret;

}// end of getAttributeHandle()

// 10.5
char *                          // returned C6 
RTIambassadorImpl::getAttributeName (
  RTI::AttributeHandle   theHandle,  // supplied C1
  RTI::ObjectClassHandle whichClass) // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;

}// end of getAttributeName()

// 10.6
RTI::InteractionClassHandle      // returned C3
RTIambassadorImpl::getInteractionClassHandle (
  const char *theName)      // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::InteractionClassHandle ret;

   return ret;
}

// 10.7
char *                              // returned C6 
RTIambassadorImpl::getInteractionClassName (
  RTI::InteractionClassHandle theHandle) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret = new char[10];

   return ret;

}// end of getInteractionClassName()

// 10.8
RTI::ParameterHandle                            // returned C3
RTIambassadorImpl::getParameterHandle (
  const char *theName,                     // supplied C4
        RTI::InteractionClassHandle whichClass) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::ParameterHandle ret;

   return ret;

}// end of getParameterHandle()
    
// 10.9
char *                               // returned C6
RTIambassadorImpl::getParameterName (
  RTI::ParameterHandle        theHandle,  // supplied C1
  RTI::InteractionClassHandle whichClass) // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::InteractionParameterNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;

}// end of getParameterName()

// 10.10
RTI::ObjectHandle                 // returned C3
RTIambassadorImpl::getObjectInstanceHandle (
  const char *theName)       // supplied C4
throw (
    RTI::ObjectNotKnown,
    RTI::FederateNotExecutionMember,
    RTI::ConcurrentAccessAttempted,
    RTI::RTIinternalError)
{
   RTI::ObjectHandle ret;

   return ret;

}// end of getObjectInstanceHandle()

// 10.11
char *                     // returned C6  
RTIambassadorImpl::getObjectInstanceName (
  RTI::ObjectHandle theHandle)  // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret = new char[10];
   
   return ret;

}// end of getObjectInstanceName()

// 10.12
RTI::SpaceHandle                // returned C3
RTIambassadorImpl::getRoutingSpaceHandle (
  const char *theName)     // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::SpaceHandle ret;

   return ret;

}// end of getRoutingSpaceHandle()

// 10.13
char *                         // returned C6
RTIambassadorImpl::getRoutingSpaceName (
  const RTI::SpaceHandle theHandle) // supplied C4
throw (
  RTI::SpaceNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret = new char[10];

   return ret;

}// end of getRoutingSpaceName()

// 10.14
RTI::DimensionHandle                   // returned C3
RTIambassadorImpl::getDimensionHandle (
  const char         *theName,    // supplied C4
        RTI::SpaceHandle   whichSpace) // supplied C1
throw (
  RTI::SpaceNotDefined,
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::DimensionHandle ret;

   return ret;
}
    
// 10.15
char *                        // returned C6
RTIambassadorImpl::getDimensionName (
  RTI::DimensionHandle theHandle,  // supplied C1
  RTI::SpaceHandle     whichSpace) // supplied C1
throw (
  RTI::SpaceNotDefined,
  RTI::DimensionNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;
}

// 10.16
RTI::SpaceHandle                      // returned C3
RTIambassadorImpl::getAttributeRoutingSpaceHandle (
  RTI::AttributeHandle   theHandle,   // supplied C1
  RTI::ObjectClassHandle whichClass)  // supplied C1
throw (
  RTI::ObjectClassNotDefined,
  RTI::AttributeNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::SpaceHandle ret;

   return ret;
}

// 10.17
RTI::ObjectClassHandle            // returned C3
RTIambassadorImpl::getObjectClass (
  RTI::ObjectHandle theObject)    // supplied C1
throw (
  RTI::ObjectNotKnown,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::ObjectClassHandle ret;
   
   return ret;
}

// 10.18
RTI::SpaceHandle                             // returned C3
RTIambassadorImpl::getInteractionRoutingSpaceHandle (
  RTI::InteractionClassHandle   theHandle)   // supplied C1
throw (
  RTI::InteractionClassNotDefined,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::SpaceHandle ret;

   return ret;
}

// 10.19
RTI::TransportationHandle      // returned C3
RTIambassadorImpl::getTransportationHandle (
  const char *theName)    // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::TransportationHandle ret;

   return ret;
}

// 10.20
char *                            // returned C6 
RTIambassadorImpl::getTransportationName (
  RTI::TransportationHandle theHandle) // supplied C1
throw (
  RTI::InvalidTransportationHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char * ret = new char[10];

   return ret;
}

// 10.21
RTI::OrderingHandle         // returned C3
RTIambassadorImpl::getOrderingHandle (
  const char *theName) // supplied C4
throw (
  RTI::NameNotFound,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::OrderingHandle ret;

   return ret;

}// end of getOrderingHandle()

// 10.22
char *                      // returned C6 
RTIambassadorImpl::getOrderingName (
  RTI::OrderingHandle theHandle) // supplied C1
throw (
  RTI::InvalidOrderingHandle,
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   char *ret = new char[10];

   return ret;

}// end of getOrderingName()

// 10.23
void RTIambassadorImpl::enableClassRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.24
void RTIambassadorImpl::disableClassRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.25
void RTIambassadorImpl::enableAttributeRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.26
void RTIambassadorImpl::disableAttributeRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.27
void RTIambassadorImpl::enableAttributeScopeAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.28
void RTIambassadorImpl::disableAttributeScopeAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.29
void RTIambassadorImpl::enableInteractionRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 10.30
void RTIambassadorImpl::disableInteractionRelevanceAdvisorySwitch()
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::SaveInProgress,
  RTI::RestoreInProgress,
  RTI::RTIinternalError)
{}

// 
RTI::Boolean // returned C3
RTIambassadorImpl::tick ()
throw (
  RTI::SpecifiedSaveLabelDoesNotExist,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::Boolean ret;
   return ret;
}

RTI::Boolean             // returned C3
RTIambassadorImpl::tick (
  RTI::TickTime minimum, // supplied C1
  RTI::TickTime maximum) // supplied C1
throw (
  RTI::SpecifiedSaveLabelDoesNotExist,
  RTI::ConcurrentAccessAttempted,
  RTI::RTIinternalError)
{
   RTI::Boolean ret;
   return ret;
}

RTI::RegionToken
RTIambassadorImpl::getRegionToken(
  RTI::Region *)
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RegionNotKnown,
  RTI::RTIinternalError)
{
   RTI::RegionToken ret;
   return ret;
}

RTI::Region *
RTIambassadorImpl::getRegion(RTI::RegionToken tk)
throw(
  RTI::FederateNotExecutionMember,
  RTI::ConcurrentAccessAttempted,
  RTI::RegionNotKnown,
  RTI::RTIinternalError)
{
   RTI::Region *ret ;

   return ret;

}// end of getRegion()



