/***************************************************************************/
/** For data formating support between federates when they are on          */
/** different machines or written in different languages.                  */
/***************************************************************************/
#ifndef _CDRInputStream_h_
#define _CDRInputStream_h_

#define CDRDouble double
#define CDRFloat  float
#define CDRLong   int
#define CDRChar   char
#define CDRByte   char
#define CDRShort  short

#define CDRDouble_SIZE 8
#define CDRFloat_SIZE  4
#define CDRLong_SIZE   4
#define CDRChar_SIZE   1
#define CDRByte_SIZE   1
#define CDRShort_SIZE  2

#ifdef __WIN32__
#define do_swap        1
#else
#define do_swap        0
#endif

class CDRCoderEncoder
{
   public:
	CDRCoderEncoder() {};
    ~CDRCoderEncoder() {};

    static char* toByte(CDRByte val, unsigned & length)
    {
       length = CDRByte_SIZE;
       char* ret = new char[CDRByte_SIZE];
       *((CDRByte*)ret) = val;
       return ret;
    };

    static char* toByte(CDRShort val, unsigned & length)
    {
       length = CDRShort_SIZE;
       char* ret = new char[CDRShort_SIZE];

       if(!do_swap) *((CDRShort*)ret) = val;
       else
        {
           unsigned char *lp = (unsigned char *) &val;
           for(int i=0;i<4;i++) ret[CDRShort_SIZE-1-i]=lp[i];
        }

       return ret;

    };

    static char* toByte(CDRLong val, unsigned & length)
    {
       length = CDRLong_SIZE;
       char* ret = new char[CDRLong_SIZE];

       if(!do_swap) *((CDRLong*)ret) = val;
       else
        {
           unsigned char *lp = (unsigned char *) &val;
           for(int i=0;i<CDRLong_SIZE;i++) ret[CDRLong_SIZE-1-i]=lp[i];
        }

       return ret;
    };

    static char* toByte(CDRDouble val, unsigned & length)
    {
       length = CDRDouble_SIZE;
       char* ret = new char[CDRDouble_SIZE];

       if(!do_swap) *((CDRDouble*)ret) = val;
       else
        {
           unsigned char *lp = (unsigned char *) &val;
           for(int i=0;i<CDRDouble_SIZE;i++) ret[CDRDouble_SIZE-1-i]=lp[i];
        }

       return ret;

    };

    static void extract(CDRByte& val, char *buf)
    {
       val = *((CDRByte*)buf);
    };

    static void extract(CDRShort& val, char *buf)
    {
       if(!do_swap) val = *((CDRShort*)buf);
       else
        {
           char *ret = new char[CDRShort_SIZE];
           for(int i=0;i<CDRShort_SIZE;i++) ret[CDRShort_SIZE-1-i]=buf[i];
           val = *((CDRShort*)ret);
           delete[] ret;
        }
    };

    static void extract(CDRLong& val, char *buf)
    {
       if(!do_swap) val = *((CDRLong*)buf);
       else
        {
           char *ret = new char[CDRLong_SIZE];
           for(int i=0;i<CDRLong_SIZE;i++) ret[CDRLong_SIZE-1-i]=buf[i];
           val = *((CDRLong*)ret);
           delete[] ret;
        }
    };

    static void extract(CDRDouble& val, char *buf)
    {
       if(!do_swap) val = *((CDRDouble*)buf);
       else
        {
           char *ret = new char[CDRLong_SIZE];
           for(int i=0;i<CDRDouble_SIZE;i++) ret[CDRDouble_SIZE-1-i]=buf[i];
           val = *((CDRDouble*)ret);
           delete[] ret;
        }
    };

};// end of CDRInputStream

#endif
