package owrti;
/**
*/

import RTIcap.*;


/**
* AttributeInfo defines the state of one object instance's one attribute.
* Its 
*
*   Owner : FederateId
*   Transportation Type:
*   Order Type:
*   Region
*   Which ObjectInstance it belongs to
*   Attribute Name
*   Attribute Handle
*
* <br>
*
* Each attribute is mapped to only one region. It can be in default region
* so that this object intance's this particular attribute value can be
* received by all federates which subscribed the Object Class' this
* attribute.
*
* <br>
*
* Otherwise, this region definition decides which federates should receive
* update of this attribute.
*
* <br>
*
* @date May 05, 1998 ( Updated May 05, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
AttributeInfo implements RTIDebug
{

   boolean        ownershipAcqusitionInProgress = false;
   boolean        ownershipTransferPermitted = false;
   short          ownerId = -1; 

   ObjectInstanceRecord father;      // Object 
   RegionInfo           region;  // Associated Region

   short          theAttributeHandle;
   String         theAttributeName;

   RTIcap.TransportType trans;
   RTIcap.OrderType     order;

   /**
   */
   public
   AttributeInfo( ObjectInstanceRecord _father, short _theHandle, 
                  String _theName, short _ownerId, RegionInfo _region,
                  RTIcap.TransportType _trans, RTIcap.OrderType _order)
   {

      father    = _father;
      theAttributeHandle = _theHandle;
      theAttributeName   = _theName;
      trans     = _trans;
      order     = _order;
      ownerId   = _ownerId;
      region    = _region;

   }// end of CONSTRUCTOR

   /**
   */
   public short 
   getHandle() { return theAttributeHandle; }

   /**
   */
   public String 
   getName() { return theAttributeName; }

   /**
   */
   public RTIcap.TransportType 
   getTransportType() { return trans; }

   /**
   */
   public RTIcap.OrderType 
   getOrderType() { return order; }

   /**
   */
   public void
   setTransportType(RTIcap.TransportType _trans) { trans = _trans; }

   /**
   */
   public void
   setOrderType(RTIcap.OrderType _order) { order=_order; }


   /**
   */
   public RegionInfo 
   getRegionInfo() { return region; }

   /**
   */
   public void 
   setRegionInfo(RegionInfo nRegion) { region = nRegion; }

   /**
   */
   public int 
   getRoutingSpaceHandle() { return region.getRoutingSpaceHandle(); }

   /**
   */
   public short 
   getOwnerId() { return ownerId; }

   /**
   */
   public void 
   setOwnerId( short _newOwner) { ownerId = _newOwner; }

   /**
   */
   public boolean 
   getOwnershipTransferPermission() { return ownershipTransferPermitted; }

   /**
   */
   public void 
   setOwnershipTransferPermission() { ownershipTransferPermitted=true; }

   /**
   */
   public void 
   resetOwnershipTransferPermission() { ownershipTransferPermitted=false; }

   /**
   */
   public boolean 
   getOwnershipAcqusitionInProgress() { return ownershipAcqusitionInProgress; }

   /**
   */
   public void 
   setOwnershipAcqusitionInProgress() { ownershipAcqusitionInProgress=true; }

   /**
   */
   public void 
   resetOwnershipAcqusitionInProgress() { ownershipAcqusitionInProgress=false; }

}// end of AttributeInfo

