/**
*  AttributeWatcher is responsible for the subscription/publication of
*  all the attributes of particular class. It will registers all the puiblishing
*  activities and broadcast this all the subscribed federates.
*
*/
package owrti;

import RTIcap.*;
import java.util.*;

/**
*
* @date April 15, 1998 ( Updated at May 06, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class AttributeWatcher implements RTIDebug
{
   boolean      ownershipAcqusitionInProgress = false;
   boolean      ownershipTransferPermitted = false;
   short        ownerId = -1; 
   ObjectRecord father;      // Object 
   short        theHandle;
   String       theName;
   Hashtable    publishers  = new Hashtable();  // Publishers

   RegionInfo   region;     // Associated Region
   RegionInfo   defRegion;  // Default Region Definition in this Space
   Hashtable    subscribing_regions = new Hashtable();
    

   SpaceInfo    r_space;

   /**
   *
   */
   public
   AttributeWatcher(ObjectRecord _boss, short _theHandle, String _theName,
                    short _ownerId) 
   { 
       father = _boss;
       theHandle = _theHandle;
       theName   = _theName;
       ownerId   = _ownerId;
       r_space   = new SpaceInfo();
       defRegion = r_space.getDefaultRegion();

   }// end of CONSTRUCTOR

   /**
   *
   */
   public
   AttributeWatcher(ObjectRecord _boss, short _theHandle, String _theName,
                    short _ownerId, SpaceInfo _space) 
   { 
       father = _boss;
       theHandle = _theHandle;
       theName   = _theName;
       ownerId   = _ownerId;
       r_space   = ( _space != null ? _space : new SpaceInfo() );
       defRegion = r_space.getDefaultRegion();

   }// end of CONSTRUCTOR

   /**
   */
   public RegionInfo 
   getRegionInfo() { return defRegion; }

   /**
   */
   public RegionInfo 
   getDefaultRegion() { return defRegion; }

   /**
   */
   public int 
   getRoutingSpaceHandle() { return r_space.getHandle(); }

   /**
   */
   public Enumeration 
   getSubscribers() { return defRegion.getSubscribers(); }

   /**
   * Add this federate to the default region subscription.
   */
   public synchronized void
   register_subscriber(FederateInfo fedInf)
   {
      if(DEBUG) System.out.println(fedInf.getFederateID()+" is added to list of "+theName);
      defRegion.register_subscriber(fedInf);
      if(DEBUG) System.out.println(fedInf.getFederateID()+" is added to list of "+theName);

   }// end of subscribe()

   /**
   * Remove it from default region.
   */
   public synchronized void
   deregister_subscriber(short fedHandle)
   {
      defRegion.deregister_subscriber(fedHandle);

   }// end of subscribe()

   /**
   *
   */
   public synchronized void
   register_publisher(FederateInfo fedInf)
   {
    
      short fedHandle = fedInf.getFederateID();
      FederateInfo n = (FederateInfo) publishers.get(new Short(fedHandle));
   
      if(n==null) 
      { 
         publishers.put(new Short(fedHandle),fedInf);
         if(DEBUG) System.out.println(fedHandle+" is added to publisher list of "+theName);
      }// end of if 
      
   }// end of register_publisher()

   /**
   *
   */
   public synchronized void
   deregister_publisher(short fedHandle)
   {
    
      FederateInfo n = (FederateInfo) publishers.get(new Short(fedHandle));
   
      /** If it exists, then you can delete */
      if(n!=null) publishers.remove(new Short(fedHandle));
      
   }// end of register_publisher()

   /**
   */
   public synchronized void
   broadcast( int theObject, RTIcap.AttributeHandleValuePair att, double theTime, 
              String theTag, FederateAmbassador _fedHandle, short _fedId )
              throws RTIcap.FederateInternalError, RTIcap.ObjectNotKnown ,
                     RTIcap.InvalidFederationTime, RTIcap.AttributeNotKnown
                     
   {
      if(DEBUG) System.out.println(" in broadcast() message from "+_fedId);
      /** It has to be publisher Federate */

      FederateInfo ret = (FederateInfo) publishers.get(new Short(_fedId));
     

      if(ret!=null)
      {
         RTIcap.AttributeHandleValuePair attSet[] = new RTIcap.AttributeHandleValuePair[1];
         attSet[0] = att;

         RTIcap.EventRetractionHandle retract = new RTIcap.EventRetractionHandle(theTime,0,_fedId);

         RTIEventRepository glbRepo = ret.getGlobalEvtRepository();
         int handle = glbRepo.addEvent(new EvtReflectAttributeValues(theObject,attSet,theTag,retract,theTime));
         EventInfo evtInfo = new EventInfo(handle,EventInfo.GLOBAL);

         /******************************************/
         /** Broadcast everybody in default region */
         /******************************************/
         if(DEBUG) System.out.println(" before defRegion.getSubscribers broadcast() ");
         Enumeration e = defRegion.getSubscribers();
         if(DEBUG) System.out.println(" After defRegion.getSubscribers broadcast() ");

         if( e!= null )
         for(;e.hasMoreElements();)
         {
            FederateInfo n = (FederateInfo) e.nextElement();

            //if(n!=null && key.shortValue()!=_fedId) 
            if(n!=null && n.getFederateID()!=_fedId) 
            {
               n.addEventInfo(evtInfo);
               if(DEBUG) System.out.println(" I sent to "+n.getFederateID());

            }// end of if

         }// end of for

         if(DEBUG) System.out.println(" before DDM broadcast() ");
         /************************************************/
         /** Broadcast everybody in intersecting regions */
         /************************************************/
         ObjectInstanceRecord objRec = father.getInstance(theObject);
         if( objRec != null )
         {
             AttributeInfo attInfo = objRec.getAttributeInfo(att.handle);

             if( attInfo != null ) 
                 broadcastInRegion(evtInfo,attInfo.getRegionInfo());

         }// end of if


      }// end of if
      if(DEBUG) System.out.println(" out of broadcast() ");

   }// end of broadcast()

   /**
   * Add this region to the subscribing regions list
   */
   public final void
   register_subscribingRegion(RegionInfo _region)
   {
      RegionInfo rgn = 
          (RegionInfo) subscribing_regions.get(new Integer(_region.getHandle()));

      if( rgn == null ) 
          subscribing_regions.put(new Integer(_region.getHandle()),_region);

   }// end of register_subscribingRegion()

   /**
   * Remove this region to the subscribing regions list
   */
   public final void
   deregister_subscribingRegion(int regionHandle)
   {
      RegionInfo rgn = 
                (RegionInfo) subscribing_regions.get(new Integer(regionHandle));

      if( rgn != null ) subscribing_regions.remove(new Integer(regionHandle));

   }// end of deregister_subscribingRegion()

   /**
   * Remove this region to the subscribing regions list
   */
   public final void
   deregister_subscribingRegion(RegionInfo _region)
   {
      deregister_subscribingRegion(_region.getHandle());

   }// end of deregister_subscribingRegion()

   /**
   * 2. Broadcast federates which are in the regions which intersect with
   *    the given region.
   */
   public void
   broadcastInRegion( EventInfo evtInfo , RegionInfo region )
   {
      FederateInfo n;

      Enumeration e = subscribing_regions.elements();

      if(region != null && e != null )
         for(;e.hasMoreElements();)
         {
            RegionInfo rgn = (RegionInfo) e.nextElement();

            if( rgn != null && rgn.canIntersect(region) )
                broadcastEventInfo( evtInfo , rgn.getSubscribers() );

         }// end of for

   }// end of broadcastInRegion()

   /**
   * 2. Broadcast federates 
   */
   final void
   broadcastEventInfo( EventInfo evtInfo , Enumeration e )
   {
      FederateInfo n;

      if( e != null )
        for(;e.hasMoreElements();)
        {
           n = (FederateInfo) e.nextElement();
           if(n!=null) n.addEventInfo(evtInfo);

        }// end of for

   }// end of broadcastEventInfo()

}// end of class AttributeWatcher

