/**
*
*/
package owrti;

import RTIcap.*;
import npacrti.*;

/*
*
* @date April 20, 1998 ( Updated at May 05, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
DataDistributionManager extends _DataDistributionManagerInterfaceImplBase
{
    FederationExecution fedEx;
    SpaceNaming         spaceNaming;
    ObjectClassNaming   objNaming ;
    InteractionNaming   interNaming;

    /**
    */
    public
    DataDistributionManager( FederationExecution _fedEx, 
                             SpaceNaming _spaceNaming, 
                             InteractionNaming _interNaming,
                             ObjectClassNaming _objNaming )
    {
              fedEx = _fedEx;
       spaceNaming  = _spaceNaming; 
       objNaming    = _objNaming;
       interNaming  = _interNaming;

    }// end of CONSTRUCTOR

    /**
    * Defines a Region with the given number of Extents in the given
    * Routing space.
    *
    * <ol>
    *  <li> Get SpaceInfo from SpaceNaming
    *  <li> Define RegionInfo
    *  <li> Register this RegionInfo to SpaceInfo
    * </ol>
    *
    */
    public RTIcap.Region 
    createRegion(int theSpace, int numberOfExtents, short theHandle)
        throws RTIcap.SpaceNotDefined, RTIcap.InvalidExtents, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        /** Define Extent set */
        RTIcap.Extent[] extents = new RTIcap.Extent[numberOfExtents];
        RTIcap.Region ret = new RTIcap.Region(extents,theSpace);

        RegionInfo rgn = new RegionInfo(ret,theHandle);
        
        /** Add it to Space Info */
        SpaceInfo spcInf = spaceNaming.getSpaceInfo(theSpace);
        if( spcInf == null ) throw new RTIcap.SpaceNotDefined();

        spcInf.addRegionInfo(rgn);

        /** Add it to Space Naming */
        rgn.setHandle(spaceNaming.addRegionInfo(rgn));

        return ret;

    }// end of createRegion()

 
    /**
    * Changes the Region definitions.
    *
    * <ol>
    *  <li> Get RegionInfo from SpaceNaming
    *  <li> Update RegionInfo
    * </ol>
    *
    */
    public void 
    notifyAboutRegionModification(RTIcap.Region theRegion, short theHandle)
        throws RTIcap.RegionNotKnown, RTIcap.InvalidExtents, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       RegionInfo region = spaceNaming.getRegionInfo(theRegion);
       if(region==null) throw new RTIcap.RegionNotKnown();

       region.modify(theRegion);

    }// end of notifyAboutRegionModification()

    /**
    *  Delete Region definition.
    * <ol>
    *  <li> Get RegionInfo from SpaceNaming
    *  <li> Get its SpaceInfo
    *  <li> Remove it from SpaceInfo
    *  <li> Remove it from SpaceNaming 
    * </ol>
    */
    public void 
    deleteRegion(RTIcap.Region theRegion, short theHandle)
        throws RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
        RegionInfo rgn   = spaceNaming.getRegionInfo(theRegion);
        if( rgn == null )  throw new RTIcap.RegionNotKnown();

        spaceNaming.removeRegionInfo(rgn);

        try
        {  SpaceInfo spInfo = spaceNaming.getSpaceInfo(rgn.getRoutingSpaceHandle());
           spInfo.removeRegion(theRegion);
        } catch(RTIcap.SpaceNotDefined snd) 
          { throw new RTIcap.RTIinternalError();}

    }// end of deleteRegion()

    /**
    *  Defines an object with a name withing given region.
    * <ol>
    *  <li> Get ObjectRecord from ObjectNaming
    *  <li> Define object instance with name.
    *  <li> Associate this object instance with the given region.
    * </ol>
    */
    public int 
    registerObjectInstanceWithRegionAndName(short _class, String theName, 
        short[] attributes, RTIcap.Region[] regions, int theNumberOfhandles, 
        short theFederate)
        throws RTIcap.ObjectClassNotDefined, RTIcap.ObjectClassNotPublished, 
               RTIcap.ObjectAlreadyRegistered, RTIcap.RegionNotKnown, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {

        FederateInfo fedInf = fedEx.getFederateInfo(theFederate);
        if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

        RTIcap.FederateAmbassador fedAmb = fedInf.getFederateAmbassador();
        if(fedAmb==null) throw new RTIcap.RTIinternalError();

        ObjectManagerInterface objManager = fedEx.getObjectManager();

        int ret = objManager.registerObjectInstanceWithName(_class,theName,fedAmb,theFederate);
        ObjectInstanceRecord rec = objNaming.getObjectInstance(ret);

        rec.associateAttributesWithRegions(attributes,regions,spaceNaming);
       
        return ret; 
        
    }// end of registerObjectInstanceWithRegionAndName()

    /**
    *  Defines an object withing given region.
    * <ol>
    *  <li> Get ObjectRecord from ObjectNaming
    *  <li> Define object instance.
    *  <li> Associate this object instance with the given region.
    * </ol>
    *
    */
    public int 
    registerObjectInstanceWithRegion(short _class, short[] attributes, 
        RTIcap.Region[] regions, int theNumberOfhandles, short theFederate)
        throws RTIcap.ObjectClassNotDefined, RTIcap.ObjectClassNotPublished, 
               RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
        FederateInfo fedInf = fedEx.getFederateInfo(theFederate);
        if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

        RTIcap.FederateAmbassador fedAmb = fedInf.getFederateAmbassador();
        if(fedAmb==null) throw new RTIcap.RTIinternalError();

        ObjectManagerInterface objManager = fedEx.getObjectManager();

        int ret = objManager.registerObjectInstance(_class,fedAmb,theFederate);
        ObjectInstanceRecord rec = objNaming.getObjectInstance(ret);

        rec.associateAttributesWithRegions(attributes,regions,spaceNaming);
       
        return ret; 

    }// end of registerObjectInstanceWithRegion()

    /**
    *
    *  Associates Federate with the Region as a subscriber and 
    *  Object's attributes with the given region.
    *
    * <ol>
    *  <li> Get RegionInfo from SpaceNaming
    *  <li> add this federate to this region as a subscriber
    *  <li> Associate object instance's attributes with the given Region
    * </ol>
    *
    */
    public void 
    associateRegionForUpdates(RTIcap.Region theRegion, int theObject, 
        short[] theAttributes, short theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
               RTIcap.InvalidRegionContext, RTIcap.RegionNotKnown, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {

        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.InvalidRegionContext();

        /** I need to get the AttributeWatcher of the instance */
        ObjectInstanceRecord rec = objNaming.getObjectInstance(theObject);
        if(rec==null) throw new RTIcap.ObjectNotKnown();

        rec.associateAttributesWithRegions(theAttributes,theRegion,spaceNaming);
 
    }// end of associateRegionForUpdates()

    /**
    *  Removes Federate's subscription to the given region.
    * <ol>
    *  <li> Get RegionInfo from SpaceNaming
    *  <li> remove this federate from  region's subscription list
    * </ol>
    *
    */
    public void 
    unassociateRegionForUpdates(RTIcap.Region theRegion, int theObject, 
        short theHandle)
        throws RTIcap.RegionNotKnown, RTIcap.ObjectNotKnown, 
               RTIcap.AttributeNotDefined, RTIcap.FederateNotExecutionMember, 
               RTIcap.RTIinternalError
    {
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        /** I need to get the AttributeWatcher of the instance */
        ObjectInstanceRecord rec = objNaming.getObjectInstance(theObject);
        if(rec==null) throw new RTIcap.ObjectNotKnown();

        rec.unassociateAttributesFromRegion(regInf);

    }// end of unassociateRegionForUpdates()

    /**
    *  Associates Federate with the Region as a subscriber and 
    *  Object Class' attributes with the given region.
    *
    * <ol>
    *  <li> Get RegionInfo from SpaceNaming
    *  <li> add this federate to this region as a subscriber
    *  <li> add region to the subscription region list of object class' 
    *       attributes 
    * </ol>
    *
    * Note that Object Class Level Attribute Representations have 
    * multiple subsciption/publishing regions associated with them.
    *
    */
    public void 
    subscribeObjectClassAttributesWithRegion(short theClass, 
        RTIcap.Region theRegion, short[] attributeList, RTIcap._Boolean active, 
        short theHandle)
        throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
               RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
       ObjectRecord rec = objNaming.getObjectRecord(theClass);
       if(rec == null ) throw new RTIcap.ObjectClassNotDefined();

       RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
       if(regInf==null) throw new RTIcap.RegionNotKnown();

       FederateInfo fedInf = fedEx.getFederateInfo(theHandle);
       if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

       /*****************************************************/
       /** Register Federate to this Region as a subscriber */
       /*****************************************************/
       regInf.register_subscriber(fedInf);

       /***********************************************/
       /** Associate Attributes with the given Region */
       /***********************************************/
       rec.subscribeAttributesWithRegion(attributeList,regInf);

    }// end of subscribeObjectClassAttributesWithRegion()

    /**
    *  Remove Federate's subscription to the Region in 
    *  Object Class' attributes.
    *
    * <ol>
    *  <li> Get RegionInfo from SpaceNaming
    *  <li> remove this federate to this region as a subscriber
    * </ol>
    */
    public void 
    unsubscribeObjectClassWithRegion(short theClass, RTIcap.Region theRegion, 
        short theHandle)
        throws RTIcap.ObjectClassNotDefined, RTIcap.RegionNotKnown, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        RegionInfo rgn = spaceNaming.getRegionInfo(theRegion);
        if( rgn == null ) throw new RTIcap.RegionNotKnown();
        
        rgn.deregister_subscriber(theHandle);

    }// end of unsubscribeObjectClassWithRegion()

    /**
    * 
    *  <ol>
    *   <li> Get the RegionInfo from SpaceNaming
    *   <li> Get INteractionRecord from InteractionNaming
    *   <li> Add Region to Interaction's subscribing regions.
    *   <li> Add federate to the Regions subscribing list.
    *  </ol>
    */
    public void 
    subscribeInteractionClassWithRegion(short theClass, 
        RTIcap.Region theRegion, RTIcap._Boolean active, short theHandle)
        throws RTIcap.InteractionClassNotDefined, RTIcap.RegionNotKnown, 
               RTIcap.FederateLoggingServiceCalls, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        InteractionRecord rec = interNaming.getInteractionRecord(theClass);
        if(rec==null) throw new RTIcap.InteractionClassNotDefined();
        
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        FederateInfo fedInf = fedEx.getFederateInfo(theHandle);
        if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

        rec.associateWithRegion(regInf,fedInf);

    }// end of subscribeInteractionClassWithRegion()

    /**
    *  <ol>
    *   <li> Get the RegionInfo from SpaceNaming
    *   <li> remove federate to the Regions subscribing list.
    *   <li> If there is no subscriber in the Region, then remove this
    *        region from InteractionClass' subscribing regions list.
    *  </ol>
    */
    public void 
    unsubscribeInteractionClassWithRegion(short theClass, 
        RTIcap.Region theRegion, short theHandle)
        throws RTIcap.InteractionClassNotDefined, 
               RTIcap.InteractionClassNotSubscribed, RTIcap.RegionNotKnown, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        InteractionRecord rec = interNaming.getInteractionRecord(theClass);
        if(rec==null) throw new RTIcap.InteractionClassNotDefined();
        
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        FederateInfo fedInf = fedEx.getFederateInfo(theHandle);
        if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

        rec.unassociateFromRegion(regInf,fedInf);

    }// end of unsubscribeInteractionClassWithRegion()

    /**
    *  <ol>
    *   <li> Get the InteractionRecord from InteractionNaming
    *   <li> sendInteractionWithRegionAndTime
    *  </ol>
    */
    public RTIcap.EventRetractionHandle 
    sendInteractionWithRegionAndTime(short theInteraction, 
        RTIcap.ParameterHandleValuePair[] parameters, double theTime, 
        String theTag, RTIcap.Region theRegion, short theHandle)
        throws RTIcap.InteractionClassNotDefined, 
               RTIcap.InteractionClassNotPublished, 
               RTIcap.InteractionParameterNotDefined, 
               RTIcap.InvalidFederationTime, RTIcap.RegionNotKnown, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        InteractionRecord rec = interNaming.getInteractionRecord(theInteraction);
        if(rec==null) throw new RTIcap.InteractionClassNotDefined();
        
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        FederateInfo fedInf = fedEx.getFederateInfo(theHandle);
        if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

        rec.publishParamsWithRegion(parameters,theTime,theTag,regInf,fedInf);

        RTIcap.EventRetractionHandle ret = null;

        return ret;

    }// end of sendInteractionWithRegionAndTime()

    /**
    *  <ol>
    *   <li> Get the InteractionRecord from InteractionNaming
    *   <li> sendInteractionWithRegion
    *  </ol>
    */
    public void 
    sendInteractionWithRegion(short theInteraction, 
        RTIcap.ParameterHandleValuePair[] parameters, String theTag, 
        RTIcap.Region theRegion, short theHandle)
        throws RTIcap.InteractionClassNotDefined, 
               RTIcap.InteractionClassNotPublished, 
               RTIcap.InteractionParameterNotDefined, RTIcap.RegionNotKnown, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        InteractionRecord rec = interNaming.getInteractionRecord(theInteraction);
        if(rec==null) throw new RTIcap.InteractionClassNotDefined();
        
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        FederateInfo fedInf = fedEx.getFederateInfo(theHandle);
        if(fedInf==null) throw new RTIcap.FederateNotExecutionMember();

        rec.publishParamsWithRegion(parameters,0.0,theTag,regInf,fedInf);

    }// end of sendInteractionWithRegion()

    /**
    * 
    * <ol>
    *  <li> Get ObjectRecord from ObjectNaming
    *  <li> requestClassAttributesUpdateWithRegion from ObjectRecord.
    * </ol>
    */
    public void 
    requestClassAttributeValueUpdateWithRegion(short theClass, 
        short[] theAttributes, RTIcap.Region theRegion, short theHandle)
        throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
               RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        ObjectRecord rec = objNaming.getObjectRecord(theClass);
        if(rec==null) throw new RTIcap.ObjectClassNotDefined();

        rec.requestUpdateAttributesInRegion(theAttributes,regInf);

    }// end of requestClassAttributeValueUpdateWithRegion()

    /**
    * <ol>
    *  <li> Get ObjectInstance from ObjectNaming
    *  <li> requestObjectAttributesUpdateWithRegion from ObjectInstance.
    * </ol>
    */
    public void 
    requestObjectAttributeValueUpdateWithRegion(int theObject, 
        short[] theAttributes, RTIcap.Region theRegion, short theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.RegionNotKnown, 
               RTIcap.AttributeNotDefined, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
        RegionInfo regInf = spaceNaming.getRegionInfo(theRegion);
        if(regInf==null) throw new RTIcap.RegionNotKnown();

        ObjectInstanceRecord rec = objNaming.getObjectInstance(theObject);
        if(rec==null) throw new RTIcap.ObjectNotKnown();

        rec.requestUpdateAttributesInRegion(theAttributes,regInf);

    }// end of requestObjectAttributeValueUpdateWithRegion()

}// end of DataDistributionManager

