package owrti;

import RTIcap.*;
import java.util.*;
import java.io.*;

/**
*
* @date April 20, 1997 ( Updated April 22, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class EvtQueueRO extends EvtQueue  implements RTIDebug
{

   Vector evts = new Vector();
   Hashtable idToEvtInfo = new Hashtable();
   protected RTIcap.FederateAmbassador fedAmb;
   protected RTIEventRepository que;
   protected FederateInfo fedInf;
   protected String FederateName ;
   long      numberOfEvents = 0;

   /**
   */
   public 
   EvtQueueRO( FederateInfo _fed, RTIEventRepository _que )
   {
      orderType = ReceiveOrder;
      fedAmb = _fed.getFederateAmbassador();
      fedInf = _fed;
      FederateName = fedInf.getFederateName();
      que = _que;

   }// end of CONSTRUCTOR

   /**
   * Executes  all the events in the EventQue
   */
   public synchronized int processEvents()
   {

      return(fedAmb!=null?processEvents(fedAmb):0);

   }// end of processEvents()

   /**
   * Executes  all the events in the EventQue
   */
   protected synchronized int processEvents(RTIcap.FederateAmbassador _fed)
   {

      int   num_of_exec = 0;
      EventInfo evtInf;
      RTIEvent evtInstance;
      Enumeration e ;

      synchronized( evts ) { e = evts.elements(); }

      if(!e.hasMoreElements() && DEBUG) 
        System.out.println("There is nothing to process for "+FederateName+"!");
      for(;e.hasMoreElements()&&num_of_exec<10;)
      {
          evtInf = (EventInfo) e.nextElement();
          if(evtInf!=null) 
          { 
            evtInstance = null;

            /** Find the RTIEvent object */
            if(que!=null) evtInstance = que.getRTIEvent(evtInf.getHandle());

            if(evtInstance!=null) 
            {
               log(" Execute ",evtInstance);
               /** Execute this event */
               evtInstance.execute(_fed);
               num_of_exec++;
 
               /** Delete this record from the Event List */
               synchronized( evts ) { evts.removeElement(evtInf); }

               /** Delete this record from the Event List */
               synchronized( idToEvtInfo )
               { idToEvtInfo.remove(new Integer(evtInf.getHandle())); }

               numberOfEvents--;
 
            }// end of if 

            evtInf = null;

          }// end of if

      }// end of for

      e = null;

      return num_of_exec;

   }// end of processEvents()

   /**
   */
   public synchronized void 
   releaseEvents()
   {
       synchronized( evts ) { evts.removeAllElements(); }
       synchronized( idToEvtInfo ) { idToEvtInfo.clear(); } 

   }// end of releaseEvents()

   /**
   */
   public synchronized void 
   addEventInfo(EventInfo evtInfo)
   {
      if(numberOfEvents<3000)
      {
         synchronized( evts ) { evts.addElement(evtInfo); }
         synchronized( idToEvtInfo ) 
         { idToEvtInfo.put(new Integer(evtInfo.getHandle()),evtInfo); }

         numberOfEvents++;
      }// end of if

   }// end of addEventInfo

   /**
   */
   public synchronized void
   cancelEvent(int evtID)
   {
      EventInfo evtInfo = null;
      synchronized( idToEvtInfo ) 
      { evtInfo = (EventInfo) idToEvtInfo.get(new Integer(evtID)); }

      if(evtInfo!=null)
      {
         synchronized( idToEvtInfo ) { idToEvtInfo.remove(new Integer(evtID)); }
         synchronized( evts ) { evts.removeElement(evtInfo); }
         numberOfEvents--;

      }// end of if

   }// end of cancelEvent()

   /**
   */
   final synchronized void
   log(String label, RTIEvent evt)
   {
      String logFileName = new String("/tmp/fed"+fedInf.getFederateID()+".log");
      try
      {
        PrintWriter pw = new PrintWriter(new FileWriter(logFileName,true),true);
        pw.println(label+" "+evt.toString());
        pw.close();

      } catch(java.io.IOException ioe) {}

   }// end of logEvent()

}// end of class EvtQueueRO

