package owrti;

import RTIcap.*;
import java.util.*;

import java.io.*;

/**
*
* @date April 20, 1998 ( Updated at April 27, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
EvtQueueTSO extends EvtQueue  implements RTIDebug
{

   EventInfoListItem head;

   protected RTIcap.FederateAmbassador fedAmb;
   protected RTIEventRepository que;
   protected FederateInfo fedInf;
   protected String FederateName ;

   /**
   */
   public 
   EvtQueueTSO( FederateInfo _fed, RTIEventRepository _que )
   {
      orderType = ReceiveOrder;
      fedAmb = _fed.getFederateAmbassador();
      fedInf = _fed;
      FederateName = fedInf.getFederateName();
      que = _que;

   }// end of CONSTRUCTOR

   /**
   * Executes  all the events in the EventQue
   */
   public synchronized int processEvents()
   {
      return(fedAmb!=null?processEvents(fedAmb):0);

   }// end of processEvents()

   /**
   * Executes  all the events in the EventQue
   */
   protected synchronized int 
   processEvents(RTIcap.FederateAmbassador _fed)
   {

      int   num_of_executed = 0;
      EventInfo evtInf;
      RTIEvent evtInstance ;
      EventInfoListItem curr = head;

      while(curr!=null)
      {
          evtInstance = null;
          if(que!=null)
             evtInstance = que.getRTIEvent(curr.getObject().getHandle()); 

          if(evtInstance!=null) { evtInstance.execute(_fed);num_of_executed++;}
          
          head = head.getSuccessor();
          curr = head;
 
      }// end of while

      return num_of_executed;

   }// end of processEvents()

   /**
   * Executes  all the events in the EventQue until time
   */
   public synchronized int processEvents(double time)
   {
      return(fedAmb!=null?processEvents(fedAmb,time):0);
   }// end of processEvents()

   /**
   * Executes  all the events in the EventQue
   */
   protected synchronized int 
   processEvents(RTIcap.FederateAmbassador _fed, double time)
   {

      int   num_of_events=0;
      EventInfo evtInf;
      RTIEvent evtInstance ;
      EventInfoListItem curr = head;
      boolean stay = true;

      while(curr!=null && stay)
      {
          evtInstance = null;
          if(que!=null)
             evtInstance = que.getRTIEvent(curr.getObject().getHandle()); 

          if(evtInstance!=null && curr.getObject().getTime()<=time) 
          {
             log("Execute ",evtInstance);
             evtInstance.execute(_fed);
             head = head.getSuccessor();
             curr = head;
             num_of_events++;
          } 
          else stay = false;
 
      }// end of while

      return num_of_events;

   }// end of processEvents()

   /**
   */
   public synchronized void
   releaseEvents()
   {
       head = null;

   }// end of releaseEvents()

   /**
   */
   public synchronized void 
   addEventInfo(EventInfo evtInfo)
   {
      EventInfoListItem curr;
      EventInfoListItem currP;
      EventInfoListItem newEle = new EventInfoListItem(evtInfo);
      boolean found=false;

      if(head==null) head = newEle;
      else
        {
           /********************************************************/
           /** Actually it might be better to search from the tail */
           /********************************************************/
           currP = null;
           curr = head;
           while(curr!=null && !found )
           {
              if(evtInfo.getTime()<curr.getObject().getTime()) found = true;
              else { currP = curr; curr = curr.getSuccessor(); } 
              
           }// end of while

           /** curr */
           if(curr!=null) curr.setPredecessor(newEle);

           newEle.setSuccessor(curr);
           newEle.setPredecessor(currP);

           if(currP!=null) currP.setSuccessor(newEle);
           else head = newEle;

        }

   }// end of addEventInfo

   /**
   */
   public synchronized EventInfo 
   peek() { return (head!=null?head.getObject():null); }

   /**
   */
   public synchronized void
   cancelEvent(int evtID)
   {

      EventInfo evtInf;
      RTIEvent evtInstance ;
      EventInfoListItem curr = head;
      EventInfoListItem currP = null;
      boolean found = false;

      while(curr!=null && !found)
      {
          evtInf = curr.getObject();
          if(evtInf.getHandle()==evtID) found=true;
          else { currP=curr; curr=curr.getSuccessor(); }
         
      }// end of while

      if(found)
      {
         if(currP==null) // head
         {
            head = curr.getSuccessor();
            head.setPredecessor(null); 
         }
         else
         {
            currP.setSuccessor(curr.getSuccessor());
            if(curr.getSuccessor()!=null)
               curr.getSuccessor().setPredecessor(currP);
         }

         curr = null;
 
      }// end of if

   }// end of cancelEvent() 

   final synchronized void
   log(String label, RTIEvent evt)
   {
      String logFileName = new String("/tmp/fed"+fedInf.getFederateID()+".log");
      try
      {
        PrintWriter pw = new PrintWriter(new FileWriter(logFileName,true),true);
        pw.println(label+" "+evt.toString());
        pw.close();

      } catch(java.io.IOException ioe) {}

   }// end of logEvent() 

}// end of class EvtQueueTSO

