/**
*/
package owrti;

import RTIcap.*;
import npacrti.*;

import java.io.*;

/**
*
* @date April 16, 1998 ( Updated at September 15, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
FederateInfo implements RTIDebug
{
   //FederateHandle  Federate;
   short  Federate;
   String FederateType;
   String FederateName;
   String FederateHost;
   String RTIversion;
   String FEDid;

   boolean TimeConstrained = false;
   boolean TimeRegulating  = false;
   boolean AsynchronousDelivery = false;

   npacrti.FederateStateType     currentState;
   npacrti.TimeManagerStateType  timeState;

   /*FederationTime   FederateTime;
   FederationTime   Lookahead;
   FederationTime   LBTS;
   FederationTime   MinNextEventTime;
   */ 

   double   FederateTime;
   double   Lookahead;
   double   LBTS;
   double   MinNextEventTime;
   int              ROlength;
   int              TSOlength;
   int              ReflectionsReceived;
   int              UpdatesSent;
   int              InteractionsReceived;
   int              InteractionsSent;
   int              ObjectsOwned;
   int              ObjectsUpdated;
   int              ObjectsReflected;

   RTIcap.FederateAmbassador federateAmbassador;
   RTIEventRepository        evtRepository;
   EvtQueueRO                ROevtQue;
   EvtQueueTSO               TSOevtQue;
   FederationExecution       fedEx;
   int TimeMngPhase = TimeManager.TIME_GRANTED;
   String logFileName = new String("/tmp/unknown.log");
   boolean  ticking = false;
   boolean  destroy_in_progress = false;

   /**
   */
   public 
   FederateInfo( short _fedHandle, String _fedName, 
                 RTIcap.FederateAmbassador _federateAmbassador,
                 RTIEventRepository  _evtRepo, FederationExecution _fedEx)
   {
      federateAmbassador = _federateAmbassador;
      Federate = _fedHandle;
      FederateName = _fedName;
      evtRepository = _evtRepo;
      ROevtQue = new EvtQueueRO(this,evtRepository);
      TSOevtQue = new EvtQueueTSO(this,evtRepository);
      fedEx = _fedEx;

      logFileName = new String("/tmp/fed"+_fedHandle+".log");

      try
      {
        PrintWriter pw = new PrintWriter(new FileWriter(logFileName),true);
        pw.close();
      } catch(java.io.IOException ioe) {}

   }// end of constructor

   /**
   */
   public  String
   getFederateName() { return FederateName; }

   /**
   */
   public  short
   getFederateID() { return Federate; }

   /**
   */
   public  FederationExecution
   getFederationExecution() { return fedEx; }

   /**
   */
   public  RTIcap.FederateAmbassador
   getFederateAmbassador() { return federateAmbassador; }

   /**
   */
   public  double
   getFederateTime() { return FederateTime; }

   /**
   */
   public double
   getFederateLookahead() { return Lookahead; }

   /**
   */
   public double
   getFederateLBTS() { return LBTS; }

   /**
   */
   public RTIEventRepository
   getGlobalEvtRepository() { return evtRepository; }

   /**
   */
   public void
   setTimeAdvancing() throws RTIcap.TimeAdvanceAlreadyInProgress 
   { 
      if(TimeMngPhase!=TimeManager.TIME_ADVANCING) 
           TimeMngPhase = TimeManager.TIME_ADVANCING;
      else throw new RTIcap.TimeAdvanceAlreadyInProgress();

   }// end of setTimeAdvancing()

   /**
   */
   public void
   setTimeGranted() { TimeMngPhase = TimeManager.TIME_GRANTED;}

   /**
   */
   public int
   getTimeManagementState() { return TimeMngPhase; }

   /**
   *  Tries to set the current time value for the federate and its lookahead.
   *  This time value might not be realistic. So that we check the
   *  Federation's time value first and assign the value according to this
   *  comparison.
   */
   public void
   setTimeRegulation(double t, double l) 
   { 

      if(destroy_in_progress) return ;
      double tm = Math.min(t,fedEx.calculateLBTS(Federate,this,t));

      if(DEBUG_TIME) System.out.println(" setTimeRegulation : "+tm);

      int handle = evtRepository.addEvent( new EvtTimeRegulationEnabled(tm));
      EventInfo evtInfo = new EventInfo(handle,EventInfo.GLOBAL);
      addEventInfo(evtInfo);

      if(DEBUG_TIME) System.out.println(" after federateAmbassador.timeRegulationEnabled "+tm);
      TimeRegulating=true; 
      FederateTime = tm;
      setLookahead(l);

   }// end of setTimeRegulation()

   /**
   */
   public void
   resetTimeRegulation() 
   { TimeRegulating=false; 
   }

   /**
   */
   public void
   setTimeConstrained() 
   { 
      if(destroy_in_progress) return ;
      double tm = fedEx.getFederationTime();

      int handle = evtRepository.addEvent( new EvtTimeConstrainedEnabled(tm));
      EventInfo evtInfo = new EventInfo(handle,EventInfo.GLOBAL);
      addEventInfo(evtInfo);

      TimeConstrained=true;
      FederateTime = tm;

   }// end of setTimeConstrained()

   /**
   */
   public void
   resetTimeConstrained() 
   { TimeConstrained=false;
   }

   /**
   *  If federate is regulating, federate should produce TSO's bigger or
   *  equal to T+L except that L==0. If L==0, then federate should produce TSO
   *  BIGGER THAN T.
   */
   public void
   setLookahead(double _val) 
   { 
      if(destroy_in_progress) return ;
      Lookahead = Math.max(_val,0f);
   }// end of setLookahead()

   /**
   */
   public void
   setAsynchronousDelivery() 
   { 
      if(destroy_in_progress) return ;
      AsynchronousDelivery = true; 
      try
      {
         if(ROevtQue!=null) ROevtQue.processEvents();
      } catch(org.omg.CORBA.SystemException se)
        {
        }

   }// end of setAsynchronousDelivery()

   /**
   */
   public void
   resetAsynchronousDelivery() 
   { 
      if(destroy_in_progress) return ;
      AsynchronousDelivery = false; 

      try
      {
      if( TimeMngPhase == TimeManager.TIME_ADVANCING && TimeConstrained && 
          ROevtQue!=null ) 
         ROevtQue.processEvents();
      } catch(org.omg.CORBA.SystemException se)
        {
        }

   }// end of resetAsynchronousDelivery()

   /**
   */
   synchronized RTIcap._Boolean
   tick()
   {
      if(destroy_in_progress) return RTIcap._Boolean.RTI_TRUE;
      RTIcap._Boolean ret = RTIcap._Boolean.RTI_FALSE; 

      logMessage(" in tick() ");
      ticking=true;

      try
      {
         // ProcessEvents in the RO Queue
         if(ROevtQue.processEvents()>0) ret = RTIcap._Boolean.RTI_TRUE;

      } catch(org.omg.CORBA.SystemException se)
        { }

      ticking=false;

      if(DEBUG_TICK) System.out.println(" out of tick() of FederateInfo "); 
      logMessage(" out of tick() ");

      return ret;
      
   }// end of tick()

   /**
   *  
   *  <ul>
   *  <p><li> release all the messages in the RO queue.
   * 
   *  <p><li> If federate is constrained, then release all the messages which 
   *          does not violates the LBTS of the federation.
   *
   *  <p><li> advance current time of the federate according to LBTS and the asked
   *  time(theTime).
   *  </ul>
   */
   public void
   advanceTime( double theTime )
      throws RTIcap.TimeAdvanceAlreadyInProgress, 
             RTIcap.FederationTimeAlreadyPassed,
             RTIcap.InvalidFederationTime 

   {
      if(destroy_in_progress) return ;
      if(DEBUG_TIME) System.out.println(" in advanceTime() of FederateInfo for "+FederateName+" to : "+theTime);

      /** Change the state in Time Manager Point of View */
      setTimeAdvancing();

      try
      {
      if(ROevtQue!=null) ROevtQue.processEvents();

      if(DEBUG_TIME) System.out.println(" after ROevtQue.processEvents() of FederateInfo for "+FederateName);

      /********************************************************************/
      /** If it is not Time Constrained them let her advance to the point */
      /** she wants.                                                      */
      /********************************************************************/
      if( TimeConstrained )
      {
         if(DEBUG_TIME) System.out.println(FederateName+" is TimeConstrained Federate");
         /** Ask the LBTS of the Federation */
         LBTS = fedEx.calculateLBTS(Federate,this,theTime);

         if(LBTS==-1) LBTS = theTime;

         /** Deliver Events */
         if(TSOevtQue!=null) TSOevtQue.processEvents(Math.min(LBTS,theTime));
         else TSOevtQue = new EvtQueueTSO(this,evtRepository); /** weird */

         /** Beyond LBTS is not allowed */
         releaseTimeGrant( Math.min(LBTS,theTime) );

      } else releaseTimeGrant( theTime );

      } catch(org.omg.CORBA.SystemException se)
        {}

      /** Change the state in Time Manager Point of View */
      setTimeGranted();

   }// end of advanceTime()

   /**
   *  <ul>
   *  <p><li> release all the messages in the RO queue.
   * 
   *  <p><li> Find the minimum time stamped event in the TSO
   *      <ul>
   *      <p><li> If event does not exist, FederateTime = min(LBTS,theTime)
   *      <p><li> If event exist, FederateTime = min(evtTime,LBTS)
   *      </ul>
   *
   *  </ul>
   */
   public void
   advanceTimeToNextEvent( double theTime )
      throws RTIcap.TimeAdvanceAlreadyInProgress, 
             RTIcap.FederationTimeAlreadyPassed,
             RTIcap.InvalidFederationTime 

   {
      if(destroy_in_progress) return ;
      if(DEBUG_T) System.out.println(" in advanceTime() of FederateInfo for "+FederateName);

      /** Change the state in Time Manager Point of View */
      setTimeAdvancing();

      try
      {

      if(ROevtQue!=null) ROevtQue.processEvents();

      if(DEBUG_T) System.out.println(" after ROevtQue.processEvents() of FederateInfo for "+FederateName);

      if( TimeConstrained )
      {
         /** Ask the LBTS of the Federation */
         LBTS = fedEx.calculateLBTS(Federate,this,theTime);

         if( TSOevtQue != null) 
         { 
            /** Find the minimum Time Stamped event in the Queue */
            EventInfo evtInfo = TSOevtQue.peek();
 
            /** Compare with the requested time value */
            if(evtInfo!=null && evtInfo.getTime()<theTime) 
            {   
               /** Process events to this point in time */
               TSOevtQue.processEvents(evtInfo.getTime());

               /** Update Federate Time */
               releaseTimeGrant( Math.min(evtInfo.getTime(),LBTS) );

            } else releaseTimeGrant( Math.min(theTime,LBTS) );

         } 
         else 
          {  
             TSOevtQue = new EvtQueueTSO(this,evtRepository); /** weird */
             releaseTimeGrant( Math.min(theTime,LBTS) );
          }

      } else releaseTimeGrant( theTime );

      } catch(org.omg.CORBA.SystemException se)
        {}

      /** Change the state in Time Manager Point of View */
      setTimeGranted();

   }// end of advanceTimeToNextEvent()

   /**
   *  It is time to flush the RO queue.
   */
   public void
   advanceTimeTo( double theTime )
      throws RTIcap.TimeAdvanceAlreadyInProgress, 
             RTIcap.FederationTimeAlreadyPassed,
             RTIcap.InvalidFederationTime 

   {
      if(destroy_in_progress) return ;
      if(DEBUG_T) System.out.println(" in advanceTime() of FederateInfo for "+FederateName);

      /** Change the state in Time Manager Point of View */
      setTimeAdvancing();

      try
      {
      if(ROevtQue!=null) ROevtQue.processEvents();

      if(DEBUG_T) System.out.println(" after ROevtQue.processEvents() of FederateInfo for "+FederateName);

      /********************************************************************/
      /** If there is no delivered messages, then we can advance the time */
      /********************************************************************/
      if( TSOevtQue!=null )
      { 
          if(TSOevtQue.processEvents(theTime)==0 ) releaseTimeGrant(theTime) ;
      }
      else 
       {  
          TSOevtQue = new EvtQueueTSO(this,evtRepository); /** weird */
          releaseTimeGrant(theTime) ;
       }

      } catch(org.omg.CORBA.SystemException se)
        {}

      /** Change the state in Time Manager Point of View */
      setTimeGranted();

   }// end of advanceTimeTo()

   /**
   */
   final void
   releaseTimeGrant(double theTime)
      throws RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime 
   {
      if(destroy_in_progress) return ;
      try
      {
         if(DEBUG_T) System.out.println(" Before federateAmbassador.timeAdvanceGrant() ");
         federateAmbassador.timeAdvanceGrant(theTime);
         FederateTime = theTime;
         if(DEBUG_T) System.out.println(" After federateAmbassador.timeAdvanceGrant() FederateTime : "+FederateTime);
      } catch(RTIcap.FederateInternalError fie) {}
        catch(RTIcap.TimeAdvanceWasNotInProgress tawnip) {}

   }// end of releaseTimeGrant()

   /**
   *
   */
   public void
   addRTIEvent(RTIEvent evtRTI)
   { 
      if(destroy_in_progress) return ;
      EventInfo evtInfo = 
                new EventInfo(evtRepository.addEvent(evtRTI),EventInfo.LOCAL);
      if(ROevtQue!=null) 
      {
         ROevtQue.addEventInfo(evtInfo);
         logEvent(" Add to RO ",evtInfo);
      }
      else
        {
           ROevtQue = new EvtQueueRO(this,evtRepository); /** weird */
           ROevtQue.addEventInfo(evtInfo);
           logEvent(" Add to RO ",evtInfo);
        }

      //if(AsynchronousDelivery) ROevtQue.processEvents();

   }// end of addRTIEvent()

   /**
   *
   */
   public void
   addEventInfo(EventInfo evtInfo)
   { 
      
      if(destroy_in_progress) return ;
      if(ROevtQue!=null) 
      {  
         ROevtQue.addEventInfo(evtInfo);
         logEvent(" Add to RO ",evtInfo);
      }
      else
        {
           ROevtQue = new EvtQueueRO(this,evtRepository); /** weird */
           ROevtQue.addEventInfo(evtInfo);
           logEvent(" Add to RO ",evtInfo);
        }

      //if(AsynchronousDelivery) ROevtQue.processEvents();

   }// end of addEventInfo()

   /**
   *  For TimeConstrained Federates, these messages should be stored in
   *  TSO.
   */
   public void
   addRTIEvent(RTIEvent evtRTI, double receiveTSO)
   { 
      
      if(destroy_in_progress) return ;
      //if(AsynchronousDelivery ==false )
      EventInfo evtInfo = 
                new EventInfo(evtRepository.addEvent(evtRTI),EventInfo.LOCAL,receiveTSO);
      if(!TimeConstrained) addEventInfo(evtInfo);
      else if(TSOevtQue!=null) 
      {  TSOevtQue.addEventInfo(evtInfo);
         logEvent(" Add to TSO ",evtInfo);
      }
      else
        {
           TSOevtQue = new EvtQueueTSO(this,evtRepository); /** weird */
           TSOevtQue.addEventInfo(evtInfo);
           logEvent(" Add to TSO ",evtInfo);
        }

   }// end of addEventInfo()

   /**
   *  For TimeConstrained Federates, these messages should be stored in
   *  TSO.
   */
   public void
   addEventInfo(EventInfo evtInfo, double receiveTSO)
   { 
      
      if(destroy_in_progress) return ;
      //if(AsynchronousDelivery ==false )
      if(!TimeConstrained) addEventInfo(evtInfo);
      else if(TSOevtQue!=null) 
      {  TSOevtQue.addEventInfo(evtInfo);
         logEvent(" Add to TSO ",evtInfo);
      }
      else
        {
           TSOevtQue = new EvtQueueTSO(this,evtRepository); /** weird */
           TSOevtQue.addEventInfo(evtInfo);
           logEvent(" Add to TSO ",evtInfo);
        }

   }// end of addEventInfo()

   /**
   */
   public double 
   minimumTSOOfNextEvent()
   {
      if(destroy_in_progress) return LBTS;
      double ret = LBTS;
      
      if(TSOevtQue!=null)
      {
         EventInfo inf = TSOevtQue.peek();
         if(inf!=null && inf.getTime()<ret) ret = inf.getTime();

      }// end of if

      return ret;

   }// end of minimumTSOOfNextEvent()

   /**
   */
   public void
   cancelEvent(int evtID)
   {
      if(destroy_in_progress) return;
      if(ROevtQue!=null) ROevtQue.cancelEvent(evtID);
      if(TSOevtQue!=null) TSOevtQue.cancelEvent(evtID);

   }// end of cancelEvent()

   /**
   */
   final void
   logEvent(String label, EventInfo evt)
   {
      try
      {
        PrintWriter pw = new PrintWriter(new FileWriter(logFileName,true),true);
        pw.println(label+" "+evt.toString());
        pw.close();

      } catch(java.io.IOException ioe) {}

   }// end of logEvent

   /**
   */
   final void
   logMessage(String label)
   {
      try
      {
        PrintWriter pw = new PrintWriter(new FileWriter(logFileName,true),true);
        pw.println(label);
        pw.close();

      } catch(java.io.IOException ioe) {}

   }// end of logMessage

   /**
   * In destroy, Federate will clear out its queues.
   * It will remove all the object instances that it simulates.
   * or release their ownership.
   * It should not be in TICK or TIME_ANVANCE phase.
   */
   public synchronized void
   destroy()
   {
      destroy_in_progress = true;
      if(DEBUG_TICK) System.out.println(" in destroy() of FederateInfo "); 
      // wait until tick is over.
      while(ticking) ;
      while(TimeMngPhase != TimeManager.TIME_GRANTED);

      // Release events in the queue 
      if(ROevtQue!=null) ROevtQue.releaseEvents();
      if(TSOevtQue!=null) TSOevtQue.releaseEvents();
      if(DEBUG_TICK) System.out.println(" out of destroy() of FederateInfo "); 

   }// end of destroy()

}// end of FederateInfo

