/**
*/
package owrti;

import RTIcap.*;
import npacrti.*;

import java.util.*;
import java.net.*;
import com.docuverse.freedom.*;
import org.w3c.dom.*;
import owrti.xml.*;

/**
* It needs to know who is on this Federation Execution (federates)
* Objects which belongs to this Execution.
* name of the FED ( Federation Execution Data) file
*  
* @date April 29, 1998 (Updated at August 11, 1998)
* @author H. Timucin Ozdemir
*  
*/
public class 
FederationExecution extends _FederationExecutionInterfaceImplBase 
                    implements RTIDebug
{
   DeclarationManager      decManager ;
   ObjectManager           objManager ;
   OwnershipManager        ownManager ;
   TimeManager             timManager ;
   DataDistributionManager datManager ;
   RTIServices             rtiServices ;
   RTIKernel               rtiKernel ;

   int numberOfFederates = 0;

   public static final int RUNNING         = 0;
   public static final int SAVE_PENDING    = 1;
   public static final int SAVING          = 2;
   public static final int RESTORE_PENDING = 3;
   public static final int RESTORING       = 4;

   int       currentState;
   String    FEDFileName;

   Hashtable federates = new Hashtable();
   Hashtable idToName  = new Hashtable();
   Hashtable idToFedInfo  = new Hashtable();
   Hashtable nameToId  = new Hashtable();

   Hashtable regulatingFederates   = new Hashtable();
   Hashtable constrainedFederates  = new Hashtable();

   ObjectClassNaming objNaming;
   InteractionNaming interNaming;
   SpaceNaming       spaceNaming;
   RegionNaming      regionNaming;

   String    FederationName;
   Hashtable FederatesInFederation = new Hashtable();
   String    RTIversion;
   String    FEDid;
   String    LastSaveName;
   double    LastSaveTime;
   String    NextSaveName;
   double    NextSaveTime; 

   double    FederationTime;
   double    FederationLBTS;

   short     numberOfObjectClasses = 0;
   short     numberOfInteractionClasses = 0;

   RTIEventRepository evtRepository = new RTIEventRepository();

   /**
   * FederationExecution Creates its own managers.
   */
   public 
   FederationExecution( String fedDataName, RTIKernel _rtiKernel )
   {
       rtiKernel    = _rtiKernel;
       interNaming  = new InteractionNaming(); 
       spaceNaming  = new SpaceNaming(); 
       regionNaming = new RegionNaming(); 
       objNaming  = new ObjectClassNaming(interNaming); 
       decManager = new DeclarationManager(objNaming,interNaming,this);
       objManager = new ObjectManager(objNaming,interNaming,this);
       ownManager = new OwnershipManager(this,objNaming);
       timManager = new TimeManager(this);
       datManager = new DataDistributionManager(this,spaceNaming,interNaming,objNaming);
       rtiServices = new RTIServices(objNaming,interNaming);

       /*
       FEDFileName = fedDataName+".fed";
       if(DEBUG) System.out.println(" FED NAME :"+FEDFileName);
       if(FEDFileName.equals("HelloWorld.fed")) defineHelloWorld();  
       if(FEDFileName.equals("Jager.fed")) defineJager();  
       //defineHelloWorld();  
       */
       FEDFileName = "config/"+fedDataName+".xml";
       if(DEBUG) System.out.println(" FED NAME :"+FEDFileName);
       defineFederationClassHierarchy(FEDFileName);

   }// end of CONSTRUCTOR

   /**
   */
   void
   defineFederationClassHierarchy(String fileName)
   {
      try
      {
        Freedom dom = new Freedom("com.docuverse.freedom.SAXDriver");
        dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");

        Document doc = dom.openDocument(DIFDocument.makeAbsoluteURL(fileName));

        DIFDocument dif = new DIFDocument(doc);
      
        /** Define each object */
        Enumeration e3;
        Enumeration e2;
        Enumeration e = dif.getObjectClasses();
        ObjClassDef current;
        AttributeDef myAttr;
        Stack       myStack = new Stack();

        if(e!=null && e.hasMoreElements())
        {
          for(;e.hasMoreElements();)
          {
            current = (ObjClassDef) e.nextElement();
            myStack.push(current);
            while(!myStack.empty())
            {
               current = (ObjClassDef) myStack.pop();

               if(current!=null) 
               {  //o.println(current.getFullName());
                  ObjectRecord rec = new ObjectRecord(this);

                  if(DEBUG1) 
                  { 
                     System.out.println(" objName : "+current.getFullName());
                     System.out.println(" obj  Id : "+numberOfObjectClasses);
                  }

                  rec.setName(current.getFullName());
                  rec.setId(numberOfObjectClasses++);
                  e3 = current.getAttributes();

                  if(e3!=null && e3.hasMoreElements())
                    for(;e3.hasMoreElements();) 
                    { 
                      myAttr = (AttributeDef) e3.nextElement();
                      if(myAttr!=null) 
                      {
                         rec.addAttribute(myAttr.getName());
                         if(DEBUG1) 
                           System.out.println(" attName : "+myAttr.getName());
                      }
                    }

                  objNaming.addObjectRecord(rec);
                  rec.dump(System.out);

                  e2 = current.getChildren();
                  if(e2!=null && e2.hasMoreElements())
                     for(;e2.hasMoreElements();) myStack.push(e2.nextElement());

               }// end of if

            }// end of while

          }// end of for

        }// end of if

        /** Define each interaction class */
        e = dif.getInteractionClasses();
        InteractionDef icurrent;
        myStack = new Stack();
        String par_name;

        if(e!=null && e.hasMoreElements())
        {
          for(;e.hasMoreElements();)
          {
            icurrent = (InteractionDef) e.nextElement();
            myStack.push(icurrent);
            while(!myStack.empty())
            {
               icurrent = (InteractionDef) myStack.pop();

               if(icurrent!=null) 
               {  
                  InteractionRecord irec = new InteractionRecord();
                  irec.setName(icurrent.getFullName());
                  irec.setId(numberOfInteractionClasses++);
                  e3 = icurrent.getParameters();

                  if(e3!=null && e3.hasMoreElements())
                    for(;e3.hasMoreElements();) 
                    { 
                      par_name = (String) e3.nextElement();
                      if(par_name!=null) irec.addParameter(par_name);
                    }

                  interNaming.addInteractionRecord(irec);

                  irec.dump(System.out);

                  e2 = icurrent.getChildren();
                   if(e2!=null && e2.hasMoreElements())
                      for(;e2.hasMoreElements();) myStack.push(e2.nextElement());

               }// end of if

            }// end of while

          }// end of for

        }// end of if

      } catch(MalformedURLException mfue) {}

   }// end of defineFederationClassHierarchy() 

   /**
   void
   defineCommons()
   {
      if(DEBUG) System.out.println(" defineCommons() ");

      ObjectRecord rec = new ObjectRecord(this);
      if(DEBUG) System.out.println(" after new ObjectRecord() ");
      rec.setName("objectRoot.Manager.Federate");
      if(DEBUG) System.out.println(" after setName() ");
      rec.setId((short)1);
      if(DEBUG) System.out.println(" after setId() ");

      rec.addAttribute(new String("FederateHandle"));
      rec.addAttribute(new String("FederateType"));
      rec.addAttribute(new String("FederateHost"));
      rec.addAttribute(new String("FederateName"));
      rec.addAttribute(new String("RTIversion"));
      rec.addAttribute(new String("FEDid"));
      rec.addAttribute(new String("TimeConstrained"));
      rec.addAttribute(new String("TimeRegulating"));
      rec.addAttribute(new String("AsynchronousDelivery"));
      rec.addAttribute(new String("FederateState"));
      rec.addAttribute(new String("TimeManagerState"));
      rec.addAttribute(new String("FederateTime"));
      rec.addAttribute(new String("Lookahead"));
      rec.addAttribute(new String("LBTS"));
      rec.addAttribute(new String("MinNextEventTime"));
      rec.addAttribute(new String("ROlength"));
      rec.addAttribute(new String("TSOlength"));
      rec.addAttribute(new String("ReflectionsReceived"));
      rec.addAttribute(new String("UpdatesSent"));
      rec.addAttribute(new String("InteractionsReceived"));
      rec.addAttribute(new String("InteractionsSent"));
      rec.addAttribute(new String("ObjectsOwned"));
      rec.addAttribute(new String("ObjectsUpdated"));
      rec.addAttribute(new String("ObjectsReflected"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      objNaming.addObjectRecord(rec);
      if(DEBUG) System.out.println(" after addObjectRecord() ");

      rec.dump(System.out);

      rec = new ObjectRecord(this);
      if(DEBUG) System.out.println(" after new ObjectRecord() ");
      rec.setName("objectRoot.Manager.Federation");
      if(DEBUG) System.out.println(" after setName() ");
      rec.setId((short)2);
      if(DEBUG) System.out.println(" after setId() ");

      rec.addAttribute(new String("FederationName"));
      rec.addAttribute(new String("FederatesInFederation"));
      rec.addAttribute(new String("RTIversion"));
      rec.addAttribute(new String("FEDid"));
      rec.addAttribute(new String("LastSaveName"));
      rec.addAttribute(new String("LastSaveTime"));
      rec.addAttribute(new String("NextSaveName"));
      rec.addAttribute(new String("NextSaveTime"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      objNaming.addObjectRecord(rec);
      if(DEBUG) System.out.println(" after addObjectRecord() ");

      rec.dump(System.out);

   }// end of defineHelloWorld() 
   */

   /**
   void
   defineHelloWorld()
   {
      defineCommons();
      if(DEBUG) System.out.println(" defineHelloWorld() ");
      ObjectRecord rec = new ObjectRecord(this);
      if(DEBUG) System.out.println(" after new ObjectRecord() ");
      rec.setName("objectRoot.Country");
      if(DEBUG) System.out.println(" after setName() ");
      rec.setId((short)10);
      if(DEBUG) System.out.println(" after setId() ");

      rec.addAttribute(new String("Name"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      rec.addAttribute(new String("Population"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      objNaming.addObjectRecord(rec);
      if(DEBUG) System.out.println(" after addObjectRecord() ");

      rec.dump(System.out);

      InteractionRecord irec = new InteractionRecord();
      irec.setName("interactionRoot.Communication");
      if(DEBUG) System.out.println(" after setName() ");
      irec.setId((short)1);
      if(DEBUG) System.out.println(" after setId() ");

      irec.addParameter(new String("Message"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      interNaming.addInteractionRecord(irec);
      if(DEBUG) System.out.println(" after addObjectRecord() ");

      irec.dump(System.out);

      if(DEBUG) System.out.println(" end of defineHelloWorld() ");

   }// end of defineHelloWorld() 
   */

   /**
   void
   defineJager()
   {
      defineCommons();
      if(DEBUG) System.out.println(" defineJager() ");
      ObjectRecord rec = new ObjectRecord(this);
      if(DEBUG) System.out.println(" after new ObjectRecord() ");
      rec.setName("objectRoot.Model");
      if(DEBUG) System.out.println(" after setName() ");
      rec.setId((short)10);
      if(DEBUG) System.out.println(" after setId() ");

      rec.addAttribute(new String("Durability"));
      rec.addAttribute(new String("Damage"));
      rec.addAttribute(new String("Location"));
      rec.addAttribute(new String("Mass"));
      rec.addAttribute(new String("Orientation"));
      rec.addAttribute(new String("Shape"));
      rec.addAttribute(new String("Velocity"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      objNaming.addObjectRecord(rec);
      if(DEBUG) System.out.println(" after addObjectRecord() ");

      rec.dump(System.out);

      InteractionRecord irec = new InteractionRecord();
      irec.setName("interactionRoot.Communication");
      if(DEBUG) System.out.println(" after setName() ");
      irec.setId((short)11);
      if(DEBUG) System.out.println(" after setId() ");

      irec.addParameter(new String("Message"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      interNaming.addInteractionRecord(irec);
      if(DEBUG) System.out.println(" after addInteractionRecord() ");

      irec.dump(System.out);

      irec = new InteractionRecord();
      irec.setName("interactionRoot.Collision");
      if(DEBUG) System.out.println(" after setName() ");
      irec.setId((short)12);
      if(DEBUG) System.out.println(" after setId() ");

      irec.addParameter(new String("RemoteObjectId"));
      irec.addParameter(new String("DeltaVelocity"));
      irec.addParameter(new String("DeltaDamage"));
      if(DEBUG) System.out.println(" after addAttribute() ");

      interNaming.addInteractionRecord(irec);
      if(DEBUG) System.out.println(" after addInteractionRecord() ");

      irec.dump(System.out);

      if(DEBUG) System.out.println(" end of defineJager() ");

   }// end of defineJager() 
   */

   /**
   */
   public synchronized boolean
   destroyExecution()
   { return (idToName.size()==0); }// end of destroyExecution()

   /**
   * returns true  : if the given federate name is already exist.
   *         false : if the given federate name is not exist.
   */
   public synchronized boolean
   hasMember(String yourName)
   {
      boolean ret = true;
      RTIcap.FederateAmbassador fedAmb = 
                  (RTIcap.FederateAmbassador) federates.get(yourName); 

      return (fedAmb!=null);

   }// end of hasMember()

   /**
   */
   public synchronized int
   getNumberOfFederates()
   {
      return numberOfFederates;
   }

   /**
   */
   public synchronized FederateInfo
   getFederateInfo(short theFederate)
   {
      return (FederateInfo) idToFedInfo.get(new Short(theFederate));
   }

   /**
   */
   public synchronized Enumeration
   getAllFederateInfos()
   { return idToFedInfo.elements(); }

   /**
   */
   public synchronized int
   getCurrentState() { return currentState; }

   /**
   */
   public synchronized double
   getFederationTime() { return FederationTime; }

   /***********************************/
   /** Federation Management Services */
   /***********************************/

   /**
   * 4.2
   */
   public synchronized void 
   createFederationExecution(String executionName, String FED )
         throws RTIcap.FederationExecutionAlreadyExists, RTIcap.RTIinternalError
   {
      if(executionName.equals(FederationName)) 
         throw new RTIcap.FederationExecutionAlreadyExists(); 

      if(rtiKernel!=null) 
         rtiKernel.createFederationExecution(executionName,FED);
      else
         throw new RTIcap.RTIinternalError(); 

   }// end of createFederationExecution()

   /**
   * 4.3
   */
   public synchronized void 
   destroyFederationExecution(String executionName, short theFederate)
        throws RTIcap.FederatesCurrentlyJoined, 
                RTIcap.FederationExecutionDoesNotExist, RTIcap.RTIinternalError 
   {
      if(rtiKernel==null) 
         throw new RTIcap.RTIinternalError(); 
      else if(destroyExecution()) 
              rtiKernel.destroyFederationExecution(executionName); 
        
   }// end of destroyFederationExecution() 

   /** 
   * Federate joins the execution through this interface
   */
   public synchronized short
   joinFederationExecution(String yourName, String executionName, 
                           RTIcap.FederateAmbassador federateAmbassador)
         throws RTIcap.FederateAlreadyExecutionMember,
                RTIcap.FederationExecutionDoesNotExist, RTIcap.CouldNotOpenFED,
                RTIcap.ErrorReadingFED,  RTIcap.RTIinternalError

   {
      
      short ret ;
      Short fedId = (Short) nameToId.get(yourName);
      
      if(fedId==null)
      {
         ret = (short) (++numberOfFederates);
      

         federates.put(yourName,federateAmbassador);
         idToName.put(new Short(ret),yourName);
         nameToId.put(yourName,new Short(ret));
         idToFedInfo.put(new Short(ret),new FederateInfo(ret,yourName,federateAmbassador,evtRepository,this));
         if(DEBUG) System.out.println(" id of "+yourName+" is "+ret);

      } else ret = fedId.shortValue();

      return ret;

   }// end of joinFederationExecution()

   /**  
   * Should we delete all the Object Instances defined by this Federate?
   * We should cancel all the subscriptions made by this federate.
   */
   public synchronized void 
   resignFederationExecution( RTIcap.ResignAction theAction, 
         String FederationName, short theFederate)
         throws RTIcap.FederateOwnsAttributes, RTIcap.InvalidResignAction, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
   {
      if(DEBUG) 
         System.out.println(theFederate+" wants to leave "+FEDFileName);

      String name = (String) idToName.get(new Short(theFederate));

      if(DEBUG&&name!=null) 
         System.out.println(name+" wants to leave "+FEDFileName);

      if(name!=null)
      {
         Short fID = new Short(theFederate);

         removeTimeConstrainedList( theFederate );
         removeTimeRegulatingList( theFederate );

         FederateInfo fdInfo = (FederateInfo) idToFedInfo.get(fID);
         if(fdInfo!=null) fdInfo.destroy();

         federates.remove(name);
         nameToId.remove(name);
         idToName.remove(fID); 
         idToFedInfo.remove(fID); 

         if(DEBUG) System.out.println(name+" is removed from"+FEDFileName);

      } else 
        { 
           if(DEBUG) 
              System.out.println(" FederateDoesNotExist in "+FEDFileName);
           
           throw new RTIcap.FederateNotExecutionMember();
        }
      
   }// end of resignFederationExecution()

    /**
    */
    public void 
    registerFederationSynchronizationPoint(String label, String tag, 
        short theFederate)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of registerFederationSynchronizationPoint()

    /**
    */
    public void 
    registerFederationSynchronizationPointWithSet(String label, String tag, 
        short[] syncSet, short theFederate)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of registerFederationSynchronizationPointWithSet()

    /**
    */
    public void 
    synchronizationPointAchieved(String label, short theFederate)
        throws RTIcap.SynchronizationPointLabelWasNotAnounced, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of synchronizationPointAchieved()

    /**
    */
    public void 
    requestFederationSaveWithTime(String label, double theTime, 
        short theFederate)
        throws RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime,
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of requestFederationSaveWithTime()

    /**
    */
    public void 
    requestFederationSave(String label, short theFederate)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of requestFederationSave()

    /**
    */
    public void 
    federateSaveBegun(short theFederate)
        throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of federateSaveBegun()

    /**
    */
    public void 
    federateSaveComplete(short theFederate)
        throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of federateSaveComplete()

    /**
    */
    public void 
    federateSaveNotComplete(short theFederate)
        throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of federateSaveNotComplete()

    /**
    */
    public void 
    requestFederationRestore(String label, short theFederate)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of requestFederationRestore()

    /**
    */
    public void 
    federateRestoreComplete(short theFederate)
        throws RTIcap.RestoreNotRequested, RTIcap.FederateNotExecutionMember, 
               RTIcap.RTIinternalError
    {
    }// end of federateRestoreComplete()

    /**
    */
    public void 
    federateRestoreNotComplete(short theFederate)
        throws RTIcap.RestoreNotRequested, RTIcap.FederateNotExecutionMember, 
               RTIcap.RTIinternalError
    {
    }// end of federateRestoreNotComplete()

    /**
    */
    public synchronized FederationManagerInterface 
    getFederationManager() 
    { return this; } 

    /**
    */
    public synchronized DeclarationManagerInterface 
    getDeclarationManager() { return decManager; }

    /**
    */
    public synchronized ObjectManagerInterface 
    getObjectManager() { return objManager; }

    /**
    */
    public synchronized OwnershipManagerInterface 
    getOwnershipManager() { return ownManager; }

    /**
    */
    public synchronized DataDistributionManagerInterface 
    getDataDistributionManager() { return datManager; }

    /**
    */
    public synchronized TimeManagerInterface 
    getTimeManager() { return timManager; }

    /**
    */
    public synchronized RTIServicesInterface 
    getRTIServices() { return rtiServices; }

    /**
    * Hashtable regulatingFederates   = new Hashtable();
    * Hashtable constrainedFederates  = new Hashtable();
    */
    public void
    addTimeConstrainedList( short theHandle, FederateInfo fedInf  )
    {
       FederateInfo _fedInf = (FederateInfo) constrainedFederates.get(new Short(theHandle)); 
       if(_fedInf==null)
          constrainedFederates.put(new Short(theHandle),fedInf); 
      
    }// end of 

    /**
    */
    public void
    removeTimeConstrainedList( short theHandle )
    {
       FederateInfo _fedInf = (FederateInfo) constrainedFederates.get(new Short(theHandle)); 
       if(_fedInf!=null)
          constrainedFederates.remove(new Short(theHandle));
    }// end of 

    /**
    */
    public void
    addTimeRegulatingList( short theHandle, FederateInfo fedInf )
    {
       FederateInfo _fedInf = (FederateInfo) regulatingFederates.get(new Short(theHandle)); 
       if(_fedInf==null)
          regulatingFederates.put(new Short(theHandle),fedInf);
    }// end of 

    /**
    */
    public void
    removeTimeRegulatingList( short theHandle )
    {
       FederateInfo _fedInf = (FederateInfo) regulatingFederates.get(new Short(theHandle)); 
       if(_fedInf!=null)
          regulatingFederates.remove(new Short(theHandle));
    }// end of 

    /**
    *  Regulating Federates will be considered in this calculation.
    *  LBTS = MIN(Ti+Li) for all RFi;
    *  -1 : There is no body regulating in the Federation Execution
    *
    *  This method should be carefully designed.
    *
    *  case 1: Only the asking federate is the Regulating 
    *  case 2: There is no Regulating Federate
    *  case 3: (Normal) There are other regulating federates.
    *
    */
    public synchronized double
    calculateLBTS( short theHandle, FederateInfo fedInf )
    {
       
       Enumeration e = regulatingFederates.elements();
       double LBTS = 10000000f;  // Math.MAX_DOUBLE (?)
       FederateInfo _fedInf;

       if(e!=null && e.hasMoreElements() )
         for(;e.hasMoreElements();)
         {
          _fedInf = (FederateInfo) e.nextElement();
          if(_fedInf!=null && _fedInf != fedInf)
             LBTS = Math.min( LBTS, _fedInf.getFederateTime()+_fedInf.getFederateLookahead());

         }// end of for
       else LBTS = -1f;

       if(LBTS==10000000f) LBTS = -1f;

       FederationLBTS = Math.max(LBTS,0);
       FederationTime = FederationLBTS;

       return LBTS;

    }// end of 

    /**
    *  Regulating Federates will be considered in this calculation.
    *  LBTS = MIN(Ti+Li) for all RFi;
    *  -1 : There is no body regulating in the Federation Execution
    *
    *  This method should be carefully designed.
    *
    *  case 1: Only the asking federate is the Regulating 
    *  case 2: There is no Regulating Federate
    *  case 3: (Normal) There are other regulating federates.
    *
    */
    public synchronized double
    calculateLBTS( short theHandle, FederateInfo fedInf , double askedTime)
    {
       
       Enumeration e = regulatingFederates.elements();
       double LBTS = Math.max(askedTime,0);
       FederateInfo _fedInf;

       if(e!=null && e.hasMoreElements() )
         for(;e.hasMoreElements();)
         {
          _fedInf = (FederateInfo) e.nextElement();
          if(_fedInf!=null && _fedInf != fedInf)
             LBTS = Math.min( LBTS, _fedInf.getFederateTime()+_fedInf.getFederateLookahead());

         }// end of for

       FederationLBTS = Math.max(LBTS,0);
       FederationTime = FederationLBTS;

       return LBTS;

    }// end of 

   /** Test Stub */
   public static void 
   main(String args[])
   {

      RTIKernel rtiK = new RTIKernel();
      FederationExecution fedEx = new FederationExecution("HelloWorld",rtiK);

      System.out.println("--- DONE ---");

   }// end of main()

}// end of FederationExecution
