package owrti;

import RTIcap.*;
import java.util.*;
import java.io.*;

/**
*
*  For each interaction there is one Routing Space definition for it.
*    
*  Maybe we should define a default region in each routing space such
*  this region covers the whole space.
* 
* @date April 15, 1998 ( Updated at May 02, 1998)
* @author H. Timucin Ozdemir
*
*/
public class InteractionRecord  implements RTIDebug
{
               short  interactionId;
              String  interactionClassName;
     ParameterNaming  paramsDirectory;   
   InteractionRecord  father;

   Hashtable publishers  = new Hashtable();

   /** Routing Space */
   SpaceInfo r_space;

   RegionInfo defRegion;

   /** Regions defined in this Routing Space Q: Do I really want to keep it? */
   Hashtable regions = new Hashtable();

   /**
   */
   public
   InteractionRecord() 
   {
      paramsDirectory = new ParameterNaming(this);
      r_space = new SpaceInfo();
      defRegion  = r_space.getDefaultRegion();

   }// end of CONSTRUCTOR

   /**
   */
   public
   InteractionRecord(SpaceInfo _space) 
   {
      paramsDirectory = new ParameterNaming(this);
      r_space = _space;
      defRegion  = r_space.getDefaultRegion();

   }// end of CONSTRUCTOR

   /**
   */
   public final SpaceInfo
   getRoutingSpace() { return r_space; }

   /**
   */
   public final int
   getRoutingSpaceHandle() { return r_space.getHandle(); }

   /**
   */
   public String
   getName() { return interactionClassName; }

   /**
   */
   public short
   getId() { return interactionId; }

   /**
   */
   public void
   setName(String name) { interactionClassName=name; }

   /**
   */
   public void
   setId(short _id) { interactionId=_id; }

    /**
    */
    public short 
    addParameter(String theName)
    {
       return paramsDirectory.addParameter(theName);
    }

    /**
    */
    public short 
    getParameterHandle(String theName)
          throws RTIcap.NameNotFound, RTIcap.FederateNotExecutionMember, 
                 RTIcap.RTIinternalError 
    {
       return paramsDirectory.getParameterHandle(theName);
    }

    /**
    */
    public String 
    getParameterName(short theHandle)
           throws RTIcap.InteractionParameterNotDefined, 
                  RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       return paramsDirectory.getParameterName(theHandle);
    }

    /**
    * Add supscription to default region.
    */
    public void
    register_subscriber( FederateInfo fedInf )
    {
       if(DEBUG) System.out.println(" in register_subscriber() ");
       defRegion.register_subscriber(fedInf);
       if(DEBUG) System.out.println(" out of register_subscriber() ");

    }// end of register_subscriber()

    /**
    * Remove subscription from default region.
    */
    public void
    deregister_subscriber( short _fedId  )
    {
       if(DEBUG) System.out.println(" in deregister_subscriber() ");
       defRegion.deregister_subscriber(_fedId);
       if(DEBUG) System.out.println(" out of deregister_subscriber() ");

    }// end of deregister_subscriber()

    /**
    */
    public void
    register_publisher( FederateInfo fedInf )
    {
       if(DEBUG) System.out.println(" in register_publisher() ");
       publishers.put(new Short(fedInf.getFederateID()),fedInf);
       if(DEBUG) System.out.println(" out of register_publisher() ");

    }// end of register_publisher()

    /**
    */
    public void
    deregister_publisher( short _fedId )
    {
       FederateInfo n = (FederateInfo) publishers.get(new Short(_fedId));

       if(n!=null) publishers.remove(new Short(_fedId));

    }// end of deregister_publisher()

    /** 
    * Broadcast this message to all queues 
    */
    final void broadcast( Enumeration e, EventInfo evtInfo , short except )
    {
       FederateInfo n ;

       if(e!=null)
         for(;e.hasMoreElements();)
         {
            n = (FederateInfo) e.nextElement();

            if(n!=null && n.getFederateID()!=except) 
            {
               n.addEventInfo(evtInfo);

               if(DEBUG) System.out.println(" I sent to "+n.getFederateID());

            }// end of if

         }// end of for

    }// end of broadcast()

    /** 
    * Broadcast this message to all queues 
    */
    final void
    publishParamsToFederates( RTIcap.ParameterHandleValuePair[] theParameters, 
                   double theTime, String theTag, FederateInfo _fedHandle,
                   Enumeration e)
    {

       if(e!=null)
       {
          short _fedId = _fedHandle.getFederateID();
          if(DEBUG) System.out.println(" In publishParamsWithRegion() ");

          RTIcap.EventRetractionHandle retract = new RTIcap.EventRetractionHandle(theTime,0,_fedId);
          RTIEventRepository glbRepo = _fedHandle.getGlobalEvtRepository();
          int handle = glbRepo.addEvent( new EvtReceiveInteraction(interactionId,theParameters,theTag,retract,theTime));
          EventInfo evtInfo = new EventInfo(handle,EventInfo.GLOBAL);

          broadcast(e,evtInfo,_fedId);

       }// end of if

    }// end of publishParamsToFederates()

    /** 
    * Broadcast this message to all queues 
    */
    public void
    publishParams( RTIcap.ParameterHandleValuePair[] theParameters, 
                   double theTime, String theTag, FederateInfo _fedHandle)
    {

       if(DEBUG) System.out.println(" In publishParams() ");
       publishParamsToFederates( theParameters,theTime,theTag,_fedHandle,
                                 defRegion.getSubscribers());


       /*****************************************************************/
       /** Sure default Region intersect with every subscription region */
       /*****************************************************************/

       Enumeration e = null;
       if( regions != null ) e = regions.elements();
       RegionInfo rgInf ;
       
       if( e!= null)
         for(;e.hasMoreElements();)
         {
            rgInf = (RegionInfo) e.nextElement();

            if( rgInf != null )
                publishParamsToFederates( theParameters,theTime,theTag,
                                          _fedHandle, rgInf.getSubscribers());
            
         }// end of for

       if(DEBUG) System.out.println(" Out of publishParams() ");

    }// end of publishParams()

    /*********************************/
    /** Data Distribution Management */
    /*********************************/

    /**
    *
    * 1. Add this region to the subscribing regions list if it does not
    *    exist.
    *
    * 2. Add Federate to the subscriber list of this region.
    *
    */
    public void
    associateWithRegion(RegionInfo regInf, FederateInfo fedInf)
    {

       if( regions.get(new Integer(regInf.getHandle())) == null) 
           regions.put(new Integer(regInf.getHandle()),regInf);

       regInf.register_subscriber(fedInf);

    }// end of associateWithRegion() 

    /**
    *
    * 1. If subscribing region exist,  then remove this federate from 
    *    Region's subscription list.
    * 
    * Q: What happens if region has 0(ZERO) subscribers? Should I remove it?
    * 
    */
    public void
    unassociateFromRegion(RegionInfo regInf, FederateInfo fedInf)
    {
       if( regions.get(new Integer(regInf.getHandle())) != null) 
           regInf.deregister_subscriber(fedInf.getFederateID());

    }// end of unassociateFromRegion() 

    /** 
    * Broadcast this message to all queues in the intersecting regions. 
    *
    * Ask SpaceInfo about the possible subscribers which overlap with the
    * region.
    *
    */
    public void
    publishParamsWithRegion( RTIcap.ParameterHandleValuePair[] theParameters, 
                             double theTime, String theTag, RegionInfo regInf,
                             FederateInfo _fedHandle)
    {

       if(DEBUG) System.out.println(" In publishParamsWithRegion() ");

       /******************************************/
       /** Send everybody in the defeault region */
       /******************************************/
       publishParamsToFederates( theParameters,theTime,theTag,_fedHandle,
                                 defRegion.getSubscribers());

       Enumeration e = regions.elements();
       RegionInfo rgInf ;
       
       if( e!= null)
         for(;e.hasMoreElements();)
         {
            rgInf = (RegionInfo) e.nextElement();

            /**************************************************************/
            /** If there is intersection with the subscription region and */
            /** the publishing region then we have to to inform all       */
            /** tenants of the subscription region.                       */
            /**************************************************************/
            if( rgInf != null && regInf.canIntersect(rgInf) )
                publishParamsToFederates( theParameters,theTime,theTag,
                                          _fedHandle, rgInf.getSubscribers());
            
         }// end of for

       if(DEBUG) System.out.println(" Out of publishParamsWithRegion() ");

    }// end of publishParamsWithRegion()

    /**
    */
    public void
    dump(OutputStream out)
    {

      System.out.println(" Interaction Name : "+interactionClassName);
      System.out.println(" Interaction Id   : "+interactionId);
      int numOfParams = paramsDirectory.getNumberOfParameters(); 
      System.out.println(" # of Parameters : "+numOfParams);

      for(int i=1;i<(numOfParams+1);i++)
      try {
         System.out.println(i+"th params name : "+paramsDirectory.getParameterName((short)i));

      } catch(RTIcap.InteractionParameterNotDefined fnem) {}

     }// end of dump()

}// end of class InteractionRecord


