package owrti;

import RTIcap.*;
import java.util.*;

/**
*
* Keeps the ObjectClass-Instance to ObjectRecord mapping.
*
* @date April 29, 1998 ( Updated at May 02, 1998)
* @author H. Timucin Ozdemir
*
*/
public class 
ObjectClassNaming implements RTIDebug
{

   /** Class Id -> Class Name */
   Hashtable idToName = new Hashtable();

   /** Class Name -> Class Id */
   Hashtable nameToId = new Hashtable();

   /** Class Id -> Object Class Record */
   Hashtable idToObjRecord = new Hashtable();

   /** Instance Id -> Object Class Id */
   Hashtable objInsIdToClassId = new Hashtable();

   /** Instance Name -> Instance Id */
   Hashtable objNameToObjInsId = new Hashtable();

   InteractionNaming interNaming = null;   /// ?????

   /**
   */
   public 
   ObjectClassNaming(InteractionNaming _interNaming) 
   { interNaming = _interNaming; }

   /**
   * returns the Object Id of the particular Object Name
   */
   public short 
   getObjectClassHandle(String theName) throws RTIcap.NameNotFound
   {
       /**
       */

       String lookUpName = theName;
       Short ret=null;

       if(theName.startsWith("objectroot")||theName.startsWith("objectRoot")) 
       {
          lookUpName = "objectRoot."+theName.substring(11,theName.length());
          ret=(Short) nameToId.get(lookUpName); 
          System.out.println(" I asked "+theName+" and searched "+lookUpName);
       }
       else 
       {
          lookUpName = "objectRoot."+theName;
          ret = (Short) nameToId.get(lookUpName); 
          System.out.println(" I asked "+theName+" and searched "+lookUpName);
       }

       if(ret==null) throw new RTIcap.NameNotFound();
       System.out.println(" The handle of "+lookUpName+" is "+ret.shortValue());

       return ret.shortValue();
    }

   /**
   */
   public String 
   getObjectClassName(short theHandle) throws RTIcap.ObjectClassNotDefined 
   {
       String ret = (String) idToName.get(new Short(theHandle));

       if(ret==null) throw new RTIcap.ObjectClassNotDefined();

       return ret;

   }// end of getObjectClassName() 

   /**
   */
   public short 
   getAttributeHandle(String theName, short whichClass)
          throws RTIcap.ObjectClassNotDefined, RTIcap.NameNotFound 
   {
       ObjectRecord ret = (ObjectRecord) idToObjRecord.get(new Short(whichClass));

       if(ret==null) throw new RTIcap.ObjectClassNotDefined();

       return ret.getAttributeHandle(theName);

   }// end of getAttributeHandle()

   /**
   */
   public String 
   getAttributeName(short theHandle, short whichClass)
           throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined 
   {
       ObjectRecord ret = (ObjectRecord) idToObjRecord.get(new Short(whichClass));

       if(ret==null) throw new RTIcap.ObjectClassNotDefined();

       return ret.getAttributeName(theHandle);

   }// end of getAttributeName()

   /**
   */
   public final ObjectRecord 
   getObjectRecord( short whichClass ) 
   {
       return (ObjectRecord) idToObjRecord.get(new Short(whichClass));

   }// end of getObjectRecord()

   /**
   */
   public final ObjectRecord 
   getObjectRecord( String objectClassName ) 
   {
      ObjectRecord ret = null;
      Short classId = (Short) nameToId.get(objectClassName);

      if( classId != null ) ret = getObjectRecord(classId.shortValue());
      
      return ret;

   }// end of getObjectRecord()

   /**
   */
   public void 
   addObjectRecord(ObjectRecord rec)
   {

      idToName.put(new Short(rec.getId()),rec.getName());
      nameToId.put(rec.getName(),new Short(rec.getId()));
      idToObjRecord.put(new Short(rec.getId()),rec);
       
   }// end of addObjectRecord()

   /**
   */
   public void 
   removeObjectRecord(ObjectRecord rec)
   {

      idToName.remove(new Short(rec.getId()));
      nameToId.remove(rec.getName());
      idToObjRecord.remove(new Short(rec.getId()));
       
   }// end of removeObjectRecord()

   /**
   */
   public void 
   removeObjectRecord(short recId)
   {

      ObjectRecord rec = getObjectRecord(recId);
      if( rec != null ) removeObjectRecord(rec);
       
   }// end of removeObjectRecord()

   /**
   */
   public void 
   removeObjectRecord( String objectClassName )
   {

      ObjectRecord rec = getObjectRecord(objectClassName);
      if( rec != null ) removeObjectRecord(rec);
       
   }// end of removeObjectRecord()

   /**
   */
   public void 
   addObjectInstance(ObjectInstanceRecord rec)
   {

      objInsIdToClassId.put( new Integer(rec.getObjectInstanceHandle()), 
                             new Short(rec.getObjectClassHandle()) );

      objNameToObjInsId.put( rec.getObjectName(), new Integer(rec.getObjectInstanceHandle())); 
       
   }// end of addObjectInstance()

   /**
   */
   public void 
   removeObjectInstance(ObjectInstanceRecord rec)
   {
      objInsIdToClassId.remove( new Integer(rec.getObjectInstanceHandle())); 
      objNameToObjInsId.remove( rec.getObjectName() ); 

   }// end of removeObjectInstance()

   /**
   */
   public void 
   removeObjectInstance( int theHandle )
   {
      ObjectInstanceRecord rec = getObjectInstance(theHandle);
    
      if( rec != null ) removeObjectInstance(rec);

   }// end of removeObjectInstance()

   /**
   */
   public void 
   removeObjectInstance( String theName )
   {
      ObjectInstanceRecord rec = getObjectInstance(theName);
    
      if( rec != null ) removeObjectInstance(rec);

   }// end of removeObjectInstance()

   /**
   public ObjectInstanceRecord 
   getObjectInstance(int theHandle)
   {
      ObjectInstanceRecord ret = null; 
      ObjectRecord rec = (ObjectRecord) idToObjRecord.get(new Short((short)theHandle)); 
      if(rec!=null) ret = rec.getInstance(theHandle); 

      return ret; 

   }// end of addObjectInstance()
   */

   /**
   *  Find its class id
   *  Finds its ObjectRecord
   *  ask ObjectRecord to provide the particular ObjectInstance
   */
   public ObjectInstanceRecord 
   getObjectInstance(int theHandle)
   {
      ObjectInstanceRecord ret = null; 
      Short ClassId = (Short) objInsIdToClassId.get( new Integer(theHandle) );

      if( ClassId != null )
      {
          ObjectRecord rec = (ObjectRecord) idToObjRecord.get(new Short(ClassId.shortValue())); 
          if( rec != null ) ret = rec.getInstance(theHandle); 

      }// end of if

      return ret; 

   }// end of getObjectInstance()

   /**
   *  Find its class id
   *  Finds its ObjectRecord
   *  ask ObjectRecord to provide the particular ObjectInstance
   */
   public ObjectInstanceRecord 
   getObjectInstance( String theName )
   {
      ObjectInstanceRecord ret = null; 

      Integer objId = (Integer) objNameToObjInsId.get(theName);
      
      if( objId != null ) ret = getObjectInstance(objId.intValue());

      return ret; 

   }// end of getObjectInstance()

}// end of class
