/**
* Keeps the instance number, object class number, federate handle who
* defined this instance.
*/
package owrti;

import RTIcap.*;
import java.util.*;

/**
* 
* @date April 20, 1998 ( Updated at May 06, 1998)
* @author H. Timucin Ozdemir
*
*/
public class 
ObjectInstanceRecord extends ObjectRecord implements RTIDebug
{

   short  privelegeToDelete = -1;

   /** One who defined this object at the first place */
   short  theFedHandle;

   /** Instance of Which Object Class */
   short  theObjectClassHandle;

   /** Object Instance Id */
   int    theInstanceHandle;

   /** Object Instance Name */
   String    theInstanceName;

   FederateInfo fedInfo;

   ObjectRecord objFactory;

   /** Attribute Information center */
   AttributeNaming attributeDirectory;

   AttributeInfoNaming myAttributes ;

   /** */
   Hashtable attWatcher = new Hashtable();

   /**
   */
   public
   ObjectInstanceRecord(short _theFedHandle, short _theObjectClassHandle, 
                        int _theInstanceHandle, FederateInfo _father,
                        ObjectRecord _objFactory)
   {

      String n = new String( _objFactory.getName()+_theInstanceHandle );
      myAttributes = new AttributeInfoNaming(this);

      defineObjectInstance( _theFedHandle, _theObjectClassHandle,
                            _theInstanceHandle, _father, _objFactory, n );

   }// end of CONSTRUCTOR

   /**
   */
   public
   ObjectInstanceRecord(short _theFedHandle, short _theObjectClassHandle, 
                        int _theInstanceHandle, FederateInfo _father,
                        ObjectRecord _objFactory, String n)
   {

      myAttributes = new AttributeInfoNaming(this);
      defineObjectInstance( _theFedHandle, _theObjectClassHandle,
                            _theInstanceHandle, _father, _objFactory, n );

   }// end of CONSTRUCTOR

   /**
   */
   final void
   defineObjectInstance( short _theFedHandle, short _theObjectClassHandle,
                         int _theInstanceHandle, FederateInfo _father,
                         ObjectRecord _objFactory, String theName )
   {
      theFedHandle         = _theFedHandle;
      theObjectClassHandle = _theObjectClassHandle;
      theInstanceHandle    = _theInstanceHandle;
      fedInfo               = _father;
      privelegeToDelete    = theFedHandle;
      objFactory           = _objFactory;
      try
      {  attributeDirectory   = objFactory.getAttributeNaming((short)0);
      } catch(RTIcap.RTIinternalError fnem) {}

      defineAttributeWatchers();

      theInstanceName = new String( theName );

   }// end of defineObjectInstance()

   /**
   */
   final void defineAttributeWatchers()
   {
      if(attributeDirectory==null) return;

      Enumeration e = attributeDirectory.getAllNames();
      short ret ;
      String name; 

      for(;e.hasMoreElements();)
      {
         name = (String) e.nextElement();

         if(name!=null)
         {
            try
            {  
               ret = attributeDirectory.getAttributeHandle(name);

               if(ret>=0)
               {
                  attWatcher.put(new Short(ret),new AttributeWatcher(this,ret,name,theFedHandle));
                  myAttributes.addAttributeInfo(new AttributeInfo(this,ret,name,
                               theFedHandle,
                               objFactory.getDefaultRegionForAttribute(ret),
                               RTIcap.TransportType.BEST_EFFORT,
                               RTIcap.OrderType.RECEIVE));

               } // end of if

            } catch(RTIcap.NameNotFound fnem) {}

         }// end of if

      }// end of for

   }// end of defineAttributeWatchers()

   /**
   */
   public String 
   getObjectName() { return theInstanceName; }

   /**
   */
   public short 
   getFederateHandle() { return theFedHandle; }

   /**
   */
   public FederateAmbassador 
   getFederateAmbassador() { return fedInfo.getFederateAmbassador(); }

   /**
   */
   public FederateInfo 
   getFederateInfo() { return fedInfo; }

   /**
   */
   public short 
   getObjectClassHandle() { return theObjectClassHandle; }

   /**
   */
   public int   
   getObjectInstanceHandle() { return theInstanceHandle; }

   /**
   */
   public short   
   getOwnerHandle() { return privelegeToDelete; }

   /**
   */
   public final AttributeInfo   
   getAttributeInfo(short attributeHandle ) 
   { return myAttributes.getAttributeInfo(attributeHandle); }

   /**
   */
   public void   
   releaseOwnerships(short[] attributes, boolean unconditional ) 
   { 
       AttributeInfo attInfele;

       for(int i=0;i<attributes.length;i++)
       {
          attInfele = getAttributeInfo(attributes[i]); 

          if(attInfele!=null) 
          {   
             if(unconditional) attInfele.setOwnerId((short)-1);
             attInfele.setOwnershipTransferPermission();

          }// end of if

       }// end of for
      
   }// end of releaseOwnerships()

   /**
   */ 
   public final AttributeWatcher   
   getAttributeWatcher(short attribIndex)
   { return (AttributeWatcher) attWatcher.get(new Short(attribIndex)); }

   /**
   *
   * Go through all the attributeInfo objects in the attributeinfonaming
   * and set their region definition to the default region definition if
   * this attribute is in the given region.
   *
   */ 
   public void   
   unassociateAttributesFromRegion(RegionInfo theRegion)
   { 

          Enumeration   e = attWatcher.keys();
                Short   key;
        AttributeInfo   att;
      AttributeWatcher  aWatch;

      for(;e.hasMoreElements();)
      {
         key = (Short) e.nextElement();
      
         if(key!=null)
         {
            aWatch = (AttributeWatcher) attWatcher.get(key);
            att = (AttributeInfo) myAttributes.getAttributeInfo(key.shortValue());
            if( att != null && aWatch != null ) 
                att.setRegionInfo(aWatch.getDefaultRegion());

         }// end of if 

      }// end of for
 
   }// end of unassociateAttributesFromRegion()

   /**
   * 
   */ 
   public void   
   associateAttributesWithRegions(short[] attributes, RTIcap.Region rgn, 
          SpaceNaming spaceNaming )
   { 

       AttributeInfo crr;

       RegionInfo    regInf = spaceNaming.getRegionInfo(rgn);

       if( regInf != null )
         for(int i=0;i<attributes.length;i++)
         {
            crr = myAttributes.getAttributeInfo(attributes[i]);
         
            if( crr != null ) crr.setRegionInfo(regInf);
 
         }// end of for

   }// end of associateAttributesWithRegions()

   /**
   */ 
   public void   
   associateAttributesWithRegions(short[] attributes, RTIcap.Region[] rgns, 
          SpaceNaming spaceNaming )
   { 

      AttributeInfo crr;
      RegionInfo regInf ;

      for(int i=0; (i<attributes.length) && (i<rgns.length) ;i++)
      {
         crr = myAttributes.getAttributeInfo(attributes[i]);

         regInf = spaceNaming.getRegionInfo(attributes[i]);

         if(regInf!=null && crr!=null) crr.setRegionInfo(regInf);

      }// end of for

   }// end of unassociateFromRegion()

   /**
   *  For each attribute
   *  1. Get its Owner. If nobody owns it, then exit.
   *  2. Get its Region
   *  3. If its region is null or intersects with the given region.
   *     then send 'provideAtt..' message to this federate.
   */
   public void
   requestUpdateAttributesInRegion(short[] attributes, RegionInfo regInf)
   {
      AttributeInfo    att;
      RegionInfo       r;
      short            fid;
      FederateInfo     fdInf;
      short[]          val = new short[1];

      for(int i=0;i<attributes.length;i++)
      {
         att = myAttributes.getAttributeInfo(attributes[i]);

         if(att!=null)
         {
            fid = att.getOwnerId();

            if(fid>=0)
            {
               fdInf = fedEx.getFederateInfo(fid);

               if(fdInf!=null)
               {
                  r = att.getRegionInfo();
                  val[0] = attributes[i];

                  if( r == null || r.canIntersect(regInf) )
                     fdInf.addRTIEvent( new EvtProvideAttributeValueUpdate(theInstanceHandle,val));

               }// end of if

            }// end of if

         }// end of if

      }// end of for

   }// end of requestUpdateAttributesInRegion() 

   /**
   *  For each attribute, ask its owner to provide update for that
   *  particular attribute.
   *  If attribute has not an owner, then there is nobody to ask.
   */
   public void
   requestObjectAttributeValueUpdate(short[] theAttributes)
   {
      AttributeInfo att;
      short[] val = new short[1];
      FederateInfo fdInf;
      FederationExecution fedEx = fedInfo.getFederationExecution();
      RTIEventRepository glbRepo = fedInfo.getGlobalEvtRepository();
      int handle;

      /**************************************************/
      /** If FederationExecution and EventRepository OK */
      /**************************************************/
      if( fedEx != null && glbRepo != null )
      {
        /***********************/
        /** for each attribute */
        /***********************/
        for(int i=0;i<theAttributes.length;i++)
        {
           att = getAttributeInfo(theAttributes[i]); 

           /******************************/
           /** If attribute has an owner */
           /******************************/
           if( att != null && att.getOwnerId() >= 0 )
           {
              /***************************/
              /** Get owner of attribute */
              /***************************/
              fdInf = fedEx.getFederateInfo(att.getOwnerId());
   
              if( fdInf != null )
              {
                 val[0] = theAttributes[i];
                 handle = glbRepo.addEvent( new EvtProvideAttributeValueUpdate(theInstanceHandle,val));

                 /******************************/
                 /** Ask her to provide update */
                 /******************************/
                 fdInf.addEventInfo( new EventInfo(handle,EventInfo.GLOBAL));

              }// end of if

           }// end of if

        }// end of for

      }// end of if

   }// end of requestObjectAttributeValueUpdate()

   /**
   * Returns the Federate Id of Owner of given attribute
   */
   public short
   getAttributeOwner( short theHandle )
   {
      AttributeInfo att = getAttributeInfo(theHandle); 
      return ( att!=null? att.getOwnerId() : -1 );

   }// end of getAttributeOwner()

}// end of ObjectInstanceRecord
