/**
*
*/
package owrti;

import RTIcap.*;
import java.util.*;
import java.io.*;

/**
*
*  ObjectRecord object keeps information about one particular object
*  class. It knows its place in the Object Class Hierarchy. It knows 
*  its instances and its attributes.
*
* @date April 17, 1998 ( Last Updated May 2, 1998)
* @author H. Timucin Ozdemir
*
*/
public class 
ObjectRecord  implements RTIDebug
{
   /** Id of the object. It came as an short value. Might be an object */
   short objectId;

   /** Name of the object. It contains all the predecesors */
   String objectClassName;

   /** If root, it is null. */
   ObjectRecord father;                // Father in the Object Tree

   /** Attribute Information center */
   AttributeNaming attributeDirectory;   

   /** */
   Hashtable attWatcher = new Hashtable();

   /** Object Instances are going to be handled here */
   Hashtable objFactory = new Hashtable();

   //RegionInfo defRegion;
   //Hashtable  regions = new Hashtabel();

   /** In which federation this object exists */
   FederationExecution fedEx; 

   /** 
   */
   protected
   ObjectRecord() 
   {  attributeDirectory = new AttributeNaming(this); 
   }// end of ObjectRecord

   /** 
   */
   public
   ObjectRecord(FederationExecution _fedEx) 
   {  attributeDirectory = new AttributeNaming(this); 
      fedEx = _fedEx;
   }// end of ObjectRecord

   /** 
   */
   public
   ObjectRecord(FederationExecution _fedEx, ObjectRecord _father) 
   { 
       father = _father;
       attributeDirectory = new AttributeNaming(this); 
       fedEx = _fedEx;
   }// end of ObjectRecord

   /**
   * Returns the name of the attribute.
   */
   public String 
   getAttributeName(short theHandle) throws RTIcap.AttributeNotDefined 
   {
       return attributeDirectory.getAttributeName(theHandle);

   }// end of getAttributeName()

   /**
   * returns the id of the attribute.
   */
   public short 
   getAttributeHandle(String theName) throws RTIcap.NameNotFound
   {
       System.out.println(" Search for " +theName+" in getAttributeHandle() " );
       return attributeDirectory.getAttributeHandle(theName);

   }// end of getAttributeHandle()

   /**
   * returns the id of the attribute.
   */
   public final short[] 
   getAllAttributeHandles() 
   {  return attributeDirectory.getAllHandles();
   }// end of getAllAttributeHandles()

   /**
   */
   public String
   getName() { return objectClassName; }

   /**
   */
   public short
   getId() { return objectId; }

   /**
   */
   public void
   setName(String name) { objectClassName=name; }

   /**
   */
   public void
   setId(short _id) { objectId=_id; }

   /** 
   * Actually we need more info about the Attribute.
   * But, right now, this will do it.
   */
   public void
   addAttribute(String name) 
   { 
      if(DEBUG) System.out.println(" name : "+name); 

      if(name!=null) 
      {   
         short ret = attributeDirectory.addAttribute(name);

         /*******************************************************/
         /** For Class attribute definitions, there is no owner */
         /*******************************************************/
         if(ret>0) 
           attWatcher.put(new Short(ret),new AttributeWatcher(this,ret,name,(short)-1));

      }// end of if

   }// end of addAttribute()

   /**
   */
   public AttributeNaming 
   getAttributeNaming(short theHandle) throws RTIcap.RTIinternalError
   {
       return attributeDirectory;
   }

   /**
   */
   public AttributeWatcher 
   getAttributeWatcher(short theAttHandle) throws RTIcap.RTIinternalError
   {
       return (AttributeWatcher)attWatcher.get(new Short(theAttHandle));
   }

   /**
   *  Deletes an object and broadcast this event to everybody.
   *
   *  1. Find the ObjectInstanceRecord
   *  2. Tell everybody to delete this object from their locals
   *  3. Remove this object from the object factory
   *  4. Remove this object from all Attribute Watchers. 
   *
   */
   public void
   deregister_object( int theObject, FederateInfo _fedHandle )
   {
      ObjectInstanceRecord rec = (ObjectInstanceRecord) objFactory.get(new Integer(theObject));
      if(rec!=null)
      {
         /******************************************************/
         /** Inform everybody who has the proxy of this object */
         /******************************************************/
         if(_fedHandle!=null) 
            broadcastRemoveObject(theObject,_fedHandle,_fedHandle.getFederateID(),fedEx);
         objFactory.remove(new Integer(theObject));

         /********************************/
         /** Remove it from all Watchers */
         /********************************/
         Enumeration e = attWatcher.elements();
         AttributeWatcher curr;

         for(;e.hasMoreElements();)
         {
             curr = (AttributeWatcher) e.nextElement();

             if(curr!=null) curr.deregister_subscriber(_fedHandle.getFederateID());
           
         }// end of for
         

      }// end of if

   }// end of deregister_object()

   /**
   *  Broadcasts RemoveObjectInstance event to all federates in the 
   *  Federation.
   */
   public void
   broadcastRemoveObject( int theObject, FederateInfo _fedHandle, 
                          short _fedId , FederationExecution fedEx)
   {
      if(DEBUG) System.out.println(" In broadcastRemoveObject() " );
      Enumeration e = fedEx.getAllFederateInfos();

      if(e!=null)
      {
      
         if(DEBUG) System.out.println(" e is not NULL " );
         FederateInfo n;
         double time = 10;
         String tag  = new String("remove");
         RTIcap.EventRetractionHandle retract = new RTIcap.EventRetractionHandle(time,0,_fedId);

         RTIEventRepository glbRepo = _fedHandle.getGlobalEvtRepository();
         int handle = glbRepo.addEvent( new EvtRemoveObjectInstance(theObject,tag,retract));
         EventInfo evtInfo = new EventInfo(handle,EventInfo.GLOBAL);

         if(DEBUG) System.out.println(" before for " );
         for(;e.hasMoreElements();)
         { 
            n = (FederateInfo) e.nextElement();

            if(DEBUG) System.out.println(" FederateID : "+n.getFederateID() );
            if( n!=null && _fedHandle.getFederateID()!=n.getFederateID() ) 
            {
                n.addEventInfo(evtInfo);
            }

         }// end of for

      }// end of if

      if(DEBUG) System.out.println(" Out Of broadcastRemoveObject() " );

   }// end of broadcastRemoveObject()

   /**
   *  Defines an object and broadcast this event to everybody.
   */
   public void
   register_object(int theObject, FederateInfo _fedHandle, short _fedId,
FederationExecution fedEx )
   {
      ObjectInstanceRecord rec = new ObjectInstanceRecord(_fedId,objectId,theObject,_fedHandle,this);

      objFactory.put(new Integer(theObject), rec);

      if(_fedHandle!=null) 
      {
         broadcastDiscoverObject(theObject,_fedHandle,_fedId,fedEx);
         RTIEventRepository glbRepo = _fedHandle.getGlobalEvtRepository();
         int handle = glbRepo.addEvent( new EvtTurnUpdatesOnForObjectInstance(theObject,getAllAttributeHandles()));
         _fedHandle.addEventInfo( new EventInfo(handle,EventInfo.GLOBAL));

         handle = glbRepo.addEvent( new EvtStartRegistrationForObjectClass(objectId));
         _fedHandle.addEventInfo( new EventInfo(handle,EventInfo.GLOBAL));

         if(fedEx.getNumberOfFederates()<2)
         {  handle = glbRepo.addEvent( new EvtTurnUpdatesOffForObjectInstance(theObject,getAllAttributeHandles()));
           _fedHandle.addEventInfo( new EventInfo(handle,EventInfo.GLOBAL));
         }// end of if 

      }// end of if 

   }// end of register_object()

   /**
   *  Inform all federates in the federation execution about the 
   *  new object definition.
   */
   public void
   broadcastDiscoverObject( int theObject, FederateInfo _fedHandle, 
                            short _fedId , FederationExecution fedEx)
   {
      if(DEBUG) System.out.println(" In broadcastDiscoverObject() " );
      Enumeration e = fedEx.getAllFederateInfos();

      if(e!=null)
      {
      
         if(DEBUG) System.out.println(" e is not NULL " );
         FederateInfo n;
         double time = 10;
         String tag  = new String("discover");
         RTIcap.EventRetractionHandle retract = new RTIcap.EventRetractionHandle(time,0,_fedId);

         RTIEventRepository glbRepo = _fedHandle.getGlobalEvtRepository();
         int handle = glbRepo.addEvent( new EvtDiscoverObject(theObject,objectId,_fedId,time));
         EventInfo evtInfo = new EventInfo(handle,EventInfo.GLOBAL);

         if(DEBUG) System.out.println(" before for " );
         for(;e.hasMoreElements();)
         { 
            n = (FederateInfo) e.nextElement();

            if(DEBUG) System.out.println(" FederateID : "+n.getFederateID() );
            if( n!=null && _fedHandle.getFederateID()!=n.getFederateID() ) 
            {
                n.addEventInfo(evtInfo);
            }

         }// end of for

      }// end of if

      if(DEBUG) System.out.println(" Out Of broadcastDiscoverObject() " );

   }// end ofbroadcastDiscoverObject()

   /**
   *  Inform the Federate about existance of other object instances.
   */
   public void
   broadcastObjectInstances( FederateInfo _fedHandle )
   {
      if(DEBUG) System.out.println(" In broadcastObjectInstances() " );
      Enumeration e = objFactory.elements();

      if( e != null)
      {
      
         for(;e.hasMoreElements();)
         { 
            ObjectInstanceRecord rec = (ObjectInstanceRecord) e.nextElement();
  
            if( rec!=null && rec.getOwnerHandle()!= _fedHandle.getFederateID() )
             _fedHandle.addRTIEvent(new EvtDiscoverObject(rec.getObjectInstanceHandle(), rec.getObjectClassHandle(), rec.getOwnerHandle() ));
         }// end of for

      }// end of if

      if(DEBUG) System.out.println(" Out Of broadcastObjectInstances() " );

   }// end of broadcastObjectInstances()

   /**
   *  Broadcast everybody the update message about this particular object
   *  and its attributes. Should I keep copy of these attributes?
   */
   public synchronized void
   updateAttributeValues( int theObject, 
                          RTIcap.AttributeHandleValuePair[] theAttributes,
                          double theTime, String theTag,
                          FederateAmbassador _fedHandle, short _fedId )

         throws RTIcap.AttributeNotDefined,
                RTIcap.AttributeNotOwned, RTIcap.InvalidFederationTime,
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
   {
      isExecutionMember(_fedId); 
      
      if(DEBUG) System.out.println(" In updateAttributeValues() for "+theObject ) ;
      if(DEBUG) System.out.println(" theAttributes.length "+theAttributes.length ) ;
      for(int i=0;i<theAttributes.length;i++)
      {
         AttributeWatcher attWatch = (AttributeWatcher)attWatcher.get(new Short(theAttributes[i].handle));

         if(attWatch!=null) 
          try
          {
            attWatch.broadcast(theObject,theAttributes[i],theTime,theTag,_fedHandle,_fedId);
          } catch(RTIcap.FederateInternalError fie) {}
            catch(RTIcap.ObjectNotKnown onk) {}
            catch(RTIcap.AttributeNotKnown ank) {}

      }// end of for

      if(DEBUG) System.out.println(" Out of updateAttributeValues() for "+theObject ) ;

   }// end of updateAttributeValues()

   /**
   * Ask each federate who has an instance of this object class to update
   * their attributes.
   */
   public void
   requestClassAttributeValueUpdate(short[] theAttributes, FederateInfo fedInf)
         throws RTIcap.AttributeNotDefined, RTIcap.FederateNotExecutionMember,
                RTIcap.RTIinternalError
   {
      Enumeration e = objFactory.keys();
      Integer key;
      FederateInfo n;
      ObjectInstanceRecord objInstance;

      RTIEventRepository glbRepo = fedInf.getGlobalEvtRepository();
      int handle ;

      for(;e.hasMoreElements();)
      { 
         key = (Integer) e.nextElement();
   
         if(key!=null)
         {
            /** Get the Object instance Record */ 
            objInstance = (ObjectInstanceRecord) objFactory.get(key); 

            if(objInstance!=null) 
            { 
               /** Get the handle of responsible FederateAmbassador object */
               n = objInstance.getFederateInfo();

               /**************************************************/
               /** Ask him/her to update this attributes of that */
               /** particular object                             */
               /**************************************************/
               if(n!=null) 
               { 
                  handle = glbRepo.addEvent( new EvtProvideAttributeValueUpdate(key.intValue(),theAttributes));
                  n.addEventInfo(new EventInfo(handle,EventInfo.GLOBAL)); 

               }// end of if

            }// end of if 

         }// end of if

      }// end of for
      
   }// end of requestClassAttributeValueUpdate()

   /**
   * Ask each object to apply requestObjectAttributeValueUpdate()
   */
   public void
   requestClassAttributeValueUpdate(short[] theAttributes)
   {
      Enumeration e = objFactory.elements();
      ObjectInstanceRecord objInstance;

      for(;e.hasMoreElements();)
      { 
         /** Get the Object instance Record */ 
         objInstance = (ObjectInstanceRecord) e.nextElement();

         if(objInstance!=null) 
            objInstance.requestObjectAttributeValueUpdate(theAttributes);

      }// end of for
      
   }// end of requestClassAttributeValueUpdate()


   /**
   *
   */
   final void 
   isExecutionMember( short fedId ) throws RTIcap.FederateNotExecutionMember
   {
      //if(fedEx.getFederateInfo(fedId)==null) 
      //   throw new RTIcap.FederateNotExecutionMember();
   } 

   /**
   */
   public ObjectInstanceRecord 
   getInstance( int theHandle ) 
   { return (ObjectInstanceRecord) objFactory.get(new Integer(theHandle)); } 

   /*********************************/
   /** Data Distribution Management */
   /*********************************/

   /**
   *  send the same message to all object instances of this class.
   */
   public void
   requestUpdateAttributesInRegion( short[] attributes, RegionInfo regInf)
   {
      Enumeration e = objFactory.elements();
      ObjectInstanceRecord rec;

      for(;e.hasMoreElements();)
      {
         rec = (ObjectInstanceRecord) e.nextElement();
         if(rec!=null) rec.requestUpdateAttributesInRegion(attributes,regInf);

      }// end of for

   }// end of requestUpdateAttributesInRegion()

   /**
   * 1. Find AttWatcher
   * 2. Add this region to AttributeWatcher Subscribing regions list.
   */ 
   public void
   subscribeAttributesWithRegion( short[] attributeList, RegionInfo rgn)
   {
      AttributeWatcher att;

      for(int i=0;i<attributeList.length;i++)
      {
         try 
         {  
            att = getAttributeWatcher(attributeList[i]);

            if( att != null ) att.register_subscribingRegion(rgn); 

         } catch( RTIcap.RTIinternalError ie) {}

      }// end of for

   }// end of subscribeAttributesWithRegion()

   /**
   * Returns default region definition for the attribute.
   */
   public RegionInfo
   getDefaultRegionForAttribute( short attribute )
   {
      RegionInfo ret = null;
      AttributeWatcher att;

      try 
      {  
          att = getAttributeWatcher(attribute);
         if( att != null ) ret = att.getRegionInfo();

      } catch(RTIcap.RTIinternalError ie) {}

      return ret;

   }// end of getDefaultRegionForAttribute()

   /**
   * Returns the handle of all owners of given attribute
   */
   public Enumeration
   getAllOwnersOfAttribute( short theHandle )
   {
      Vector fdIds = new Vector();
      Enumeration e = objFactory.elements();
      ObjectInstanceRecord rec;
      int ownId;
      Short ky;

      Vector ret = new Vector();
      FederateInfo n;

      for(;e.hasMoreElements();)
      {
         rec = (ObjectInstanceRecord) e.nextElement();

         if( rec != null )
         {
            ownId = rec.getAttributeOwner(theHandle);
            if(ownId >0 && !fdIds.contains(new Integer(ownId)))
               fdIds.addElement(new Integer(ownId));
               
         }// end of if

      }// end of for

      for(int i=0;i<fdIds.size();i++)
      {
         ky = (Short) ret.elementAt(i);
         if( ky != null )
         {
            n = fedEx.getFederateInfo(ky.shortValue());
            if(n!=null) ret.addElement(n); 

         }// end of if

      }// end of foR

      return ret.elements();

   }// end of getAllOwnersOfAttribute()

   /**
   *
   *  Sends "turnUpdatesOnForObjectinstance" message to all owners of the
   *  particular attribute of an object instance.
   *
   */
   void
   turnUpdatesOnOffForAllOwners(short theHandle, boolean OnOff)
   {

      short[] attribs = new short[1];
      attribs[0] = theHandle;

      Enumeration e = objFactory.elements();
      ObjectInstanceRecord rec;
      int handle;
      short ownId;
      FederateInfo n;

      for(;e.hasMoreElements();)
      {
         rec = (ObjectInstanceRecord) e.nextElement();

         if( rec != null )
         {
            ownId = rec.getAttributeOwner(theHandle);

            if( ownId > 0 )
            { 
               n = fedEx.getFederateInfo(ownId);

               if(n!=null) 
               {
                  RTIEventRepository glbRepo = n.getGlobalEvtRepository();
      
                  if(OnOff) 
                     handle = glbRepo.addEvent( new EvtTurnUpdatesOnForObjectInstance(rec.getObjectInstanceHandle(),attribs) );
                  else
                     handle = glbRepo.addEvent( new EvtTurnUpdatesOffForObjectInstance(rec.getObjectInstanceHandle(),attribs) );

                  n.addEventInfo( new EventInfo( handle, EventInfo.LOCAL ) );

               }// end of if

            }// end of if
               
         }// end of if

      }// end of for

   }// end of turnUpdatesOnOffForAllOwners()

   /**
   *
   */
   public void
   dump(OutputStream out)
   {

      System.out.println(" Object Name : "+objectClassName);
      System.out.println(" Object Id   : "+objectId);
      int numOfAtt = attributeDirectory.getNumberOfAttributes(); 
      System.out.println(" # of Attributes : "+numOfAtt);

      for(int i=1;i<(numOfAtt+1);i++)
      try {
         System.out.println(i+"th att name : "+attributeDirectory.getAttributeName((short)i));

      } catch(RTIcap.AttributeNotDefined  and) {}

   }// end of dump()

   public void
   dump2(OutputStream out)
   {
      try {
      PrintWriter pw = new PrintWriter(out);
      pw.println(" Object Name : ");
      pw.println(" Object Id   : ");
      int numOfAtt = attributeDirectory.getNumberOfAttributes(); 
      pw.println(" # of Attributes : "+numOfAtt);

      for(short i=0;i<numOfAtt;i++)
         pw.println(i+"th att name : "+attributeDirectory.getAttributeName(i));

      } catch(RTIcap.AttributeNotDefined  and) {}

   }// end of dump()

}// end of class ObjectRecord

