/**
*  RTIambassador object has to know where the RTIKernel is and which
*  Federate it represents.
*  
*  May 1st, Migration to RTI 1.3 started
*/
package owrti;

import RTIcap.*;
import npacrti.*;

import org.omg.CORBA.*;

/**
*
* @date April 20, 1998 ( Updated at April 30, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
RTIAmb extends RTIcap._RTIambassadorImplBase implements RTIDebug
{

    RTIKernel          rtiCore ;
    short              FederateId;
    String             FederateName;
    String             FederationName;
    FederateAmbassador fedAmbassador; 

    FederationManagerInterface       fedManager ;
    DeclarationManagerInterface      decManager ;
    ObjectManagerInterface           objManager ;
    OwnershipManagerInterface        ownManager ;
    TimeManagerInterface             timManager ;
    DataDistributionManagerInterface datManager ;
    RTIServicesInterface             rtiServices ;

    /**
    */
    public
    RTIAmb(RTIKernel _rtiCore)
    {

       rtiCore = _rtiCore;

    }// end of RTIAmb

    /**
    */
    public
    void beginTicking(int sleepTime) throws RTIcap.TickingPreviouslyStarted
    {
       System.out.println(" I will start tickining with "+sleepTime);
    }

    /**
    */
    public
    void endTicking() throws RTIcap.TickingNotEnabled
    {
    }

    /****************************/
    /**** Federation Management */
    /****************************/

    /**
    */
    public void 
    createFederationExecution(String executionName, String FED)
         throws RTIcap.FederationExecutionAlreadyExists, RTIcap.RTIinternalError 
    {

       try 
       {  rtiCore.createFederationExecution(executionName,FED);
       } catch(RTIcap.FederationExecutionAlreadyExists feae) {}
 
       try 
       {  
          fedManager = rtiCore.getFederationManager(executionName);
          decManager = rtiCore.getDeclarationManager(executionName);
          objManager = rtiCore.getObjectManager(executionName);
          ownManager = rtiCore.getOwnershipManager(executionName);
          timManager = rtiCore.getTimeManager(executionName);
          datManager = rtiCore.getDataDistributionManager(executionName);
          rtiServices = rtiCore.getRTIServices(executionName);
       } catch(RTIcap.FederationExecutionDoesNotExist feae) {}

    }// end of createFederationExecution()

    /**
    */
    public void 
    destroyFederationExecution(String executionName)
         throws RTIcap.FederatesCurrentlyJoined, 
                RTIcap.FederationExecutionDoesNotExist, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.destroyFederationExecution(executionName,FederateId);
    }// end of destroyFederationExecution() 

    /**
    */
    public short 
    joinFederationExecution(String yourName, String executionName, 
                 RTIcap.FederateAmbassador federateAmbassador)
          throws RTIcap.FederateAlreadyExecutionMember, 
                 RTIcap.FederationExecutionDoesNotExist, RTIcap.CouldNotOpenFED, 
                 RTIcap.ErrorReadingFED, RTIcap.SaveInProgress,
                 RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        FederateName = new String(yourName);
        FederationName = new String(executionName);
        fedAmbassador = federateAmbassador; 
        FederateId = fedManager.joinFederationExecution(yourName,executionName,fedAmbassador);

        String[] ret = null;
        //ORB orb = ORB.init(ret, null);
        //System.out.println(" FedAmb IOR:"+org.omg.CORBA.ORB.init().object_to_string(fedAmbassador));
        return FederateId;

    }// end of joinFederationExecution()

    /**
    */
    public void 
    resignFederationExecution(RTIcap.ResignAction theAction )
         throws RTIcap.FederateOwnsAttributes, 
                RTIcap.FederateNotExecutionMember, RTIcap.InvalidResignAction, 
                RTIcap.RTIinternalError 
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.resignFederationExecution(theAction,FederationName,FederateId); 
    }// end of resignFederationExecution()

    /**
    */
    public void 
    registerFederationSynchronizationPoint(String label, String tag )
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.registerFederationSynchronizationPoint(label,tag,FederateId); 
    }// end of registerFederationSynchronizationPoint()

    /**
    */
    public void 
    registerFederationSynchronizationPointWithSet(String label, String tag, 
        short[] syncSet)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.registerFederationSynchronizationPointWithSet(label,tag,syncSet,FederateId); 
    }// end of registerFederationSynchronizationPointWithSet()

    /**
    */
    public void 
    synchronizationPointAchieved(String label)
        throws RTIcap.SynchronizationPointLabelWasNotAnounced, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.synchronizationPointAchieved(label,FederateId); 
    }// end of synchronizationPointAchieved()

    /**
    */
    public void 
    requestFederationSaveWithTime(String label, double theTime )
        throws RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime,
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.requestFederationSaveWithTime(label,theTime,FederateId); 
    }// end of requestFederationSaveWithTime()

    /**
    */
    public void 
    requestFederationSave(String label)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.requestFederationSave(label,FederateId); 
    }// end of requestFederationSave()

    /**
    */
    public void 
    federateSaveBegun()
        throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.federateSaveBegun(FederateId); 
    }// end of federateSaveBegun()

    /**
    */
    public void 
    federateSaveComplete()
        throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.federateSaveComplete(FederateId); 
    }// end of federateSaveComplete()

    /**
    */
    public void 
    federateSaveNotComplete()
        throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.federateSaveNotComplete(FederateId); 
    }// end of federateSaveNotComplete()

    /**
    */
    public void 
    requestFederationRestore(String label)
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.requestFederationRestore(label,FederateId); 
    }// end of requestFederationRestore()

    /**
    */
    public void 
    federateRestoreComplete()
        throws RTIcap.RestoreNotRequested, RTIcap.FederateNotExecutionMember, 
               RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.federateRestoreComplete(FederateId); 
    }// end of federateRestoreComplete()

    /**
    */
    public void 
    federateRestoreNotComplete()
        throws RTIcap.RestoreNotRequested, RTIcap.FederateNotExecutionMember, 
               RTIcap.RTIinternalError
    {
        if(fedManager==null) throw new RTIcap.RTIinternalError();
        fedManager.federateRestoreNotComplete(FederateId); 
    }// end of federateRestoreNotComplete()

    /*****************************/
    /**** Declaration Management */
    /*****************************/

    /**
    */
    public void 
    publishObjectClass(short theClass, short[] attributeList)
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.publishObjectClass(theClass,attributeList,FederateId);
    }// end of publishObjectClass()

    /**
    */
    public void 
    unpublishObjectClass(short theClass)
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateOwnsAttributes, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.ObjectClassNotPublished, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.unpublishObjectClass(theClass,FederateId);
    }// end of unpublishObjectClass() 

    /**
    */
    public
    void publishInteractionClass(short theInteraction)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.publishInteractionClass(theInteraction,FederateId);
    }// end of publishInteractionClass()

    /**
    */
    public void 
    unpublishInteractionClass(short theInteraction)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.InteractionClassNotPublished, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.unpublishInteractionClass(theInteraction,FederateId);
    }// end of unpublishInteractionClass()

    /**
    */
    public void 
    subscribeObjectClassAttributes(short theClass, short[] attributeList,
         RTIcap._Boolean active)
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.RTIinternalError 
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.subscribeObjectClassAttributes(theClass,attributeList,active,FederateId);
    }// end of subscribeObjectClassAttributes()

    /**
    */
    public void 
    subscribeObjectClassAttribute(short theClass, short theAttribute)
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       //decManager.unsubscribeObjectClassAttribute2(theClass,theAttribute,fedAmbassador,FederateId);
    }

    /**
    */
    public void 
    unsubscribeObjectClass(short theClass)
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateNotExecutionMember, 
                RTIcap.RestoreInProgress, RTIcap.SaveInProgress, 
                RTIcap.ObjectClassNotSubscribed, 
                RTIcap.RTIinternalError 
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.unsubscribeObjectClass(theClass,FederateId);
    }// end of unsubscribeObjectClass()

    /**
    */
    public void 
    subscribeInteractionClass(short theClass, RTIcap._Boolean active)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.FederateLoggingServiceCalls, 
                RTIcap.RestoreInProgress, 
                RTIcap.SaveInProgress, 
                RTIcap.RTIinternalError
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       System.out.println(" In subscribeInteractionClass()");
       decManager.subscribeInteractionClass(theClass,active,FederateId);
    }// end of subscribeInteractionClass()

    /**
    */
    public void 
    unsubscribeInteractionClass(short theClass)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.InteractionClassNotSubscribed, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.RestoreInProgress, 
                RTIcap.SaveInProgress, 
                RTIcap.RTIinternalError 
    {
       if(decManager==null) throw new RTIcap.RTIinternalError();
       decManager.unsubscribeInteractionClass(theClass,FederateId);
    }// end of unsubscribeInteractionClass()

    /************************/
    /**** Object Management */
    /************************/

    /**
    * Defines an object with the given name in given object class.
    * @return Object Handle
    */
    public int 
    registerObjectInstanceWithName(short theClass, String theName)
        throws RTIcap.ObjectClassNotDefined, RTIcap.ObjectClassNotPublished, 
               RTIcap.ObjectAlreadyRegistered, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(objManager==null) throw new RTIcap.RTIinternalError();
       return objManager.registerObjectInstanceWithName(theClass,theName,fedAmbassador,FederateId);
    }// end of registerObjectInstanceWithName()

    /**
    * Defines an object in given object class.
    * @return Object Handle
    */
    public int 
    registerObjectInstance(short theClass)
        throws RTIcap.ObjectClassNotDefined, RTIcap.ObjectClassNotPublished, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(objManager==null) throw new RTIcap.RTIinternalError();
       return objManager.registerObjectInstance(theClass,fedAmbassador,FederateId);
    }// end of  registerObjectInstance()

    /**
    */
    public RTIcap.EventRetractionHandle 
    updateAttributeValuesWithTime(int theObject,
        RTIcap.AttributeHandleValuePair[] theAttributes, double theTime, 
        String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined,
               RTIcap.AttributeNotOwned, RTIcap.InvalidFederationTime, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(DEBUG) System.out.println(" in updateAttributeValues for "+theObject+" length "+theAttributes.length);
       if(objManager==null) throw new RTIcap.RTIinternalError();
       return objManager.updateAttributeValuesWithTime(theObject,theAttributes,theTime,theTag,fedAmbassador,FederateId);
    }// end of updateAttributeValuesWithTime()

    /**
    */
    public void 
    updateAttributeValues(int theObject, 
        RTIcap.AttributeHandleValuePair[] theAttributes, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
               RTIcap.AttributeNotOwned, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
       if(DEBUG) System.out.println(" in updateAttributeValues for "+theObject+" length "+theAttributes.length);
       if(objManager==null) throw new RTIcap.RTIinternalError();
       objManager.updateAttributeValues(theObject,theAttributes,theTag,fedAmbassador,FederateId);
    }// end of updateAttributeValues()

    /**
    */
    public RTIcap.EventRetractionHandle 
    sendInteractionWithTime(short theInteraction,
        RTIcap.ParameterHandleValuePair[] theParameters, double theTime, 
        String theTag)
        throws RTIcap.InteractionClassNotDefined,
               RTIcap.InteractionClassNotPublished, 
               RTIcap.InteractionParameterNotDefined, 
               RTIcap.InvalidFederationTime, RTIcap.FederateNotExecutionMember, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
       if(objManager==null) throw new RTIcap.RTIinternalError();
       return objManager.sendInteractionWithTime(theInteraction,theParameters,theTime,theTag,fedAmbassador,FederateId);
    }// end of sendInteractionWithTime()

    /**
    */
    public void 
    sendInteraction(short theInteraction, 
        RTIcap.ParameterHandleValuePair[] theParameters, String theTag)
        throws RTIcap.InteractionClassNotDefined,
               RTIcap.InteractionClassNotPublished, 
               RTIcap.InteractionParameterNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(objManager==null) throw new RTIcap.RTIinternalError();
       objManager.sendInteraction(theInteraction,theParameters,theTag,fedAmbassador,FederateId);
    }// end of sendInteraction()

    /**
    */
    public
    void changeAttributeTransportType(int theObject, short[] theAttributes, 
                                      RTIcap.TransportType theType)
         throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
                RTIcap.AttributeNotOwned, RTIcap.InvalidTransportationHandle, 
                RTIcap.RestoreInProgress, RTIcap.SaveInProgress, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.changeAttributeTransportType(theObject,theAttributes,theType,fedAmbassador,FederateId);;
    }

    /**
    */
    public
    void changeInteractionTransportType(short theClass, RTIcap.TransportType theType)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.InteractionClassNotPublished, RTIcap.RestoreInProgress, 
                RTIcap.InvalidTransportationHandle, RTIcap.SaveInProgress, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.changeInteractionTransportType(theClass,theType,fedAmbassador,FederateId);;
    }// end of changeInteractionTransportType()

    /**
    */
    public void
    requestObjectAllAttributeValueUpdate(int theObject)
         throws RTIcap.ObjectNotKnown, RTIcap.FederateNotExecutionMember, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.RTIinternalError 
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.requestObjectAttributeValueUpdate2(theObject,fedAmbassador,FederateId);;
    }// end of requestObjectAllAttributeValueUpdate()

    /**
    */
    public void 
    requestObjectAttributeValueUpdate(int theObject, short[] theAttributes)
         throws RTIcap.AttributeNotDefined, RTIcap.ObjectNotKnown, 
                RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
                RTIcap.RestoreInProgress, RTIcap.RTIinternalError 
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.requestObjectAttributeValueUpdate1(theObject,theAttributes,fedAmbassador,FederateId);;
    }// end of requestObjectAttributeValueUpdate()

    /**
    */
    public void 
    requestClassAttributeValueUpdate(short theClass, short[] theAttributes)
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.requestClassAttributeValueUpdate1(theClass,theAttributes,fedAmbassador,FederateId);

    }// end of requestClassAttributeValueUpdate()

    /**
    */
    public void 
    requestClassAllAttributeValueUpdate(short theClass)
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateNotExecutionMember,
                RTIcap.RTIinternalError, RTIcap.SaveInProgress, RTIcap.RestoreInProgress
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.requestClassAttributeValueUpdate2(theClass,fedAmbassador,FederateId);
    }// end of requestClassAllAttributeValueUpdate()

    /**
    */
    public void 
    retract(RTIcap.EventRetractionHandle theHandle)
         throws RTIcap.InvalidRetractionHandle, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
    }// end of retract()

    /**
    */
    public RTIcap.EventRetractionHandle 
    deleteObjectInstanceWithTime(int objID, double theTime, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.DeletePrivilegeNotHeld,
               RTIcap.InvalidFederationTime, RTIcap.FederateNotExecutionMember,
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, RTIcap.RTIinternalError 
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        return objManager.deleteObjectInstanceWithTime(objID,theTime,theTag,fedAmbassador,FederateId);
    }// end of deleteObjectInstanceWithTime()

    /**
    */
    public void 
    deleteObjectInstance(int objID, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.DeletePrivilegeNotHeld,
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.deleteObjectInstance(objID,theTag,fedAmbassador,FederateId);

    }// end of deleteObjectInstance()

    /**
    */
    public void 
    localdeleteObjectInstance(int objID)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateOwnsAttributes,
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
        if(objManager==null) throw new RTIcap.RTIinternalError();
        objManager.localdeleteObjectInstance(objID,fedAmbassador,FederateId);
    }// end of localdeleteObjectInstance()


    /***************************/
    /**** Ownership Management */
    /***************************/

    /**
    */
    public void 
    queryAttributeOwnership(int theObject, short theAttribute)
         throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
                RTIcap.RestoreInProgress, RTIcap.SaveInProgress, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       ownManager.queryAttributeOwnership( theObject,theAttribute, FederateId);

    }// end of queryAttributeOwnership()

    /**
    public RTIcap._Boolean 
    attributeIsOwnedByFederate(int theObject, short theAttribute)
                    throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
                           RTIcap.AttributeNotKnown, 
                           RTIcap.FederateNotExecutionMember, 
                           RTIcap.ConcurrentAccessAttempted, 
                           RTIcap.RTIinternalError, RTIcap.Deadlock
    {
        _Boolean ret = RTIcap._Boolean.RTI_FALSE;

        return ret;
    }
    */

    /**
    */
    public void 
    attributeOwnerhsipAcquisitionIfAvailable(int object, short[] attributes)
        throws RTIcap.AttributeAlreadyBeingAcquired, RTIcap.AttributeNotDefined,
               RTIcap.AttributeNotPublished, RTIcap.FederateNotExecutionMember, 
               RTIcap.FederateOwnsAttributes, RTIcap.ObjectClassNotPublished, 
               RTIcap.ObjectNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       ownManager.attributeOwnerhsipAcquisitionIfAvailable( object,attributes,
                                                            FederateId);
    }// end of attributeOwnerhsipAcquisitionIfAvailable()

    /**
    */
    public void 
    cancelNegotiatedAttributeOwnershipDivestiture(int object, short[] attributes)
        throws RTIcap.AttributeDivestitureWasNotRequested,
               RTIcap.AttributeNotDefined, RTIcap.AttributeNotOwned, 
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectNotKnown, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       ownManager.cancelNegotiatedAttributeOwnershipDivestiture( object,attributes,
                                                              FederateId);
    }// end of cancelNegotiatedAttributeOwnershipDivestiture()

    /**
    */
    public void 
    unconditionalAttributeOwnershipDivestiture(int object, short[] attributes)
        throws RTIcap.AttributeNotDefined, RTIcap.AttributeNotOwned,
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectNotKnown,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();

       ownManager.unconditionalAttributeOwnershipDivestiture( object,attributes,
                                                              FederateId);

    }// end of unconditionalAttributeOwnershipDivestiture()

    /**
    */
    public short[] 
    attributeOwnershipReleaseResponse(int object, short[] attributes)
        throws RTIcap.AttributeNotDefined, RTIcap.AttributeNotOwned,
               RTIcap.FederateNotExecutionMember,
               RTIcap.FederateWasNotAskedToReleaseAttribute, 
               RTIcap.ObjectNotKnown, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       return ownManager.attributeOwnershipReleaseResponse(object,attributes,FederateId);
    }// end of attributeOwnershipReleaseResponse()

    /**
    */
    public void 
    negotiatedAttributeOwnershipDivestiture(int object, short[] attributes, String tag)
        throws RTIcap.AttributeAlreadyBeingDivested, RTIcap.AttributeNotDefined,
               RTIcap.AttributeNotOwned, RTIcap.FederateNotExecutionMember, 
               RTIcap.ObjectNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       ownManager.negotiatedAttributeOwnershipDivestiture(object,attributes,tag,FederateId);
    }// end of negotiatedAttributeOwnershipDivestiture()

    /**
    */
    public void 
    attributeOwnershipAcquisition(int object, short[] attributes, String tag)
        throws RTIcap.AttributeNotDefined, RTIcap.AttributeNotPublished,
               RTIcap.FederateNotExecutionMember,
               RTIcap.FederateOwnsAttributes, RTIcap.ObjectClassNotPublished, 
               RTIcap.ObjectNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       ownManager.attributeOwnershipAcquisition(object,attributes,tag,FederateId);
    }// end of attributeOwnershipAcquisition()

    /**
    */
    public RTIcap._Boolean 
    isAttributeOwnedByFederate(int obj, short attrib)
        throws RTIcap.AttributeNotDefined, RTIcap.ConcurrentAccessAttempted, 
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectNotKnown, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       return ownManager.isAttributeOwnedByFederate(obj,attrib,FederateId);

    }// end of isAttributeOwnedByFederate()

    /**
    */
    public void 
    cancelAttributeOwnershipAcquisition(int object, short[] attributes)
        throws RTIcap.AttributeAlreadyOwned, RTIcap.AttributeNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectNotKnown, 
               RTIcap.AttributeAcquisitionWasNotRequested, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
       if( ownManager == null ) throw new RTIcap.RTIinternalError();
       ownManager.cancelAttributeOwnershipAcquisition(object,attributes,FederateId);
    }// end of cancelAttributeOwnershipAcquisition()

    /**********************/
    /**** TIME MANAGAMENT */
    /**********************/

    public RTIcap._Boolean tick() throws RTIcap.RTIinternalError
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       return timManager.tick(FederateId);
    }//

    public RTIcap._Boolean tickMinMax(double minimum, double maximum)
        throws RTIcap.RTIinternalError
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       return timManager.tickMinMax(minimum,maximum,FederateId);
    }


    /**
    */
    public double 
    queryLBTS( )
           throws RTIcap.FederateNotExecutionMember, 
                  RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                  RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       return timManager.queryFederateTime(FederateId);

    }// end of queryLBTS()

    /**
    */
    public double 
    queryFederateTime( )
           throws RTIcap.FederateNotExecutionMember, 
                  RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                  RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       return timManager.queryFederateTime(FederateId);

    }// end of queryFederateTime()

    /**
    */
    public double 
    queryMinNextEventTime()
           throws RTIcap.FederateNotExecutionMember, 
                  RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                  RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       return timManager.queryMinNextEventTime(FederateId);

    }// end of queryMinNextEventTime()

    /**
    */
    public void 
    modifyLookahead(double theLookahead)
         throws RTIcap.FederateNotExecutionMember, 
                RTIcap.InvalidLookahead, 
                RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.modifyLookahead(theLookahead,FederateId);

    }// end of modifyLookahead()

    /**
    */
    public double 
    queryLookahead()
           throws RTIcap.FederateNotExecutionMember, 
                  RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
                  RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       return timManager.queryLookahead(FederateId);

    }// end of queryLookahead()

    /**
    */
    public
    void timeAdvanceRequest(double theTime)
         throws RTIcap.TimeAdvanceAlreadyInProgress, 
                RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
                RTIcap.RestoreInProgress, RTIcap.EnableTimeConstrainedPending, 
                RTIcap.EnableTimeRegulationPending, RTIcap.RTIinternalError 
    {
       /*try
       {
         fedAmbassador.timeAdvanceGrant(theTime+1);
       } catch(RTIcap.FederateInternalError fie) {}
         catch(RTIcap.TimeAdvanceWasNotInProgress tawnip) {}
       */

       //if(DEBUG) System.out.println(" in timeAdvanceRequest for "+FederateId);
       timManager.timeAdvanceRequest(theTime,FederateId);

       /*try
       {
       fedAmbassador.initiatePause("Hello");
       } catch(RTIcap.FederateInternalError fie) {}
         catch(RTIcap.FederateAlreadyPaused tawnip) {}
       */

    }// end of timeAdvanceRequest()

    /**
    */
    public void 
    timeAdvanceRequestAvailable(double theTime)
        throws RTIcap.EnableTimeConstrainedPending, 
               RTIcap.EnableTimeRegulationPending, 
               RTIcap.FederationTimeAlreadyPassed, 
               RTIcap.FederateNotExecutionMember, RTIcap.InvalidFederationTime, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress, RTIcap.TimeAdvanceAlreadyInProgress
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.timeAdvanceRequestAvailable(theTime,FederateId);

    }// end of timeAdvanceRequestAvailable()

    /**
    */
    public void 
    nextEventRequest(double theTime)
         throws RTIcap.TimeAdvanceAlreadyInProgress, 
                RTIcap.FederationTimeAlreadyPassed, 
                RTIcap.EnableTimeConstrainedPending, 
                RTIcap.InvalidFederationTime, RTIcap.FederateNotExecutionMember, 
                RTIcap.RestoreInProgress, RTIcap.SaveInProgress, 
                RTIcap.EnableTimeRegulationPending, 
                RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.nextEventRequest(theTime,FederateId);

    }// end of nextEventRequest()

    /**
    */
    public void 
    nextEventRequestAvailable(double theTime)
        throws RTIcap.EnableTimeConstrainedPending, 
               RTIcap.EnableTimeRegulationPending,
               RTIcap.FederateNotExecutionMember, RTIcap.InvalidFederationTime, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.FederationTimeAlreadyPassed, 
               RTIcap.SaveInProgress, RTIcap.TimeAdvanceAlreadyInProgress
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.nextEventRequestAvailable(theTime,FederateId);

    }// end of nextEventRequestAvailable()

    /**
    */
    public void 
    flushQueueRequest(double theTime)
         throws RTIcap.TimeAdvanceAlreadyInProgress, 
                RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.RestoreInProgress, 
                RTIcap.EnableTimeConstrainedPending, 
                RTIcap.SaveInProgress, 
                RTIcap.EnableTimeRegulationPending, 
                RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.flushQueueRequest(theTime,FederateId);

    }// end of flushQueueRequest()
    
    /**
    */
    public void 
    enableTimeRegulation(double theTime, double lookahead)
        throws RTIcap.ConcurrentAccessAttempted,
               RTIcap.EnableTimeRegulationPending, 
               RTIcap.FederateNotExecutionMember, 
               RTIcap.InvalidFederationTime, RTIcap.InvalidLookahead, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError,
               RTIcap.SaveInProgress, RTIcap.TimeAdvanceAlreadyInProgress,
               RTIcap.TimeRegulationAlreadyEnabled
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.enableTimeRegulation(theTime,lookahead,FederateId);

    }// end of enableTimeRegulation()

    /**
    */
    public void 
    disableTimeRegulation()
        throws RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress, 
               RTIcap.TimeRegulationWasNotEnabled
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.disableTimeRegulation(FederateId);

    }// end of disableTimeRegulation()

    /**
    */
    public void 
    enableTimeConstrained()
        throws RTIcap.EnableTimeConstrainedPending, 
               RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress, 
               RTIcap.TimeAdvanceAlreadyInProgress,
               RTIcap.TimeConstrainedAlreadyEnabled
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.enableTimeConstrained(FederateId);

    }// end of enableTimeConstrained()

    /**
    */
    public void 
    disableTimeConstrained()
        throws RTIcap.FederateNotExecutionMember, 
               RTIcap.RestoreInProgress, RTIcap.SaveInProgress,
               RTIcap.TimeConstrainedWasNotEnabled,
               RTIcap.RTIinternalError
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.disableTimeConstrained(FederateId);

    }// end o fdisableTimeConstrained()

    /**
    */
    public void 
    enableAsynchronousDelivery()
        throws RTIcap.AsynchronousDeliveryAlreadyEnabled, 
               RTIcap.ConcurrentAccessAttempted, 
               RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.enableAsynchronousDelivery(FederateId);

    }// end of enableAsynchronousDelivery()

    /**
    */
    public void 
    disableAsynchronousDelivery()
        throws RTIcap.AsynchronousDeliveryAlreadyDisabled,
               RTIcap.ConcurrentAccessAttempted, 
               RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.disableAsynchronousDelivery(FederateId);

    }// end of disableAsynchronousDelivery()

    /**
    */
    public void 
    changeInteractionOrderType(short theClass, int theType)
         throws RTIcap.InteractionClassNotPublished, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.SaveInProgress, 
                RTIcap.InvalidOrderingHandle, 
                RTIcap.InteractionClassNotKnown, 
                RTIcap.RestoreInProgress, 
                RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.changeInteractionOrderType(theClass,theType,FederateId);

    }// end of changeInteractionOrderType()

    /**
    */
    public void 
    changeAttributeOrderType(int theClass, short[] theAttributes, int theType)
         throws RTIcap.FederateNotExecutionMember, 
                RTIcap.AttributeNotDefined, 
                RTIcap.AttributeNotOwned, 
                RTIcap.SaveInProgress, 
                RTIcap.InvalidOrderingHandle, 
                RTIcap.RestoreInProgress, 
                RTIcap.ObjectNotKnown, 
                RTIcap.RTIinternalError 
    {
       if( timManager == null ) throw new RTIcap.RTIinternalError();
       timManager.changeAttributeOrderType(theClass,theAttributes,theType,FederateId);

    }// end of changeInteractionOrderType()

    /*********************************/
    /*** DataDistribution Management */
    /*********************************/

    /**
    public
    void createUpdateRegion(int theSpace, RTIcap.Range[][] theExtents, int theRegion)
         throws RTIcap.SpaceNotDefined, RTIcap.InvalidExtents, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }
    */
    /**
    public
    void createSubscriptionRegion(String theSpace, RTIcap.Range[][] theExtents, int theRegion)
         throws RTIcap.SpaceNotDefined, RTIcap.InvalidExtents, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }
    */

    /**
    */
    public
    void associateUpdateRegion1(int theRegion, int theObject, short[] theAttributes)
         throws RTIcap.RegionNotKnown, RTIcap.ObjectNotKnown, 
                RTIcap.AttributeNotDefined, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void disassociateUpdateRegion1(int theRegion, int theObject, short[] theAttributes)
         throws RTIcap.RegionNotKnown, RTIcap.ObjectNotKnown, 
                RTIcap.AttributeNotDefined, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }
    /**
    */
    public
    void associateUpdateRegion2(int theRegion, short theClass)
         throws RTIcap.RegionNotKnown, RTIcap.InteractionClassNotKnown, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }
    /**
    */
    public
    void disassociateUpdateRegion2(int theRegion, short theClass)
         throws RTIcap.RegionNotKnown, RTIcap.InteractionClassNotKnown, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public void 
    notifyAboutRegionModification(RTIcap.Region theRegion)
         throws RTIcap.RegionNotKnown, RTIcap.InvalidExtents, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.RestoreInProgress, RTIcap.SaveInProgress, 
                RTIcap.RTIinternalError 
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.notifyAboutRegionModification(theRegion,FederateId);

    }// end of notifyAboutRegionModification()

    /**
    */
    public void 
    deleteRegion(RTIcap.Region theRegion)
         throws RTIcap.RegionNotKnown, RTIcap.FederateNotExecutionMember, 
                RTIcap.RestoreInProgress, RTIcap.SaveInProgress, 
                RTIcap.RTIinternalError 
                
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.deleteRegion(theRegion,FederateId);

    }// end of deleteRegion()

    /**
    */
    public RTIcap.Region 
    createRegion(int space, int numberOfExtents )
        throws RTIcap.FederateNotExecutionMember, RTIcap.InvalidExtents, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress, RTIcap.SpaceNotDefined
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        return datManager.createRegion(space,numberOfExtents,FederateId);
    }// end of createRegion()

    /**
    */
    public void 
    unassociateRegionForUpdates(RTIcap.Region theRegion, int object)
        throws RTIcap.FederateNotExecutionMember, 
               RTIcap.ObjectNotKnown, RTIcap.RegionNotKnown, 
               RTIcap.AttributeNotDefined, 
               RTIcap.RTIinternalError
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.unassociateRegionForUpdates(theRegion,object,FederateId);
    }// end of unassociateRegionForUpdates()

    /**
    */
    public void 
    associateRegionForUpdates(RTIcap.Region theRegion , int object, 
               short[] attributes)
        throws RTIcap.AttributeNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.InvalidRegionContext,
               RTIcap.ObjectNotKnown, RTIcap.RegionNotKnown, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.associateRegionForUpdates(theRegion,object,attributes,FederateId);
    }// end of associateRegionForUpdates()

    /**
    */
    public int 
    registerObjectInstanceWithRegionAndName(short _class, String theName,
                  short[] theAttributes, RTIcap.Region[] theRegions, 
                  int theNumberOfhandles)
        throws 
               RTIcap.FederateNotExecutionMember, 
               RTIcap.ObjectAlreadyRegistered, RTIcap.ObjectClassNotDefined,
               RTIcap.ObjectClassNotPublished, RTIcap.RegionNotKnown, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
       if( datManager == null ) throw new RTIcap.RTIinternalError();
       return datManager.registerObjectInstanceWithRegionAndName(_class,theName,theAttributes,theRegions,theNumberOfhandles,FederateId);
    }// end of registerObjectInstanceWithRegionAndName()

    /**
    */
    public int 
    registerObjectInstanceWithRegion(short _class, short[] theAttributes, 
                  RTIcap.Region[] theRegions, int theNumberOfhandles)
        throws 
               RTIcap.FederateNotExecutionMember, 
               RTIcap.ObjectClassNotDefined,
               RTIcap.ObjectClassNotPublished, RTIcap.RegionNotKnown, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
       if( datManager == null ) throw new RTIcap.RTIinternalError();
       return datManager.registerObjectInstanceWithRegion(_class,theAttributes,theRegions,theNumberOfhandles,FederateId);
    }// end of registerObjectInstanceWithRegionAndName()

    /**
    */
    public void 
    subscribeObjectClassAttributesWithRegion(short _class,
        RTIcap.Region theRegion, short[] theAttributes, RTIcap._Boolean active)
        throws RTIcap.AttributeNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectClassNotDefined,
               RTIcap.RegionNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.subscribeObjectClassAttributesWithRegion(_class,theRegion,theAttributes,active,FederateId);
    }// end of subscribeObjectClassAttributesWithRegion()

    /**
    */
    public void 
    unsubscribeObjectClassWithRegion(short _class, RTIcap.Region theRegion)
        throws RTIcap.FederateNotExecutionMember, RTIcap.ObjectClassNotDefined, 
               RTIcap.RegionNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.unsubscribeObjectClassWithRegion(_class,theRegion,FederateId);
    }// end of unsibscribeObjectClassWithRegion()

    /**
    */
    public void 
    subscribeInteractionClassWithRegion(short _class, RTIcap.Region theRegion, 
        RTIcap._Boolean active)
        throws RTIcap.FederateLoggingServiceCalls, 
               RTIcap.FederateNotExecutionMember, 
               RTIcap.InteractionClassNotDefined,
               RTIcap.RegionNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.subscribeInteractionClassWithRegion( _class,theRegion,active,FederateId);
    }// end of subscribeInteractionClassWithRegion()

    /**
    */
    public void 
    unsubscribeInteractionClassWithRegion(short _class, RTIcap.Region theRegion)
        throws RTIcap.FederateNotExecutionMember, 
               RTIcap.InteractionClassNotDefined,
               RTIcap.InteractionClassNotSubscribed, RTIcap.RegionNotKnown, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError,
               RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.unsubscribeInteractionClassWithRegion( _class,theRegion,FederateId);
    }// end of unsubscribeInteractionClassWithRegion()


    /**
    */
    public void 
    sendInteractionWithRegion(short interaction, 
        RTIcap.ParameterHandleValuePair[] parameters, String theTag, 
        RTIcap.Region theRegion)
        throws RTIcap.FederateNotExecutionMember, 
               RTIcap.InteractionClassNotDefined, 
               RTIcap.InteractionClassNotPublished,
               RTIcap.InteractionParameterNotDefined, RTIcap.RegionNotKnown,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.sendInteractionWithRegion( interaction,parameters,theTag,
                                              theRegion,FederateId);
    }// end of sendInteractionWithRegion()

    /**
    */
    public RTIcap.EventRetractionHandle 
    sendInteractionWithRegionAndTime(short interaction,
        RTIcap.ParameterHandleValuePair[] parameters, double theTime, 
        String theTag, RTIcap.Region theRegion )
        throws RTIcap.FederateNotExecutionMember, 
               RTIcap.InteractionClassNotDefined, 
               RTIcap.InteractionClassNotPublished, 
               RTIcap.InvalidFederationTime,
               RTIcap.InteractionParameterNotDefined, RTIcap.RegionNotKnown,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        return datManager.sendInteractionWithRegionAndTime(interaction,parameters,theTime,theTag,theRegion,FederateId);
    }// end of sendInteractionWithRegionAndTime()

    /**
    */
    public void 
    requestClassAttributeValueUpdateWithRegion(short _class, short[] attributes,        RTIcap.Region theRegion )
        throws RTIcap.AttributeNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectClassNotDefined,
               RTIcap.RegionNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.requestClassAttributeValueUpdateWithRegion(_class,attributes,theRegion,FederateId);
    }// end of requestClassAttributeValueUpdateWithRegion()

    /**
    */
    public void 
    requestObjectAttributeValueUpdateWithRegion(int object, short[] attributes,
        RTIcap.Region theRegion )
        throws RTIcap.AttributeNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.ObjectNotKnown,
               RTIcap.RegionNotKnown, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
        if( datManager == null ) throw new RTIcap.RTIinternalError();
        datManager.requestObjectAttributeValueUpdateWithRegion(object,attributes,theRegion,FederateId);
    }// end of requestObjectAttributeValueUpdateWithRegion()

    /*************************/
    /** RTI Support Services */
    /*************************/

    /**
    */
    public short 
    getObjectClassHandle(String theName)
          throws RTIcap.NameNotFound, RTIcap.FederateNotExecutionMember, 
                 RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getObjectClassHandle(theName);
    }// end of getObjectClassHandle()

    /**
    */
    public String 
    getObjectClassName(short theHandle)
           throws RTIcap.ObjectClassNotDefined, 
                  RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getObjectClassName(theHandle);
    }// end of getObjectClassName() 

    /**
    */
    public short 
    getAttributeHandle(String theName, short whichClass)
          throws RTIcap.ObjectClassNotDefined, RTIcap.NameNotFound, 
                 RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getAttributeHandle(theName,whichClass);
    }// end of getAttributeHandle()

    /**
    */
    public String 
    getAttributeName(short theHandle, short whichClass)
           throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                  RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getAttributeName(theHandle,whichClass);
    }// end of getAttributeName()

    /**
    */
    public short 
    getInteractionClassHandle(String theName)
          throws RTIcap.NameNotFound, RTIcap.FederateNotExecutionMember, 
                 RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getInteractionClassHandle(theName);

    }// end of getInteractionClassHandle()

    /**
    */
    public String 
    getInteractionClassName(short theHandle)
           throws RTIcap.InteractionClassNotDefined, 
                  RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getInteractionClassName(theHandle);
    }

    /**
    */
    public short 
    getParameterHandle(String theName, short whichClass)
          throws RTIcap.InteractionClassNotDefined, RTIcap.NameNotFound, 
                 RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getParameterHandle(theName,whichClass);
    }

    /**
    */
    public String 
    getParameterName(short theHandle, short whichClass)
           throws RTIcap.InteractionClassNotDefined, 
                  RTIcap.InteractionParameterNotDefined, 
                  RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getParameterName(theHandle,whichClass);
    }

    /**
    */
    public int 
    getSpaceHandle(String theName)
        throws RTIcap.NameNotFound, RTIcap.FederateNotExecutionMember, 
               RTIcap.RTIinternalError 
               
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getSpaceHandle(theName);
    }

    /**
    */
    public String 
    getSpaceName(int theHandle)
           throws RTIcap.SpaceNotDefined, RTIcap.FederateNotExecutionMember, 
                  RTIcap.RTIinternalError 
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getSpaceName(theHandle);
    }

    /**
    */
    public
    short getFederateHandle(String theName)
          throws RTIcap.FederateDoesNotExist, RTIcap.FederateNotExecutionMember, 
                 RTIcap.ConcurrentAccessAttempted, 
                 RTIcap.RTIinternalError, RTIcap.Deadlock, 
                 RTIcap.UnimplementedService
    {
        short ret=0;
        
        return ret;
    }

    /**
    */
    public
    String getFederateName(short theHandle)
           throws RTIcap.FederateDoesNotExist, 
                  RTIcap.FederateNotExecutionMember, 
                  RTIcap.ConcurrentAccessAttempted, 
                  RTIcap.RTIinternalError, RTIcap.Deadlock, 
                  RTIcap.UnimplementedService
    {
       String ret = null;

       return ret;
    }

    /**
    */
    public
    void turnRegulationOn()
         throws RTIcap.FederationTimeAlreadyPassed, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }
 

    /**
    */
    public
    double turnRegulationOnNow()
           throws RTIcap.FederateNotExecutionMember, 
                  RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                  RTIcap.Deadlock
    {
       double ret=0;

       return ret;
    }

    /**
    */
    public
    void turnRegulationOff()
         throws RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    */
    public
    void setTimeConstrained(RTIcap._Boolean state)
         throws RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    */
    public int 
    getObjectInstanceHandle(String theName)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateNotExecutionMember,
               RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getObjectInstanceHandle(theName);
    }// end of getObjectInstanceHandle()

    /**
    */
    public String 
    getObjectInstanceName(int theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateNotExecutionMember,
               RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getObjectInstanceName(theHandle);
    }// end of getObjectInstanceName() 

    /**
    */
    public int 
    getDimensionHandle(String theName, int theHandle)
        throws RTIcap.SpaceNotDefined, RTIcap.NameNotFound,
               RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getDimensionHandle(theName,theHandle);
    }// end of getDimensionHandle()

    /**
    */
    public String 
    getDimensionName(int theDimHandle, int theSpcHandle)
        throws RTIcap.SpaceNotDefined, RTIcap.DimensionNotDefined,
               RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getDimensionName(theDimHandle,theSpcHandle);
    }// end of getDimensionName()

    /**
    */
    public int 
    getAttributeRoutingSpaceHandle(short theHandle, short whichClass)
        throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined,
               RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getAttributeRoutingSpaceHandle(theHandle,whichClass);
    }// end of getAttributeRoutingSpaceHandle()

    /**
    */
    public short 
    getObjectClass(int theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateNotExecutionMember,
               RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getObjectClass(theHandle);
    }// end of getObjectClass()

    /**
    */
    public int 
    getInteractionRoutingSpaceHandle(short theHandle)
        throws RTIcap.InteractionClassNotDefined, 
               RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getInteractionRoutingSpaceHandle(theHandle);
    }// end of getInteractionRoutingSpaceHandle()

    /**
    */
    public int 
    getTransportationHandle(String theName)
        throws RTIcap.NameNotFound, RTIcap.FederateNotExecutionMember,
               RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getTransportationHandle(theName);
    }// end of getTransportationHandle()

    /**
    */
    public String 
    getTransportationName(int theHandle)
        throws RTIcap.InvalidTransportationHandle,
               RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getTransportationName(theHandle);

    }// end of getTransportationName()

    /**
    */
    public int 
    getOrderingHandle(String theName)
        throws RTIcap.NameNotFound, RTIcap.FederateNotExecutionMember,
               RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getOrderingHandle(theName);

    }// end of getOrderingHandle()

    /**
    */
    public String 
    getOrderingName(int theHandle)
        throws RTIcap.InvalidOrderingHandle, RTIcap.FederateNotExecutionMember,
               RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       return rtiServices.getOrderingName(theHandle);
    }// end of getOrderingName()

    /**
    */
    public void 
    enableClassRelevanceAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.enableClassRelevanceAdvisorySwitch();
    }// end of enableClassRelevanceAdvisorySwitch()

    /**
    */
    public void 
    disableClassRelevanceAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.disableClassRelevanceAdvisorySwitch();
    }// end of disableClassRelevanceAdvisorySwitch()

    /**
    */
    public void 
    enableAttributeRelevanceAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.enableAttributeRelevanceAdvisorySwitch();
    }// end of enableAttributeRelevanceAdvisorySwitch()

    /**
    */
    public void 
    disableAttributeRelevanceAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.disableAttributeRelevanceAdvisorySwitch();
    }// end of disableAttributeRelevanceAdvisorySwitch()

    /**
    */
    public void 
    enableAttributeScopeAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.enableAttributeScopeAdvisorySwitch();
    }// end of enableAttributeScopeAdvisorySwitch()

    /**
    */
    public void 
    disableAttributeScopeAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.disableAttributeScopeAdvisorySwitch();
    }// end of disableAttributeScopeAdvisorySwitch()

    /**
    */
    public void 
    enableInteractionRelevanceAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.enableInteractionRelevanceAdvisorySwitch();
    }// end of enableInteractionRelevanceAdvisorySwitch()

    /**
    */
    public void 
    disableInteractionRelevanceAdvisorySwitch()
        throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(rtiServices==null) throw new RTIcap.RTIinternalError();
       rtiServices.disableInteractionRelevanceAdvisorySwitch();
    }// end of disableInteractionRelevanceAdvisorySwitch()

}// end of RTIAmb

