/**
*
*  Whenever RTIKernelStartsUp it will serve the location of other Manager
*  objects os that RTIAmbassador object can work with them directly.
*  This also allows us to start these managers on different machines.
*
* @date April 1998 (last updated on May 07, 1998)
* @author H. Timucin Ozdemir
*
*/
package owrti;

import RTIcap.*;
import npacrti.*;

import java.util.*;

import org.omg.CORBA.*;

/*
*/
public class 
RTIKernel  extends _RTIKernelInterfaceImplBase implements RTIDebug
{

    DeclarationManager      decManager ;
    ObjectManager           objManager ;
    OwnershipManager        ownManager ;
    TimeManager             timManager ;
    DataDistributionManager datManager ;
    RTIServices             rtiServices ;
    ObjectClassNaming       objNaming;
    InteractionNaming       interNaming;

    Hashtable FEDExecutions  = new Hashtable();
    Hashtable fedIdTOFedName = new Hashtable();

    /**
    */
    public RTIKernel() { }// end of RTIKernel

    /**
    */
    public FederationManagerInterface 
    getFederationManager(String executionName) 
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getFederationManager();

    }// end of getFederationManager()

    /**
    */
    public DeclarationManagerInterface 
    getDeclarationManager(String executionName) 
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getDeclarationManager();

    }// end of getDeclarationManager()


    /**
    */
    public ObjectManagerInterface 
    getObjectManager(String executionName) 
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getObjectManager();

    }// end of getObjectManager()


    /**
    */
    public OwnershipManagerInterface 
    getOwnershipManager(String executionName)
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getOwnershipManager();

    }// end of getOwnershipManager()


    /**
    */
    public DataDistributionManagerInterface 
    getDataDistributionManager(String executionName)
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getDataDistributionManager();

    }// end of getDataDistributionManager()


    /**
    */
    public TimeManagerInterface 
    getTimeManager(String executionName) 
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getTimeManager();

    }// end of getTimeManager()


    /**
    */
    public RTIServicesInterface 
    getRTIServices(String executionName) 
           throws RTIcap.FederationExecutionDoesNotExist
    { 
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();

        return fExe.getRTIServices();

    }// end of getRTIServices()


    /**
    */
    public String 
    getVersion() { return "RTI Kernel 0.0.1 for spec HLA 1.3 " ; }

    /**
    *  Defines a new RTIambassador object.
    *  It is better we keep track of these objects so that we can get rid
    *  of them when federate resigns.
    */
    public RTIcap.RTIambassador 
    createRTIAmbassador() 
    { 
       if(DEBUG) System.out.println(" in createRTIAmbassador() ");
       RTIcap.RTIambassador ret = new RTIAmb(this);

       // Create the servant and register it with the ORB
       org.omg.CORBA.ORB.init().connect(ret);

       if(DEBUG) 
          System.out.println(" in ret : "+org.omg.CORBA.ORB.init().object_to_string(ret));
       
       return ret;

    }// end of createRTIAmbassador()

    /**
    * 4.2
    */
    public
    void createFederationExecution(String executionName, String FED )
         throws RTIcap.FederationExecutionAlreadyExists, RTIcap.RTIinternalError
    {

        if(DEBUG) System.out.println(" in createFederationExecution() for "+executionName);

        /** If the federation execution exists, then throw */

        if(getFederationExecution(executionName)!=null) 
        { 
             if(DEBUG) System.out.println(" FederationExecutionAlreadyExists for "+executionName);
             throw new RTIcap.FederationExecutionAlreadyExists();
        }
        
        /** Obviously, it is not. */

        FederationExecution fExe = new FederationExecution(executionName,this);
        
        FEDExecutions.put(executionName,fExe); 
       
    }// end of createFederationExecution()

    /**
    * 4.3
    */
    public void 
    destroyFederationExecution(String executionName)
         throws RTIcap.FederatesCurrentlyJoined, 
                RTIcap.FederationExecutionDoesNotExist, RTIcap.RTIinternalError 
    {
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) throw new RTIcap.FederationExecutionDoesNotExist();
  
        if(fExe.destroyExecution()) FEDExecutions.remove(executionName); 
        
    }// end of destroyFederationExecution() 

    //************************/
    //** Propriatery Methods */
    //************************/

    /**
    * Returns null : if the federation does not exist <br>
    *         valid FederationExecution : if the federation exec exists.
    */
    FederationExecution
    getFederationExecution(String FEDName)
    {
       return (FederationExecution) FEDExecutions.get(FEDName);

    }// end of getFedederationExecution()

}// end of RTIKernel
