/**
*/
package owrti;

import RTIcap.*;
import java.util.*;


/**
*
* @date April 20, 1998 ( Updated at April 22, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
RegionInfo implements RTIDebug
{

   DimensionInfo[] dims ;
   SpaceInfo r_space;
   Hashtable subscribers = new Hashtable();

   Hashtable active_subs  = new Hashtable();
   Hashtable passive_subs = new Hashtable();

   RTIcap.Region regionDefinition;

   short  owner;

   int    theRegionHandle;
   String theName;

   /**
   */
   public RegionInfo() { owner = -1; }// end of CONSTRUCTOR

   /**
   */
   public
   RegionInfo( RTIcap.Region _rgn, short _FederateId ) 
   {
       regionDefinition = _rgn;
       owner = _FederateId;

   }// end of CONSTRUCTOR

   /**
   */
   public void
   setHandle(int _nwRegionId ) { theRegionHandle = _nwRegionId; }

   /**
   */
   public int 
   getHandle() { return theRegionHandle; }

   /**
   */
   public int
   getRoutingSpaceHandle() { return regionDefinition.theSpace; }

   /**
   */
   public String 
   getName() { return theName; }

   /**
   */
   public void
   modify( RTIcap.Region _nwrgn )
   {
      regionDefinition = _nwrgn;

   }// end of modify()

   /**
   * Registers federate as a subscriber to this region.
   */
   public synchronized void
   register_subscriber(short theFederate, FederateInfo fedInf)
   {
      FederateInfo rec = (FederateInfo) subscribers.get(new Short(theFederate));
      if(rec==null) subscribers.put(new Short(theFederate),fedInf);

   }// end of register_subscriber()

   /**
   * Registers federate as a subscriber to this region.
   */
   public synchronized void
   register_subscriber(FederateInfo fedInf)
   {
      FederateInfo rec = 
              (FederateInfo) subscribers.get(new Short(fedInf.getFederateID()));
      if(rec==null) 
         subscribers.put(new Short(fedInf.getFederateID()),fedInf);

   }// end of register_subscriber()

   /**
   * cancels federate's subscription to this region.
   */
   public synchronized void
   deregister_subscriber(short theFederate, FederateInfo fedInf)
   {
      FederateInfo rec = (FederateInfo) subscribers.get(new Short(theFederate));
      if(rec!=null) subscribers.remove(new Short(theFederate));

   }// end of register_subscriber()

   /**
   * cancels federate's subscription to this region.
   */
   public synchronized void
   deregister_subscriber(short theFederate)
   {
      FederateInfo rec = (FederateInfo) subscribers.get(new Short(theFederate));
      if(rec!=null) subscribers.remove(new Short(theFederate));

   }// end of register_subscriber()

   /**
   * @return gets all subscribers in this region.
   */
   public synchronized Enumeration
   getSubscribers() { return subscribers.elements(); }

   /**
   * @return gets all the extents defined in this region.
   */
   public RTIcap.Extent[]
   getExtents() 
   { 
      return (regionDefinition!=null?regionDefinition.extents:null);

   }// end of getExtents()

   /**
   * @param DimHandle Dimension handle
   * @return null if there is no extent definition for the dimension.
   *         Otherwise, returns extent definition for the given dimension.
   */
   public RTIcap.Extent
   getDimensionExtent(int DimHandle) 
   { 
     
      RTIcap.Extent ret = null;
      
      if(regionDefinition!=null)
      { 
         RTIcap.Extent[] myextents = regionDefinition.extents;
         boolean stay=true;

         for(int i=0;stay&&i<myextents.length;i++)
            if( myextents[i].theDimension == DimHandle )
            {
                ret = new RTIcap.Extent(myextents[i].theDimension,
                                        myextents[i].lowerBound,
                                        myextents[i].upperBound);
                stay = false;

            }// end of for

      }// end of if

      return ret;

   }// end of getExtents()

   /**
   *
   * We have to compare them according to their Dimension values.
   * If they do not have overlapping Dimension, then they do not intersect.
   *
   * <p><ul>
   * <p><li>For each dimension in this Region
   *   <ul>
   *   <p><li> get the other region's Extent information
   *   <p><li> If it exists and overlaps with this region's extent set, then
   *           these two region can intersect.
   *   </ul>
   * </ul>
   */
   public boolean
   canIntersect(RegionInfo reg)
   {
      boolean ret = false;
      int i=0;
      RTIcap.Extent[] exts = getExtents();
      RTIcap.Extent reg_extns;

      
      if(exts!=null)
       while( !ret && i < exts.length )
       {
          reg_extns = reg.getDimensionExtent(exts[i].theDimension);

          if( reg_extns!= null && 
              ( ( reg_extns.lowerBound>=exts[i].lowerBound &&
                  reg_extns.lowerBound<=exts[i].upperBound    ) ||
                ( reg_extns.upperBound>=exts[i].lowerBound &&
                  reg_extns.upperBound<=exts[i].upperBound    ) ) ) 
              ret = true;
          else 
              i++;

       }// end of while 

      return ret;

   }// end of canIntersect()

}// end of RegionInfo

