/**
*/
package owrti;

import RTIcap.*;
import java.util.*;


/**
*
* @date April 20, 1998 ( Updated at May 02, 1998)
* @author H. Timucin Ozdemir
*
*/
public class 
SpaceInfo implements RTIDebug
{

   short  theHandle;
   String theName;
   Vector dims;
   Vector regions;

   RegionNaming regNaming; 

   Hashtable idToRegions = new Hashtable();

   DimensionNaming dimNaming;

   /**
   */
   public
   SpaceInfo()
   { dimNaming = new DimensionNaming();
   }// end of CONSTRUCTOR

   /**
   */
   public
   SpaceInfo( short _theHandle, String _theName, DimensionInfo[] _dims,
              RegionNaming _regNaming)
   {

      theHandle = _theHandle;
      theName   = _theName;
      regNaming = _regNaming;

      dimNaming = new DimensionNaming();
      for(int i=0;i<_dims.length;i++) dimNaming.addDimensionInfo(_dims[i]);

   }// end of CONSTRUCTOR

   /**
   */
   public short 
   getHandle() { return theHandle; }

   /**
   */
   public String 
   getName() { return theName; }

   /**
   * Default region spans all the routing space.
   */
   public RegionInfo
   getDefaultRegion()
   {
      RegionInfo ret = null ;

      if( dimNaming.getSize() > 0 )
      {
         RTIcap.Extent[] extents = new RTIcap.Extent[dimNaming.getSize()];
         Enumeration e = dimNaming.getKeys();
         DimensionInfo dInf;
         Integer handle;
         int i = 0;

         for(;e.hasMoreElements();)
         {
            handle = null;
            dInf   = null;
            handle = (Integer) e.nextElement();
            
            try
            {
               if( handle != null ) 
                   dInf = dimNaming.getDimensionInfo(handle.intValue()); 

               if( dInf != null ) 
                   extents[i++] = new Extent( dInf.getHandle(), 
                                              dInf.getLowerBound(),
                                              dInf.getUpperBound() );

            } catch( RTIcap.NameNotFound nnf) {}

         }// end of for

      } else ret = new RegionInfo();

      return ret;

   }// end of getDefaultRegion()

   public void
   addRegion( int theHandle ) { regions.addElement( new Integer(theHandle)); }

   public void
   addRegionInfo( RegionInfo theHandle ) 
   { //regions.addElement( new Integer(theHandle)); 
   }

   /**
   */
   public void
   removeRegion( int theRegion ) 
   { regions.removeElement(new Integer(theRegion)); }

   /**
   */
   public void
   removeRegion( RTIcap.Region theRegion ) 
   { //regions.removeElement(new Integer(theRegion)); 
   }

   /**
   */
   public RegionInfo
   getRegionInfo( int theRegion) {  return regNaming.getRegionInfo(theRegion); }

   /**
   */
   public final int 
   getDimensionHandle( String theName ) throws RTIcap.NameNotFound
   {
      return dimNaming.getDimensionHandle(theName);
   }// end of getDimensionHandle()

   /**
   */
   public final String 
   getDimensionName( int theHandle ) throws RTIcap.DimensionNotDefined
   {
      return dimNaming.getDimensionName(theHandle);
   }// end of getDimensionName()

   /**
   */
   public Enumeration
   getFederatesIntersectWith( RegionInfo theRegion) 
   {  
      Vector ret = new Vector();
      Enumeration e2 ;
      RegionInfo  regI, regJ;
      Enumeration handles = regions.elements();
      int[] theHandles = new int[regions.size()];
      Integer val;
      int number_of_regions=0;
      int i,j;
      FederateInfo n;

      for(;handles.hasMoreElements();) 
      {
         val = (Integer) handles.nextElement();
         if(val!=null) theHandles[number_of_regions++]=val.intValue(); 

      }// end of for

      for(i=0;i<regions.size();i++)
      {
         regI = getRegionInfo(theHandles[i]);

         for(j=i+1;j<regions.size();j++)
         {
            regJ = getRegionInfo(theHandles[j]);
            if(regI.canIntersect(regJ)) 
            {
               e2 = regJ.getSubscribers();
               for(;e2.hasMoreElements();)
               {
                  n = (FederateInfo) e2.nextElement();
                  if(n!=null) ret.addElement(n);
               }// end of for 
               
            }// end of if

         }// end of for

      }// end of for

      return ret.elements();

   }// end of getFederatesIntersectWith()

}// end of SpaceInfo



