package owrti;

import RTIcap.*;
import java.util.*;

/**
* SpaceNaming keeps RoutingSpace informations and Region Definitions.
* Region can be accessed globally(not with respect to Routing Space)
*
* @date May 02, 1998 ( Update at May 02, 1998 )
* @author H. Timucin Ozdemir
*/
public class 
SpaceNaming extends GlbNaming implements RTIDebug
{

   Hashtable idToName = new Hashtable();
   Hashtable nameToId = new Hashtable();
   Hashtable idToSpaceInfo = new Hashtable();

   Hashtable idToRegionInfo = new Hashtable();

   RegionNaming regionNaming = new RegionNaming();

   /**
   */
   public 
   SpaceNaming() {}

   /**
   */
   public int 
   getSpaceHandle( String theName ) throws RTIcap.NameNotFound
   {
       Integer ret = (Integer) getID(theName);

       if ( ret==null) throw new RTIcap.NameNotFound();

       return ret.intValue();

   }// end of getSpaceHandle()

   /**
   */
   public String 
   getSpaceName(int theHandle) throws RTIcap.SpaceNotDefined
   {
       String ret = getName(new Integer(theHandle));
       if ( ret == null ) throw new RTIcap.SpaceNotDefined();
       return ret;

   }// end of getSpaceName()

   /**
   */
   public SpaceInfo 
   getSpaceInfo( int theHandle ) throws RTIcap.SpaceNotDefined
   {
       SpaceInfo ret = (SpaceInfo) getRecord(new Integer(theHandle));
       if( ret == null ) throw new RTIcap.SpaceNotDefined();
       return ret;

   }// end of getSpaceInfo()

   /**
   */
   public SpaceInfo 
   getSpaceInfo( String theName ) 
                 throws RTIcap.SpaceNotDefined, RTIcap.NameNotFound
   {
       Integer hndl = (Integer) getID(theName);
       if( hndl == null ) throw new RTIcap.SpaceNotDefined();
       return getSpaceInfo( hndl.intValue() );

   }// end of getSpaceInfo()

   /**
   */
   public void
   addSpaceInfo(SpaceInfo rec)
   {  addEntry(new Integer(rec.getHandle()),rec.getName(),rec); 
   }// end of addSpaceInfo()

   /**
   * @param theDimName Dimension Name
   * @param theSpcHandle Routing Space Handle
   * @return The Handle of Dimension in given Routing Space
   */
   public int 
   getDimensionHandle( String theDimName, int theSpcHandle )
          throws RTIcap.SpaceNotDefined, RTIcap.NameNotFound
   {
       SpaceInfo spInf = getSpaceInfo(theSpcHandle);
       if( spInf == null ) throw new RTIcap.SpaceNotDefined();
   
       return spInf.getDimensionHandle(theDimName);

   }// end of getDimensionHandle()

   /**
   * @param theDimHandle Dimension Handle
   * @param theSpcHandle Routing Space Handle
   * @return The Name of Dimension in given Routing Space
   */
   public String 
   getDimensionName( int theDimHandle, int theSpcHandle )
          throws RTIcap.SpaceNotDefined, RTIcap.DimensionNotDefined 
   {
       SpaceInfo spInf = getSpaceInfo(theSpcHandle);
       if( spInf == null ) throw new RTIcap.SpaceNotDefined();
   
       return spInf.getDimensionName(theDimHandle);

   }// end of getDimensionName()

   /**
   */
   public SpaceInfo 
   getSpaceInfoForRegion(int theRegion) throws RTIcap.RegionNotKnown 
   {
       RegionInfo reg = (RegionInfo) idToRegionInfo.get(new Integer(theRegion));
       if ( reg == null ) throw new RTIcap.RegionNotKnown();

       return (SpaceInfo) idToSpaceInfo.get(new Integer(reg.getRoutingSpaceHandle()));

   }// end of getSpaceInfo()

   /****************************/
   /** Region Related Services */
   /****************************/

   /**
   */
   public RegionInfo 
   getRegionInfo(int theRegion)
   {
       return (RegionInfo) regionNaming.get(theRegion);

   }// end of getRegionInfo()

   /**
   */
   public RegionInfo 
   getRegionInfo(RTIcap.Region theRegion)
   {
       RegionInfo ret = null;
       return ret;

   }// end of getRegionInfo()

   /**
   */
   public int 
   addRegionInfo(RegionInfo theRegion)
   {
       return regionNaming.addRegionInfo(theRegion);

   }// end of addRegionInfo()

   /**
   */
   public void 
   removeRegionInfo(int theRegionHandle)
   {
       regionNaming.remove(theRegionHandle);

   }// end of removeRegionInfo()

   /**
   */
   public void 
   removeRegionInfo( RegionInfo theRegion )
   {
       regionNaming.remove(theRegion);

   }// end of removeRegionInfo()

}// end of class SpaceNaming


