/**
* RTI 1.3
*/
package owrti;

import RTIcap.*;
import npacrti.*;

/*
*  Note: This object should know the time related info about federates in
*  the federation.
*  such as: Who has the minimum time stamp?
*  What is the Look ahead for each federate?
*   
*  @date April 17, 1998 ( Updated at May 07, 1998 )
*  @author H. Timucin Ozdemir
*/
public class 
TimeManager extends _TimeManagerInterfaceImplBase implements RTIDebug
{
    public static final int TIME_GRANTED   = 0;
    public static final int TIME_ADVANCING = 1;

    FederationExecution fedEx;
    
    /** Suggested data structures

    // Denotes the FederateID which has the minimum time value
    int minIndex; 

    // lookahed[i] denotes the lookahead value of Federate i
    double lookahed[] ; 

    // time[i] denotes the current time value of Federate i
    double time[] ; 

    // regulating[i] denotes whether Federate i is time regulating
    boolean regulating[] ; 

    // constrained[i] denotes whether Federate i is time constrained
    boolean constrained[] ; 

    // isActive[i] denotes whether Federate i is in the federation or not
    boolean isActive[] ; 

    */

    /**
    */
    public 
    TimeManager(FederationExecution _fedEx)
    {
        fedEx = _fedEx;
    }

    /**
    * 8.2
    */
    public void 
    enableTimeRegulation(double time, double lookahead, short theHandle)
        throws RTIcap.EnableTimeRegulationPending, 
               RTIcap.FederateNotExecutionMember, RTIcap.InvalidFederationTime,
               RTIcap.InvalidLookahead, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress, 
               RTIcap.TimeAdvanceAlreadyInProgress,
               RTIcap.TimeRegulationAlreadyEnabled
    {
       if(DEBUG_TIME) System.out.println(" IN enableTimeRegulation ");

       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.setTimeRegulation(time,lookahead);

       if(DEBUG_TIME) System.out.println(" Out of enableTimeRegulation ");

       // regulating[theHandle] = true;

    }// end of enableTimeRegulation()

    /**
    * 8.4
    */
    public void 
    disableTimeRegulation(short theHandle)
        throws RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError, RTIcap.SaveInProgress, 
               RTIcap.TimeRegulationWasNotEnabled
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.resetTimeRegulation();
       // regulating[theHandle] = false;

    }//  end of disableTimeRegulation()

    /**
    * 8.5
    */
    public void 
    enableTimeConstrained(short theHandle)
        throws RTIcap.EnableTimeConstrainedPending, 
               RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress, 
               RTIcap.TimeAdvanceAlreadyInProgress
    {
       if(DEBUG_TIME) System.out.println(" IN enableTimeConstrained ");

       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.setTimeConstrained();

       if(DEBUG_TIME) System.out.println(" Out of enableTimeConstrained ");
       // constrained[theHandle] = true;

    }// end of enableTimeConstrained()

    /**
    * 8.7
    */
    public void 
    disableTimeConstrained(short theHandle)
        throws RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress, 
               RTIcap.SaveInProgress, RTIcap.TimeConstrainedWasNotEnabled,
               RTIcap.RTIinternalError 
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.resetTimeConstrained();
       // constrained[theHandle] = true;

    }// end of disableTimeConstrained()

    /**
    * 8.8
    */
    public void 
    timeAdvanceRequest(double theTime, short theFederate)
         throws RTIcap.TimeAdvanceAlreadyInProgress, 
                RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       //if(DEBUG) System.out.println(" in timeAdvanceRequest() ");
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       if(federate.getTimeManagementState()==TIME_ADVANCING) 
          throw new RTIcap.TimeAdvanceAlreadyInProgress();

       federate.advanceTime(theTime);
       //if(DEBUG) System.out.println(" out of timeAdvanceRequest() ");
       
    }// end of timeAdvanceRequest()

    /**
    * 8.9
    */
    public void 
    timeAdvanceRequestAvailable(double time, short theHandle)
        throws RTIcap.EnableTimeConstrainedPending, 
               RTIcap.EnableTimeRegulationPending,
               RTIcap.FederateNotExecutionMember, RTIcap.InvalidFederationTime, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress, RTIcap.TimeAdvanceAlreadyInProgress
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       if(federate.getTimeManagementState()==TIME_ADVANCING) 
          throw new RTIcap.TimeAdvanceAlreadyInProgress();

       try { federate.advanceTimeToNextEvent(time); }
       catch(RTIcap.FederationTimeAlreadyPassed ftap) {}

    }// end of timeAdvanceRequestAvailable()

    /**
    * 8.10
    * Until given time stamp, process all the events for given federate.
    */
    public void 
    nextEventRequest( double theTime, short theFederate )
         throws RTIcap.TimeAdvanceAlreadyInProgress, 
                RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       if(federate.getTimeManagementState()==TIME_ADVANCING) 
          throw new RTIcap.TimeAdvanceAlreadyInProgress();

       federate.advanceTimeToNextEvent(theTime);

    }// end of nextEventRequest()

    /**
    * 8.11
    * 
    */
    public void 
    nextEventRequestAvailable(double time, short theHandle)
        throws RTIcap.EnableTimeConstrainedPending, 
               RTIcap.EnableTimeRegulationPending,
               RTIcap.FederateNotExecutionMember, RTIcap.InvalidFederationTime, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError, 
               RTIcap.SaveInProgress, RTIcap.TimeAdvanceAlreadyInProgress
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       if(federate.getTimeManagementState()==TIME_ADVANCING) 
          throw new RTIcap.TimeAdvanceAlreadyInProgress();

       try { federate.advanceTimeToNextEvent(time); }
       catch(RTIcap.FederationTimeAlreadyPassed ftap) {}


    }// end of nextEventRequestAvailable()

    /**
    * 8.12
    * Optimistically flushes the Queue.
    * These events should be rollbackable.
    */
    public void 
    flushQueueRequest(double theTime, short theFederate )
         throws RTIcap.TimeAdvanceAlreadyInProgress, 
                RTIcap.FederationTimeAlreadyPassed, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       if(federate.getTimeManagementState()==TIME_ADVANCING) 
          throw new RTIcap.TimeAdvanceAlreadyInProgress();

       federate.advanceTimeTo(theTime);

    }// end of flushQueueRequest()

    /**
    * 8.14
    */
    public void 
    enableAsynchronousDelivery( short theHandle )
        throws RTIcap.AsynchronousDeliveryAlreadyEnabled,
               RTIcap.ConcurrentAccessAttempted, 
               RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.setAsynchronousDelivery();

    }// end of enableAsynchronousDelivery()

    /**
    * 8.15
    */
    public void 
    disableAsynchronousDelivery( short theHandle )
        throws RTIcap.AsynchronousDeliveryAlreadyDisabled,
               RTIcap.ConcurrentAccessAttempted, 
               RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.resetAsynchronousDelivery();

    }// end of disableAsynchronousDelivery()

    /**
    * 8.16
    */
    public double 
    queryLBTS( short theFederate )
           throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
                  RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       return federate.getFederateLBTS();

    }// end of queryLBTS()

    /**
    * 8.17
    * queryFederateTime()
    * Returns the current time of the given federate. 
    */
    public double 
    queryFederateTime( short theFederate )
           throws RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress,
                  RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       return federate.getFederateTime();
       //return time[theFederate]; 

    }// end of queryFederateTime()

    /**
    * 8.18
    */
    public double 
    queryMinNextEventTime(short theHandle)
        throws RTIcap.FederateNotExecutionMember, RTIcap.RestoreInProgress,
               RTIcap.RTIinternalError, RTIcap.SaveInProgress
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();

       FederateInfo federate = fedEx.getFederateInfo(theHandle);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       return federate.minimumTSOOfNextEvent();

    }// end of queryMinNextEventTime() 

    /**
    * 8.19
    * updates the lookahead value of the given federate.
    */
    public void 
    modifyLookahead(double theLookahead, short theFederate )
        throws RTIcap.InvalidLookahead, RTIcap.FederateNotExecutionMember,
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       federate.setLookahead(theLookahead);
       //lookahead[theFederate] = theLookahead;

    }// end of modifyLookahead()

    /**
    * 8.20
    * Returns the lookahead value of the particular federate.
    */
    public double 
    queryLookahead( short theFederate )
           throws RTIcap.FederateNotExecutionMember, RTIcap.RTIinternalError 
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.FederateNotExecutionMember();

       return federate.getFederateLookahead();
       //return lookahead[theFederate];

    }// end of queryLookahead()

    /**
    * 8.21
    */
    public void 
    retract(RTIcap.EventRetractionHandle theHandle, short theFedHandle)
        throws RTIcap.InvalidRetractionHandle, RTIcap.FederateNotExecutionMember
, 
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
    }// end of retract()

    /**
    * 8.23
    */
    public void 
    changeAttributeOrderType( int theObject, short[] theAttributes, int theType, 
                              short theFedHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined,
               RTIcap.AttributeNotOwned, RTIcap.InvalidOrderingHandle,
               RTIcap.FederateNotExecutionMember, RTIcap.SaveInProgress, 
               RTIcap.RestoreInProgress, RTIcap.RTIinternalError
    {
    }// end of changeAttributeOrderType()

    /**
    * 8.24
    */
    public void 
    changeInteractionOrderType(short theObject, int theType, short theFedHandle)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionClassNotPublished, 
               RTIcap.InvalidOrderingHandle, RTIcap.FederateNotExecutionMember,
               RTIcap.SaveInProgress, RTIcap.RestoreInProgress, 
               RTIcap.RTIinternalError
    {
    }// end of changeInteractionOrderType()

    /**
    * It accesses the RO Event Queue of the federate and process pending
    * events.
    */
    public RTIcap._Boolean 
    tick(short theFederate) throws RTIcap.RTIinternalError
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.RTIinternalError();

       return federate.tick();

    }// end of tick()

    /**
    * It accesses the RO Event Queue of the federate and process pending
    * events.
    */
    public RTIcap._Boolean 
    tickMinMax(double minimum, double maximum, short theFederate)
        throws RTIcap.RTIinternalError
    {
       if(fedEx==null) throw new RTIcap.RTIinternalError();
       FederateInfo federate = fedEx.getFederateInfo(theFederate);

       if(federate==null) throw new RTIcap.RTIinternalError();

       return federate.tick();

    }// end of tickMaxMin()


}// end of TimeManager

