/** 
* Federation Management Interface
* Keeps the current FederationExecution objects.
*/

import RTIcap.*;
import npacrti.*;

import java.util.*;

/*
*/
public class 
FederationManager  extends _FederationManagerInterfaceImplBase
                   implements RTIDebug
{
    ObjectClassNaming objNaming;
    InteractionNaming interNaming;
    Hashtable FEDExecutions  = new Hashtable();
    Hashtable fedIdTOFedName = new Hashtable();

    int numberOfFederates = 0;

    /**
    */
    public 
    FederationManager(ObjectClassNaming _objNaming, InteractionNaming _interNaming)
    {
       objNaming = _objNaming;
       interNaming = _interNaming;

    }// end of CONSTRUCTOR

    /**
    * 4.2
    */
    public
    void createFederationExecution(String executionName)
         throws RTIcap.FederationExecutionAlreadyExists, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {

        if(DEBUG) System.out.println(" in createFederationExecution() for "+executionName);
        /** If the federation execution exists, then throw */
        if(getFederationExecution(executionName)!=null) 
        { 
             if(DEBUG) System.out.println(" FederationExecutionAlreadyExists for "+executionName);
             throw new RTIcap.FederationExecutionAlreadyExists();
        }
        
        /** Obviously, it is not. */

        FederationExecution fExe = new FederationExecution(executionName,objNaming,interNaming);
        
        FEDExecutions.put(executionName,fExe); 
       
    }// end of createFederationExecution()

    /**
    * 4.3
    */
    public
    void destroyFederationExecution(String executionName)
         throws RTIcap.FederatesCurrentlyJoined, 
                RTIcap.FederationExecutionDoesNotExist, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
        FederationExecution fExe = getFederationExecution(executionName);

        /** If the federation execution does not exist, then throw */
        if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();
  
        if(fExe.destroyExecution()) FEDExecutions.remove(executionName); 
        
    }// end of destroyFederationExecution() 

    /**
    * 4.4
    */
    public
    short joinFederationExecution(String yourName, String executionName, 
                 RTIcap.FederateAmbassador federateAmbassador)
          throws RTIcap.FederateAlreadyExecutionMember, 
                 RTIcap.FederationExecutionDoesNotExist, RTIcap.CouldNotOpenFED, 
                 RTIcap.ErrorReadingFED, RTIcap.ConcurrentAccessAttempted, 
                 RTIcap.RTIinternalError, RTIcap.Deadlock
    {
       short ret = 0;
       FederationExecution fExe = getFederationExecution(executionName);

       /** If the federation execution exists, then throw */
       if(fExe==null) 
             throw new RTIcap.FederationExecutionDoesNotExist();
       
       /** Obviously, it is not. */

       if(fExe.hasMember(yourName)) 
             throw new RTIcap.FederateAlreadyExecutionMember();

       ret = fExe.joinFederationExecution(yourName,federateAmbassador);

       numberOfFederates++;

       return (short)numberOfFederates;

    }// end of joinFederationExecution()

    /**
    * 4.5
    */
    public
    void resignFederationExecution(short theFederate,  String FederationName,
                                   RTIcap.ResignAction theAction)
         throws RTIcap.FederateOwnsAttributes, RTIcap.FederateDoesNotExist, 
                RTIcap.FederationExecutionDoesNotExist, 
                RTIcap.FederateNotExecutionMember, RTIcap.InvalidResignAction, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService 
    {

        if(DEBUG) System.out.println(theFederate+" wants to leave from "+FederationName);
        FederationExecution fedExe = (FederationExecution) FEDExecutions.get(FederationName);
        
        if(fedExe!=null)
                fedExe.resignFederationExecution2(theFederate,theAction);
        else  throw new RTIcap.FederationExecutionDoesNotExist();

    }// end of resignFederationExecution()

    /**
    * 4.5
    */
    public
    void resignFederationExecution1(short theFederate, 
                                    RTIcap.ResignAction theAction)
         throws RTIcap.FederateOwnsAttributes, RTIcap.FederateDoesNotExist, 
                RTIcap.FederationExecutionDoesNotExist, 
                RTIcap.FederateNotExecutionMember, RTIcap.InvalidResignAction, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {

       String fedName = (String) fedIdTOFedName.get(new Short(theFederate));

       if(fedName!=null)
       {
          FederationExecution fedExe = (FederationExecution) FEDExecutions.get(fedName);
          if(fedExe!=null)
                fedExe.resignFederationExecution2(theFederate,theAction);
          else  throw new RTIcap.FederationExecutionDoesNotExist();

       } else throw new RTIcap.FederateDoesNotExist();

    }// end of resignFederationExecution2()

    /**
    */
    public
    void requestPause(short theFederate, String FederationName, String label)
         throws RTIcap.FederationAlreadyPaused, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
        FederationExecution fedExe = (FederationExecution) FEDExecutions.get(FederationName);
        if(fedExe!=null) fedExe.requestPause(theFederate,label);
       
    }// end of requestPause()

    /**
    */
    public
    void pauseAchieved(String label)
         throws RTIcap.UnknownLabel, RTIcap.NoPauseRequested, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void requestResume(short theFederate,  String FederationName)
         throws RTIcap.FederationNotPaused, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
        FederationExecution fedExe = (FederationExecution) FEDExecutions.get(FederationName);
        if(fedExe!=null) fedExe.requestResume(theFederate);
       
    }

    /**
    */
    public
    void resumeAchieved()
         throws RTIcap.NoResumeRequested, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    * 4.11
    */
    public
    void requestFederationSave1(String label, double theTime)
         throws RTIcap.FederationTimeAlreadyPassed, 
                RTIcap.InvalidFederationTime, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    * 4.11
    */
    public
    void requestFederationSave2(String label)
         throws RTIcap.SaveInProgress, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    * 4.13
    */
    public
    void federateSaveBegun1()
         throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    * 4.13
    */
    public
    void federateSaveBegun2(double theTime)
         throws RTIcap.SaveNotInitiated, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void federateSaveAchieved()
         throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void federateSaveNotAchieved()
         throws RTIcap.SaveNotInitiated, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void requestRestore(String label)
         throws RTIcap.SpecifiedSaveLabelDoesNotExist, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void restoreAchieved(String label)
         throws RTIcap.UnknownLabel, RTIcap.RestoreNotRequested, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    /**
    */
    public
    void restoreNotAchieved(String label)
         throws RTIcap.UnknownLabel, RTIcap.RestoreNotRequested, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock, RTIcap.UnimplementedService
    {
    }

    //************************/
    //** Propriatery Methods */
    //************************/

    /**
    * Returns null : if the federation does not exist <br>
    *         valid FederationExecution : if the federation exec exists.
    */
    FederationExecution
    getFederationExecution(String FEDName)
    {
       return (FederationExecution) FEDExecutions.get(FEDName);

    }// end of getFedederationExecution()

}// end of FederationManager

