/**
*
* This file reads DIF definition and produces DIFDocument object. 
*
* @date August 15, 1998 ( Last Updated on August 15, 1998 )
* @author H. Timucin Ozdemir
*
*/
package owrti.util;

import owrti.xml.*;                // for owrti.xml.DIFDocument object

import java.io.*;
import java.util.*;
import java.lang.*;

public class DIFParser 
{
  static final boolean DEBUG = false;

  static final int BEGIN    = 0;
  static final int FED_RCV  = 1;
  static final int FEDN_RCV = 2;
  static final int FED_OK   = 3;
  static final int VER_RCV  = 4;
  static final int VN_RCV   = 5;
  static final int VER_OK   = 6;
  static final int SP_BNGS  = 7;
  static final int A_SP     = 8;
  static final int SPN_RCV  = 9;
  static final int DIM_BGN  = 10;
  static final int DIMN_RV  = 11;
  static final int SP_OK    = 12;
  static final int FEDE_RCV = 13;
  static final int SP_END   = 14;

  static final int OBJ_BGN  = 15;
  static final int OBJC_BGN = 16;
  static final int CLN_OK   = 17;
  static final int ATTR_BGN = 18;
  static final int ATTN_OK  = 19;
  static final int TRA_OK   = 20;
  static final int ORD_OK   = 21;
  static final int ASN_OK   = 22;
  static final int OBJ_END  = 23;

  static final int INTER_BGN   = 24;
  static final int INTERC_BGN  = 25;
  static final int INTERN_OK   = 26;
  static final int INTER_TR_OK = 27;
  static final int INTER_OR_OK = 28;
  static final int PARAM_BGN   = 29;
  static final int PARAMN_OK   = 30;
  static final int INTER_END   = 31;

  static final int FED_END = 32;

  static final String k_FED          = "(FED";
  static final String k_END          = ")";
  static final String k_Federation   = "(Federation";
  static final String k_FEDversion   = "(FEDversion";
  static final String k_spaces       = "(spaces";
  static final String k_space        = "(space";
  static final String k_dimension    = "(dimension";
  static final String k_objects      = "(objects";
  static final String k_class        = "(class";
  static final String k_attribute    = "(attribute";
  static final String k_interactions = "(interactions";
  static final String k_parameter    = "(parameter";


  static final int k_FED_id          = 1;
  static final int k_END_id          = 2;
  static final int k_Federation_id   = 3;
  static final int k_FEDversion_id   = 4;
  static final int k_spaces_id       = 5;
  static final int k_space_id        = 6;
  static final int k_dimension_id    = 7;
  static final int k_objects_id      = 8;
  static final int k_class_id        = 9;
  static final int k_attribute_id    = 10;
  static final int k_interactions_id = 11;
  static final int k_parameter_id    = 12;
  static final int k_str_id          = 13;

  int  cState = BEGIN;

  Hashtable keywords = new Hashtable();

  /**
  */
  public DIFParser()
  {

     keywords.put( new String(k_FED),         new Integer(k_FED_id));
     keywords.put( new String(k_END),         new Integer(k_END_id));
     keywords.put( new String(k_Federation),  new Integer(k_Federation_id));
     keywords.put( new String(k_FEDversion),  new Integer(k_FEDversion_id));
     keywords.put( new String(k_spaces),      new Integer(k_spaces_id));
     keywords.put( new String(k_space),       new Integer(k_space_id));
     keywords.put( new String(k_dimension),   new Integer(k_dimension_id));
     keywords.put( new String(k_objects),     new Integer(k_objects_id));
     keywords.put( new String(k_class),       new Integer(k_class_id));
     keywords.put( new String(k_attribute),   new Integer(k_attribute_id));
     keywords.put( new String(k_interactions),new Integer(k_interactions_id));
     keywords.put( new String(k_parameter),   new Integer(k_parameter_id));

  }// end of constructor

  /**
  */
  public DIFDocument
  parseFile(String fileName)
  {

    DIFDocument ret = null;

    try
    {
      FileReader in = new FileReader(fileName); 
      ret = parseInputStream(in);

    } catch( java.io.FileNotFoundException fnfe) {}

    return ret; 

  }// end of parseFile()

  /**
  */
  String processEnds(String line)
  {
     String ret = line;
     int lindx = 0;

     while( (lindx=ret.indexOf(')',lindx))!=-1 )
     {
        ret = ret.substring(0,lindx)+" "+ret.substring(lindx,ret.length());
        lindx+=2; 
     }// end of while

     return ret;
     
  }// end of processEnds()

  /**
  */
  DIFDocument parseInputStream(java.io.Reader inR)
  {
     DIFDocument dif = new DIFDocument();
     Stack cstack = new Stack();
     Stack istack = new Stack();
     boolean stay = true;
     String line = new String();
     String line2;
     String aToken;
     Integer id;
     int tokenID;
     boolean notEnd = true;
     BufferedReader in = new BufferedReader(inR);
     SpaceDef sDef = null;
     String   dimension=null;
     ObjClassDef obj = null;
     AttributeDef attribute = null;
     String   parameter=null;

     InteractionDef inter_act = null;


     cState = BEGIN;

     while( stay )
     {
        line = new String();
        notEnd = true;

        try
        { 
          while(line.length()<=0)
          { 
             line2 = in.readLine();
             if(line2!=null&&!line2.startsWith(";"))
             {
             line2 = line2.trim();
             line2 = processEnds(line2);
             line += (line2.trim());
             }

          }// end of while
           
        } catch(java.io.IOException ioe) { notEnd=false; }

       
        StringTokenizer sTknz = new StringTokenizer("");
        
        if(notEnd) 
        {  sTknz = new StringTokenizer(line);
           if(DEBUG) System.out.println("line:"+line);
        }// end of if

        while( notEnd && sTknz.countTokens()>0 )
        {
           aToken = sTknz.nextToken();
           tokenID = k_str_id;

           id = (Integer) keywords.get(aToken);
           if(id!=null) tokenID = id.intValue();

           if(DEBUG) System.out.println("token: "+aToken+" id: "+tokenID);

           switch(cState)
           {

              case BEGIN :
                   if(DEBUG) System.out.println("BEGIN");

                   if(tokenID==k_FED_id) cState = FED_RCV;
                   else notEnd = false; 
                   if(DEBUG) System.out.println("cState: "+cState);
                   break;

              case FED_RCV :

                   if(DEBUG) System.out.println("(FED received");

                   if(tokenID==k_Federation_id) cState = FEDE_RCV;
                   else notEnd = false; 
                   break;

              case FEDE_RCV :

                   if(DEBUG) System.out.println("(Federation received"+aToken);

                   if(tokenID==k_str_id) 
                   { cState = FEDN_RCV;
                     dif.setFederationName(aToken);
                   }
                   else notEnd = false; 
                   break;

              case FEDN_RCV :
                   if(DEBUG) System.out.println(aToken+" is OK" );
                   if(tokenID==k_END_id) cState = FED_OK;
                   else notEnd = false; 
                   break;

              case FED_OK :
                   if(tokenID==k_FEDversion_id) cState = VER_RCV;
                   else notEnd = false; 
                   break;

              case VER_RCV :
                   if(tokenID==k_str_id) 
                   { cState = VN_RCV;
                     dif.setVersion(aToken);
                   }
                   else notEnd = false; 
                   break;

              case VN_RCV :
                   if(tokenID==k_END_id) cState = VER_OK;
                   else notEnd = false; 
                   break;

              case VER_OK :
                   if(tokenID==k_spaces_id) cState = SP_BNGS;
                   else if(tokenID==k_objects_id) cState = OBJ_BGN ;
                   else if(tokenID==k_interactions_id) cState = INTER_BGN ;
                   else if(tokenID==k_END_id) cState = FED_END ;
                   else notEnd = false; 
                   break;

              case SP_BNGS :
                   if(tokenID==k_space_id) 
                   { cState = A_SP;
                     sDef = new SpaceDef(); 
                   }
                   else notEnd = false; 
                   break;

              case A_SP :
                   if(tokenID==k_str_id) 
                   { cState = SPN_RCV ;
                     sDef.setName(aToken);
                   }
                   else notEnd = false; 
                   break;

              case SPN_RCV :
                   if(tokenID==k_END_id) 
                   { cState = SP_OK ;
                     dif.addSpaceDef(sDef); sDef=null;
                   }
                   else if(tokenID==k_dimension_id) cState = DIM_BGN ;
                   else notEnd = false; 
                   break;

              case DIM_BGN :
                   if(tokenID==k_str_id) 
                   { cState = DIMN_RV ; dimension=new String(aToken); }
                   else notEnd = false; 
                   break;

              case DIMN_RV :

                   if(tokenID==k_END_id) 
                   { 
                      cState = SPN_RCV ;
                      if(dimension!=null&&sDef!=null) 
                         sDef.addDimension(dimension);
                      dimension = null;

                   } else notEnd = false; 

                   break;

              case SP_OK :
                   if(tokenID==k_END_id) cState = SP_END ;
                   else notEnd = false; 
                   break;

              case SP_END :
                   if(tokenID==k_objects_id) cState = OBJ_BGN ;
                   else if(tokenID==k_interactions_id) cState = INTER_BGN ;
                   else if(tokenID==k_END_id) cState = FED_END ;
                   else notEnd = false; 
                   break;

              case OBJ_BGN :
                   if(DEBUG) System.out.println("OBJ_BGN token: "+aToken);
                   if(tokenID==k_END_id)  cState = OBJ_END;
                   else if(tokenID==k_class_id) cState = OBJC_BGN ;
                   else notEnd = false; 
                   break;

              case OBJC_BGN :
                   if(DEBUG) System.out.println("OBJC_BGN token: "+aToken);
                   if(tokenID==k_str_id) 
                   { cState = CLN_OK ;
                     obj = new ObjClassDef(aToken);
                   } else notEnd = false; 
                   break;

              case CLN_OK :
                   if(DEBUG) System.out.println("CLN_OK token: "+aToken);
                   if(tokenID==k_END_id) 
                   { 
                      if(cstack.empty()) 
                      {  cState = OBJ_BGN ;
                         if(obj!=null) dif.addObjectClass(obj);
                      } 
                      else 
                       { ObjClassDef obj1 = (ObjClassDef) cstack.pop();
                         obj1.addChildren(obj);
                         obj = obj1;
                       }
                   }
                   else if(tokenID==k_class_id) 
                        { cState = OBJC_BGN ; cstack.push(obj); obj=null; }
                   else if(tokenID==k_attribute_id) cState = ATTR_BGN ;
                   else notEnd = false; 
                   break;

              case ATTR_BGN :
                   if(DEBUG) System.out.println("ATTR_BGN");
                   if(tokenID==k_str_id) 
                   { cState = ATTN_OK ;
                     attribute = new AttributeDef(aToken);
                   }
                   else notEnd = false; 
                   break;

              case ATTN_OK :
                   if(DEBUG) System.out.println("ATTN_OK");
                   if(tokenID==k_str_id) 
                   { cState = TRA_OK ;
                     if(attribute!=null) 
                      { attribute.setToReliable();
                        if(aToken.equals("besteffort")) 
                           attribute.setToBestEffort();
                      } 
                   }
                   else notEnd = false; 
                   break;

              case TRA_OK :
                   if(DEBUG) System.out.println("TRA_OK");
                   if(tokenID==k_str_id) 
                   {  cState = ORD_OK ;
                      if(attribute!=null) 
                      { attribute.setToRO();
                        if(aToken.equals("timestamp")) attribute.setToTSO();
                      } 
                   }
                   else notEnd = false; 
                   break;

              case ORD_OK :
                   if(DEBUG) System.out.println("ORD_OK");
                   if(tokenID==k_str_id) 
                   {  cState = ASN_OK ;
                      //if(attribute!=null) attribute.setSpaceName(aToken); 
                   }
                   else if(tokenID==k_END_id) 
                        {  cState = CLN_OK ;
                           if(obj!=null&&attribute!=null)
                              obj.addAttribute(attribute);
                        }
                   else notEnd = false; 
                   break;

              case ASN_OK :
                   if(DEBUG) System.out.println("ASN_OK");
                   if(tokenID==k_END_id) 
                        {  cState = CLN_OK ;
                           if(obj!=null&&attribute!=null)
                              obj.addAttribute(attribute);
                        }
                   else notEnd = false; 
                   break;

              case OBJ_END :
                   if(DEBUG) System.out.println("OBJ_END token:"+aToken);
                   if(tokenID==k_interactions_id) cState = INTER_BGN ;
                   else if(tokenID==k_END_id) cState = FED_END ;
                   else notEnd = false; 
                   break;

              case INTER_BGN :
                   if(DEBUG) System.out.println("INTER_BGN token: "+aToken);
                   if(tokenID==k_class_id) cState = INTERC_BGN ;
                   else if(tokenID==k_END_id) cState = FED_END ;
                   else notEnd = false; 
                   break;

              case INTERC_BGN :
                   if(DEBUG) System.out.println("INTERC_BGN token: "+aToken);
                   if(tokenID==k_str_id) 
                   { cState = INTERN_OK ;
                     inter_act = new InteractionDef(aToken);
                   }
                   else notEnd = false; 
                   break;

              case INTERN_OK :
                   if(DEBUG) System.out.println("INTERN_OK token: "+aToken);

                   if(tokenID==k_str_id) 
                   {  
                      cState = INTER_TR_OK ;

                      if( inter_act!=null )
                      {
                         inter_act.setToReliable();
                         if(aToken.equals("besteffort")) 
                            inter_act.setToBestEffort();  

                      }// end of if

                   } else notEnd = false; 

                   break;

              case INTER_TR_OK :
                   if(DEBUG) System.out.println("INTER_TR_OK token: "+aToken);

                   if(tokenID==k_str_id) 
                   {  
                      cState = INTER_OR_OK ;

                      if( inter_act!=null )
                      {
                         inter_act.setToRO();
                         if(aToken.equals("timestamp")) inter_act.setToTSO();  
                      }// end of if

                   } else notEnd = false; 

                   break;

              case INTER_OR_OK :
                   if(DEBUG) System.out.println("INTER_OR_OK token: "+aToken);
                   if(tokenID==k_END_id) 
                   {  if(istack.empty()) 
                      { cState = INTER_BGN ;
                        if(inter_act!=null) dif.addInteractionClass(inter_act);
                      }
                      else 
                       { 
                          InteractionDef inter1 = (InteractionDef) istack.pop(); 
                          inter1.addChildren(inter_act);
                          inter_act = inter1;
                       }
                   }
                   else if(tokenID==k_class_id) 
                   { cState = INTERC_BGN ; istack.push(inter_act); inter_act=null;}
                   else if(tokenID==k_parameter_id) cState = PARAM_BGN ;
                   else notEnd = false; 
                   break;

              case PARAM_BGN :
                   if(DEBUG) System.out.println("PARAM_BGN token: "+aToken);
                   if(tokenID==k_str_id) 
                   { cState = PARAMN_OK ;
                     parameter = new String(aToken);
                   }
                   else notEnd = false; 
                   break;

              case PARAMN_OK :
                   if(DEBUG) System.out.println("PARAMN_OK token: "+aToken);
                   if(tokenID==k_END_id) 
                   { cState = INTER_OR_OK ; 
                     if(inter_act!=null) inter_act.addParameter(parameter); 
                   }
                   else notEnd = false; 
                   parameter = null;
                   break;

              case FED_END :
                   if(DEBUG) System.out.println("FED_END token: "+aToken);
                   if(tokenID==k_END_id) notEnd = false;
                   notEnd = false; 
                   break;

           }// end of switch()
       
        }// end of while

        stay = notEnd;

     }// end of while
    
     if(DEBUG)
     {
     try
     {
       dif.printDIF(new PrintStream( new FileOutputStream("out")));
     } catch(java.io.IOException ioe) {}
     }
     
     return dif;

  }// end of parseInputStream()
  
  
  /** */ 
  public static void 
  main (String args[]) throws Exception
  {
    DIFParser parser = new DIFParser();

    /** Default output media is stamdard output */
    PrintStream out = System.out;

    if (args.length < 1) 
    {
      System.err.println("Usage: java DIFParser <file> [<output_file_name>]");
      System.exit(1);
    }

    /** Define the output file stream */
    if(args.length >1 )
    {
      out = new PrintStream( new FileOutputStream(args[1])); 
    }

    /** Define the dom object 
    */
    //Freedom dom = new Freedom("com.docuverse.freedom.SAXDriver");

    /** Define the XMLParser object */
    //dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");

    /** Note that for file names 'makeAbsoluteURL' is critical */
    //Document doc = dom.openDocument(DIFDocument.makeAbsoluteURL(args[0]));

    System.out.println("~~~~Let's see~~~~~");

    /** Define the DIFDocument object which extract the info from dom */
    //DIFDocument dif = new DIFDocument(doc);

    //System.out.println(" Federation Name: "+dif.getFederationName());

    /** Let's see what it looks like in DIF format */
    //dif.printDIF(out);

    parser.parseFile(args[0]);

  }// end of main()

}// end of hmltodif
