package owrti.xml;

import org.w3c.dom.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Stack;
import java.io.PrintStream;
import java.net.*;


/**
*
* @date   August 11, 1998 ( Last Updated August 11, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
DIFDocument implements XMLDebug
{
   /** Version */
   String version;

   /** Federation Name */
   String FedName;

   /** Routing Spaces */
   //Vector spaces;
   Hashtable spaces;

   /** Object Classes */
   //Vector objClasses;
   Hashtable objClasses;

   /** Interaction Classes */
   //Vector interClasses;
   Hashtable interClasses;

   /** isExtracted? */
   boolean isExtracted = true;

   /** org.w3c.dom.Document */
   Document dom;

   /**
   */
   public DIFDocument(Document _dom)
   {
      interClasses = new Hashtable();
        objClasses = new Hashtable();
            spaces = new Hashtable();
               dom = _dom;

   }// end of CONSTRUCTOR

   /**
   */
   public DIFDocument()
   {
      isExtracted  = false;
      interClasses = new Hashtable();
        objClasses = new Hashtable();
            spaces = new Hashtable();

   }// end of CONSTRUCTOR

   /**
   */
   public String 
   getVersion() 
   { 
      if(isExtracted) extractFromDocument();
      return version; 
   }// end of getVersion()

   /**
   */
   public void 
   setVersion(String _version) 
   { 
      version = new String(_version); 

   }// end of getVersion()

   /**
   */
   public String 
   getFederationName() 
   { 
      if(isExtracted) extractFromDocument();
      return FedName; 

   }// end of setFederationName()

   /**
   */
   public void 
   setFederationName(String _name) 
   { 
      FedName = new String(_name); 

   }// end of setFederationName()

   /**
   */
   public Enumeration 
   getSpaces() 
   { 
      if(isExtracted) extractFromDocument();
      return spaces.elements(); 
   }// end of getSpaces()

   /**
   */
   public void 
   addSpaceDef(SpaceDef _space) 
   { 
      SpaceDef sp = (SpaceDef) spaces.get(_space.getName());
      if(sp==null) spaces.put(_space.getName(),_space);

   }// end of addSpaceDef()

   /**
   */
   public Enumeration 
   getObjectClasses() 
   { 
      if(isExtracted) extractFromDocument();
      return objClasses.elements(); 
   }// end of getObjectClasses()

   /**
   */
   public void 
   addObjectClass(ObjClassDef _obj) 
   { 
      ObjClassDef obj = (ObjClassDef) objClasses.get(_obj.getName());
      if(obj==null) objClasses.put(_obj.getName(),_obj);

   }// end of getObjectClasses()

   /**
   */
   public Enumeration 
   getInteractionClasses() 
   { 
      if(isExtracted) extractFromDocument();
      return interClasses.elements(); 
   }// end of getInteractionClasses()

   /**
   */
   public void 
   addInteractionClass(InteractionDef _inter) 
   { 
      InteractionDef inter = (InteractionDef) interClasses.get(_inter.getName());
      if(inter==null) interClasses.put(_inter.getName(),_inter);

   }// end of addInteractionClass()

   /**
   */
   public void 
   extractFromDocument() 
   { 
      if(DEBUG) System.out.println(" in extractFromDocument() ");
      isExtracted = false;
      if(dom!=null)
      {
         Element fedRoot = dom.getDocumentElement();
         if(fedRoot!=null ) //&& fedRoot.getTagName().equals("FED"))
         {
            extractFederationName(fedRoot);
            extractVersion(fedRoot);
            extractSpace(fedRoot);
            extractObjects(fedRoot);
            extractInteractions(fedRoot);

         }// end of if

      } else System.out.println(" Document is NULL1");

      if(DEBUG) System.out.println(" out of extractFromDocument() ");

   }// end of extractFromDocument()

   /**
   */
   final void 
   extractFederationName(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractFederationName() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() && !found)
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_federation)) 
           { 
               fedNames1 = current.getChildNodes() ;
               if(fedNames1!=null && fedNames1.getLength()>0) 
               {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { found = true; }
               }
           }

        }// end of while

        if(found) FedName = new String(current2.getNodeValue());

     }// end of if 
     if(DEBUG) System.out.println(" out of extractFederationName() ");

   }// end of extractFederationName()

   /**
   */
   final void 
   extractVersion(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractVersion() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() && !found)
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_version)) 
           { 
               fedNames1 = current.getChildNodes() ;
               if(fedNames1!=null && fedNames1.getLength()>0) 
               {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { found = true; }
               }
           }

        }// end of while

        if(found) version = new String(current2.getNodeValue());

     }// end of if 
     if(DEBUG) System.out.println(" out of extractVersion() ");

   }// end of extractVersion()

   /**
   */
   final void 
   extractInteractions(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractInteractions() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() && !found)
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_interaction)) 
               found = true; 
              

        }// end of while

        if(found) extractEachInteraction((Element)current);

     }// end of if 

     if(DEBUG) System.out.println(" out of extractInteractions() ");

   }// end of extractInteractions()

   /**
   */
   final void 
   extractEachInteraction(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractEachInteraction() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while( i<fedNames.getLength() )
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_intclass)) 
               extractAnInteraction(null,(Element)current);

        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractEachInteraction() ");

   }// end of extractEachInteraction()

   /**
   */
   final void 
   extractAnInteraction(InteractionDef parent,Element fedRoot)
   {
     if(DEBUG) System.out.println(" in extractAnObject() ");
     InteractionDef interDef = null;
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found ;
        while(i<fedNames.getLength() )
        {
           current = fedNames.item(i++);

           found = false; 
           if( current!=null )
           { 
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_name) && interDef==null) 
             { 
                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { interDef = new InteractionDef(current2.getNodeValue(),parent); found = true; 
                    if(parent!=null) parent.addChildren(interDef);
                  }
                }
             }
             else 
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                  ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_transport) && 
                  interDef!=null )
              {

                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0)
                {

                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  {
                    String tr = current2.getNodeValue();
                    if(tr.equals(HLAML_CONSTANTS.tag_reliable)) interDef.setToReliable();
                    if(tr.equals(HLAML_CONSTANTS.tag_besteffort)) interDef.setToBestEffort();
                  }
                }

              }
            else

	
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_order) && 
                interDef!=null)
              {

                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0)
                {

                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  {
                    String tr = current2.getNodeValue();
                    if(tr.equals(HLAML_CONSTANTS.tag_ro)) interDef.setToRO();
                    if(tr.equals(HLAML_CONSTANTS.tag_tso)) interDef.setToTSO();
                  }
                }
              }

             else
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_parameter) && 
                interDef!=null)
              {

                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0)
                {

                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  {
                    interDef.addParameter(current2.getNodeValue());
                  }
                }
              }
             else
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_intclass) && interDef!=null ) 
              {
                 extractAnInteraction(interDef,(Element)current);
              } 

           }// end of if

           /** Add all object which has no parent */
           //if(found && parent==null) interClasses.addElement(interDef);
           if(found && parent==null) interClasses.put(interDef.getName(),interDef);

        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractAnInteraction() ");

   }// end of extractAnInteraction()

   /**
   */
   final void 
   extractObjects(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractObjects() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() && !found)
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_objects)) found = true; 
              

        }// end of while

        if(found) extractEachObject((Element)current);

     }// end of if 

     if(DEBUG) System.out.println(" out of extractObjects() ");

   }// end of extractObjects()

   /**
   */
   final void 
   extractEachObject(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractEachObject() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while( i<fedNames.getLength() )
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_objclass)) 
               extractAnObject(null,(Element)current);

        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractEachObject() ");

   }// end of extractEachObject()

   /**
   */
   final void 
   extractAnObject(ObjClassDef parent,Element fedRoot)
   {
     if(DEBUG) System.out.println(" in extractAnObject() ");
     ObjClassDef objInstance = null;
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found ;
        while(i<fedNames.getLength() )
        {
           current = fedNames.item(i++);

           found = false; 
           if( current!=null )
           { 
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_name) && objInstance==null) 
             { 
                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { objInstance = new ObjClassDef(current2.getNodeValue(),parent); found = true; 
                    if(parent!=null) parent.addChildren(objInstance);
                  }
                }
             }
             else 
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_attList) && objInstance!=null ) 
              {
                 extractAttributes(objInstance,(Element)current);
              } 
             else
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_objclass) && objInstance!=null ) 
              {
                 extractAnObject(objInstance,(Element)current);
              } 

           }// end of if

           /** Add all object which has no parent */
           //if(found && parent==null) objClasses.addElement(objInstance);
           if(found && parent==null) objClasses.put(objInstance.getName(),objInstance);

        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractAnObject() ");

   }// end of extractAnObject()

   /**
   */
   final void 
   extractAttributes(ObjClassDef parent, Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractAttributes() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        while(i<fedNames.getLength())
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_attribute) ) 
               extractAnAttribute(parent,(Element)current);
              
        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractAttributes(() ");

   }// end of extractAttributes()

   /**
   */
   final void 
   extractAnAttribute(ObjClassDef parent, Element fedRoot)
   {
     if(DEBUG) System.out.println(" in extractAnAttribute() ");
     AttributeDef attDef = null;
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() )
        {
           current = fedNames.item(i++);

           if( current!=null )
           { 
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_name) && attDef==null) 
             { 

                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { attDef = new AttributeDef(current2.getNodeValue()); 
                    parent.addAttribute(attDef); 
                  }
                }

             }

             else 
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_transport) && attDef!=null ) 
              {

                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { 
                    String tr = current2.getNodeValue(); 
                    if(tr.equals(HLAML_CONSTANTS.tag_reliable)) attDef.setToReliable();
                    if(tr.equals(HLAML_CONSTANTS.tag_besteffort)) attDef.setToBestEffort();
                  }
                }

              } 
             else 
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_order) && attDef!=null ) 
              {

                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { 
                    String tr = current2.getNodeValue(); 
                    if(tr.equals(HLAML_CONSTANTS.tag_ro)) attDef.setToRO();
                    if(tr.equals(HLAML_CONSTANTS.tag_tso)) attDef.setToTSO();
                  }
                }

              } 

           }// end of if

        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractAnAttribute() ");

   }// end of extractAnAttribute()

   /**
   */
   final void 
   extractSpace(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractSpace() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() && !found)
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_spaceList)) found = true; 
              

        }// end of while

        if(found) extractSubSpaces((Element)current);

     }// end of if 

     if(DEBUG) System.out.println(" out of extractSpace() ");

   }// end of extractSpace()

   /**
   */
   final void 
   extractSubSpaces(Element fedRoot)
   {

     if(DEBUG) System.out.println(" in extractSubSpaces() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() && !found)
        {
           current = fedNames.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_space)) found = true; 

        }// end of while

        if(found) extractASpace((Element)current);

     }// end of if 
     if(DEBUG) System.out.println(" out of extractSubSpaces() ");

   }// end of extractSubSpaces()

   /**
   */
   final void 
   extractASpace(Element fedRoot)
   {
     if(DEBUG) System.out.println(" in extractASpace() ");
     SpaceDef nspace = null;
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found = false; 
        while(i<fedNames.getLength() )
        {
           current = fedNames.item(i++);

           if( current!=null )
           { 
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_sname) && nspace==null) 
             { 
                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { nspace = new SpaceDef(current2.getNodeValue()); found = true; }
                }
             }
             else 
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_dimList) && nspace!=null ) 
              {
                 extractDimensions(nspace,(Element)current);
              } 

           }// end of if

        }// end of while

        //if(found) spaces.addElement(nspace);
        if(found) spaces.put(nspace.getName(),nspace);

     }// end of if 
     if(DEBUG) System.out.println(" out of extractASpace() ");

   }// end of extractASpace()

   /**
   */
   final void 
   extractDimensions(SpaceDef nspace, Element fedRoot)
   {
     if(DEBUG) System.out.println(" in extractDimensions() ");
     NodeList fedNames = ((Node)fedRoot).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     if(fedNames!=null && fedNames.getLength()>0) 
     {
        boolean found ;
        while(i<fedNames.getLength() )
        {
           found = false; 
           current = fedNames.item(i++);

           if( current!=null )
           { 
             if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                ((Element)current).getTagName().equals(HLAML_CONSTANTS.tag_dimension) ) 
             { 
                fedNames1 = current.getChildNodes() ;
                if(fedNames1!=null && fedNames1.getLength()>0) 
                {
      
                  current2 = fedNames1.item(0);
                  if( current2!=null && current2.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                  { found = true; }
                }
             }

           }// end of if

           if(found) nspace.addDimension(current2.getNodeValue());

        }// end of while

     }// end of if 

     if(DEBUG) System.out.println(" out of extractDimensions() ");

   }// end of extractDimensions()

   /**
   */
   public void 
   printDIF(PrintStream o) 
   { 
      o.println( DIF_CONSTANTS.pfx_FED );
      printDIFName(o);
      printDIFVersion(o);
      printDIFSpaces(o,true);
      printDIFObjs(o,true);
      printDIFInter(o,true);
      o.println( DIF_CONSTANTS.sfx_FED );

   }// end of printDIF()

   /**
   */
   final void printDIFVersion(PrintStream o) 
   { 
       o.println( DIF_CONSTANTS.pfx_version+version+
                  DIF_CONSTANTS.sfx_version); 

   }// end of printDIFVersion()

   /**
   */
   final void printDIFName(PrintStream o) 
   { 
       o.println( DIF_CONSTANTS.pfx_federation+FedName+
                  DIF_CONSTANTS.sfx_federation); 

   }// end of printDIFName()

   /**
   */
   final void printDIFSpaces(PrintStream o, boolean isDIF) 
   { 
      SpaceDef current ;
      Enumeration e = spaces.elements();

      if(e!=null && e.hasMoreElements())
      {
        if(isDIF) o.println(DIF_CONSTANTS.pfx_spaceList);
        else o.println(HLAML_CONSTANTS.pfx_spaceList);

        for(;e.hasMoreElements();)
        {
           current = (SpaceDef) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) current.printDIF(o);
              else current.printXML(o);
           }

        }// end of for

        if(isDIF) o.println(DIF_CONSTANTS.sfx_spaceList);
        else o.println(HLAML_CONSTANTS.sfx_spaceList);

      }// end of if

   }// end of printDIFSpaces()

   /**
   */
   final void printDIFObjs(PrintStream o, boolean isDIF) 
   { 
      ObjClassDef current ;
      Enumeration e = objClasses.elements();

      if(e!=null && e.hasMoreElements())
      {
        if(isDIF) o.println(DIF_CONSTANTS.pfx_objects);
        else o.println(HLAML_CONSTANTS.pfx_objects);

        for(;e.hasMoreElements();)
        {
           current = (ObjClassDef) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) current.printDIF(o);
              else current.printXML(o);
           }

        }// end of for

        if(isDIF) o.println(DIF_CONSTANTS.sfx_objects);
        else o.println(HLAML_CONSTANTS.sfx_objects);

      }// end of if

   }// end of printDIFObjs()

   /**
   */
   final void printDIFInter(PrintStream o, boolean isDIF) 
   { 
      InteractionDef current ;
      Enumeration e = interClasses.elements();

      if(e!=null && e.hasMoreElements())
      {
        if(isDIF) o.println(DIF_CONSTANTS.pfx_interaction);
        else o.println(HLAML_CONSTANTS.pfx_interaction);

        for(;e.hasMoreElements();)
        {
           current = (InteractionDef) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) current.printDIF(o);
              else current.printXML(o);
           }

        }// end of for

        if(isDIF) o.println(DIF_CONSTANTS.sfx_interaction);
        else o.println(HLAML_CONSTANTS.sfx_interaction);

      }// end of if

   }// end of printDIFInter()

   /**
   */
   public void 
   printXML(PrintStream o) 
   { 
      o.println(HLAML_CONSTANTS.pfx_FED);
      printXMLName(o);
      printXMLVersion(o);
      printDIFSpaces(o,false);
      printDIFObjs(o,false);
      printDIFInter(o,false);
      o.println(HLAML_CONSTANTS.sfx_FED);

   }// end of printDIF()

   /**
   */
   final void printXMLVersion(PrintStream o) 
   { o.println(HLAML_CONSTANTS.pfx_version+version+HLAML_CONSTANTS.sfx_version); }

   /**
   */
   final void printXMLName(PrintStream o) 
   { o.println( HLAML_CONSTANTS.pfx_federation+FedName+
                HLAML_CONSTANTS.sfx_federation); }

   /**
   */
   public void 
   printAllObjects(PrintStream o) 
   { 
      Enumeration e2;
      Enumeration e = getObjectClasses();
      ObjClassDef current;
      Stack       myStack = new Stack();
      
      if(e!=null && e.hasMoreElements())
      {
         for(;e.hasMoreElements();)
         {
            current = (ObjClassDef) e.nextElement();
            myStack.push(current);
            while(!myStack.empty())
            {
               current = (ObjClassDef) myStack.pop();
               if(current!=null) o.println(current.getFullName());
               e2 = current.getChildren();
               if(e2!=null && e2.hasMoreElements())
                  for(;e2.hasMoreElements();) myStack.push(e2.nextElement());
               
            }// end of while

         }// end of for

      }// end of if 

   }// end of printAllObjects()

   /**
   */
   public void 
   printAllInteractions(PrintStream o) 
   { 
      Enumeration e2;
      Enumeration e = getInteractionClasses();
      InteractionDef current;
      Stack       myStack = new Stack();
      
      if(e!=null && e.hasMoreElements())
      {
         for(;e.hasMoreElements();)
         {
            current = (InteractionDef) e.nextElement();
            myStack.push(current);
            while(!myStack.empty())
            {
               current = (InteractionDef) myStack.pop();
               if(current!=null) o.println(current.getFullName());
               e2 = current.getChildren();
               if(e2!=null && e2.hasMoreElements())
                  for(;e2.hasMoreElements();) myStack.push(e2.nextElement());
               
            }// end of while

         }// end of for

      }// end of if 

   }// end of printAllInteractions()

   public static String 
   makeAbsoluteURL (String url) throws MalformedURLException
   {
      URL baseURL;
      String currentDirectory = System.getProperty("user.dir");
      String fileSep = System.getProperty("file.separator");
      String file = currentDirectory.replace(fileSep.charAt(0), '/') + '/';
      if (file.charAt(0) != '/') { file = "/" + file; }
      baseURL = new URL("file", null, file);
      return new URL(baseURL,url).toString();

   }// end of makeAbsoluteURL()


}// end of DIFDocument
