
package owrti.xml;

import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Stack;
import java.util.Date;

/**
*
* This is an example implementation for the DIFDocument processing.
* This object produces an HTML document for the given DIFDocument.
*
* @date   August 12, 1998 ( Last Updated August 11, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class DIFtoHTMLProcessor
{
   /** */
   public static final void
   processDIF( DIFDocument doc, PrintStream out)
   {
      out.println("<html>");
      if(doc!=null)
      {
         out.println("<title>");
         out.println(doc.getFederationName()+" FED File ");
         out.println("</title>");

         out.println("<body>");
   
         out.println("<p><b>Federation Name : </b>"+doc.getFederationName()+"</p>");
         out.println("<p><b>Version         : </b>"+doc.getVersion()+"</p>");
         
         printSpaces(doc,out);
         printObjects(doc,out);
         printInteractions(doc,out);
         printMark(out);

         out.println("</body>");

      }// end of if
      out.println("</html>");

   }// end of processDIF()

   /** */
   static final void
   printMark( PrintStream out )
   {
      Date adate = new Date(System.currentTimeMillis());
      out.println("<hr>");
      out.print("<i>This document is produced by <b>DIFtoHTMLProcessor</b> on");
      out.println("  "+adate+"</i>");
      out.println("<hr>");

   }// end of printMark()

   /** */
   static final void
   printSpaces( DIFDocument doc, PrintStream out)
   {
      SpaceDef current ;
      Enumeration e = doc.getSpaces();
      Enumeration e2;
      String aDim;

      if(e!=null && e.hasMoreElements())
      {
        out.println("<p><table>"); 
        out.println("<tr> <td><i>Space Name</i></td> <td><i>Dimension Name</i></td> </tr>");

        for(;e.hasMoreElements();)
        {
           current = (SpaceDef) e.nextElement();
           if(current!=null)
           {
              out.print("<tr> <td>"+current.getSpaceName()+"</td>");
              e2 = current.getDimensions();
              
              if(e2!=null && e2.hasMoreElements())
              {
                 aDim = (String) e2.nextElement();    
                 if(aDim!=null) out.print(" <td>"+aDim+"</td> <br>");

              } else out.print(" <td></td> ");

              out.println("</tr>");

           }// end of if

        }// end of for

        out.println("</table></p>"); 

      }// end of if

   }// end of printSpaces()

   /** */
   static final void
   printObjects( DIFDocument doc, PrintStream out)
   {
      ObjClassDef current ;
      Enumeration e = doc.getObjectClasses();
      AttributeDef att ;
      Stack       myStack = new Stack();
      Enumeration e2;

      if(e!=null && e.hasMoreElements())
      {
        out.println(""); 
        out.println("<p><table>"); 
        out.println("<tr> <td align='CENTER'><i>Object Class Name</i></td> <td align='CENTER'><i>Attributes</i></td> </tr>");

        for(;e.hasMoreElements();)
        {
           current = (ObjClassDef) e.nextElement();
           myStack.push(current);
           while(!myStack.empty())
           {
           current = (ObjClassDef) myStack.pop();
           if(current!=null)
           {
              out.println("<tr> <td valign='TOP'>"+current.getFullName()+"</td> ");
              e2 = current.getAttributes();

              if(e2!=null && e2.hasMoreElements())
              {
                 out.println("<td><table>"); 
                 out.print("<tr><td>Name</td><td>Transport Type</td> <td> Order</td> </tr>");
                 for(;e2.hasMoreElements();)
                 {
                    att = (AttributeDef)e2.nextElement();

                    if(att!=null)
                    {
                      out.println("<tr><td>"+att.getName()+"</td>");

                      out.println("<td>"+att.getTransportType().toString()+
                                  "</td>");

                      out.println("<td>"+att.getOrderType().toString()+
                                  "</td></tr>");
                    }// end of if

                 }// end of for

                 out.println("</table></td>"); 

              } else out.print("<td></td>");

              out.println("</tr>");

              e2 = current.getChildren();
              if(e2!=null && e2.hasMoreElements())
                 for(;e2.hasMoreElements();) myStack.push(e2.nextElement());

           }// end of if

           }// end of while

        }// end of for

        out.println("</table></p>"); 

      }// end of if

   }// end of printObjects()

   /** */
   static final void
   printInteractions( DIFDocument doc, PrintStream out)
   {
      InteractionDef current ;
      InteractionDef parent ;
      Enumeration e = doc.getInteractionClasses();
      Enumeration e2 ;
      Stack       myStack = new Stack();
      String par_name;

      if(e!=null && e.hasMoreElements())
      {
        out.println(""); 
        out.println("<p><table>"); 
        out.print("<tr><td align='CENTER'><i>Interaction Class Name</i></td>"); 
        out.print("<td align='CENTER'><i>Inherites from </i></td>"); 
        out.print("<td align='CENTER'><i>Transport Type</i></td>"); 
        out.print("<td align='CENTER'><i>Order Type</i></td>"); 
        out.println("<td align='CENTER'><i>Parameters</i></td></tr>"); 

        for(;e.hasMoreElements();)
        {
           current = (InteractionDef) e.nextElement();
           myStack.push(current);
           while(!myStack.empty())
           {   
           current = (InteractionDef) myStack.pop();
           if(current!=null)
           {
              out.print("<tr><td valign='TOP'>"+current.getName()+"</td>"); 
              parent = current.getParent(); 

              if(parent!=null) 
                   out.print("<td valign='TOP'>"+parent.getName()+"</td>"); 
              else out.print("<td valign='TOP'> NONE </td>"); 

              out.print("<td valign='TOP'>"+current.getTransportType().toString()+"</td>"); 
              out.print("<td valign='TOP'>"+current.getOrderType().toString()+"</td>"); 
              e2 = current.getParameters();

              if(e2!=null && e2.hasMoreElements())
              {
                 out.print("<td valign='TOP'>"); 

                 for(;e2.hasMoreElements();)
                 {
                    par_name = (String) e2.nextElement();
                    if(par_name!=null) out.print(par_name+"<br>");
                 }// end of for

                 out.print("</td>"); 

              } else out.print("<td></td>"); 

              out.println("</tr>"); 
              out.println(); 

              e2 = current.getChildren();
              if(e2!=null && e2.hasMoreElements())
                 for(;e2.hasMoreElements();) myStack.push(e2.nextElement());

           }// end of if

           }// end of while

        }// end of for

        out.println("</table></p>"); 
        out.println(); 

      }// end of if

   }// end of printInteractions()

}// end of DIFtoHTMLProcessor
