package owrti.xml;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.PrintStream;


/**
*
* @date   August 11, 1998 ( Last Updated August 11, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
InteractionDef
{
   /** Interaction Class Name */
   String iname;

   /** Transportation Type */
   TransportType tType ;

   /** Order */
   OrderType oType ;

   /** Parameters */
   Vector  parameters;

   /** SubClasses */
   Hashtable  subClasses;

   /** SubClasses */
   InteractionDef  parent;

   /**
   */
   public InteractionDef(String _iname)
   {
      this(_iname, null, OrderType.RECEIVE, TransportType.RELIABLE);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, InteractionDef _parent)
   {
      this(_iname, _parent, OrderType.RECEIVE, TransportType.RELIABLE);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, OrderType _oType)
   {
      this(_iname, null, _oType, TransportType.RELIABLE);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, InteractionDef _parent, 
                         OrderType _oType)
   {
      this(_iname, _parent, _oType, TransportType.RELIABLE);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, TransportType _tType)
   {
      this(_iname, null, OrderType.RECEIVE, _tType);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, InteractionDef _parent, 
                         TransportType _tType)
   {
      this(_iname, _parent, OrderType.RECEIVE, _tType);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, OrderType _oType, TransportType _tType)
   {
      this(_iname, null, _oType, _tType);

   }// end of CONSTRUCTOR

   /**
   */
   public InteractionDef(String _iname, InteractionDef _parent, 
                         OrderType _oType, TransportType _tType)
   {
      iname  = _iname; 
      tType  = _tType;
      parent = _parent;
      oType  = _oType;
      parameters = new Vector();
      subClasses = new Hashtable();

   }// end of CONSTRUCTOR

   /**
   */
   public String getName() { return iname; }

   /**
   */
   public String getFullName() 
   {  
      String ret = iname; 

      if(parent!=null) ret = parent.getFullName() + "." + ret;

      return ret;

   }// end of getFullName()

   /**
   */
   public InteractionDef getParent() { return parent; }

   /**
   */
   public TransportType getTransportType() { return tType; }

   /**
   */
   public void setToReliable()   { tType=TransportType.RELIABLE; }
   public void setToBestEffort() { tType=TransportType.BEST_EFFORT; }

   /**
   */
   public OrderType getOrderType() { return oType; }

   /**
   */
   public void setToRO()  { oType=OrderType.RECEIVE; }
   public void setToTSO() { oType=OrderType.TIMESTAMP; }

   /**
   */
   public Enumeration getParameters() { return parameters.elements(); }

   /**
   */
   public void addParameter(String parName) 
   { if(!parameters.contains(parName)) parameters.addElement(parName); }

   /**
   */
   public Enumeration getSubClasses() { return subClasses.elements(); }

   /**
   */
   public Enumeration getChildren() { return subClasses.elements(); }

   /**
   */
   public void addChildren(InteractionDef subC) 
   { 
      if(!subClasses.containsKey(subC.getName())) 
          subClasses.put(subC.getName(),subC);

   }// end of addParameter()

   /**
   */
   public final void 
   printDIF(PrintStream o)
   {
      o.println( DIF_CONSTANTS.pfx_intclass+iname+" "+tType.toString()+" "+
                 oType.toString());
      printDIFParams(o,true);
      printDIFClasses(o,true);
      o.println(DIF_CONSTANTS.sfx_intclass);

   }// end of printDIF

   /**
   */
   final void 
   printDIFParams(PrintStream o, boolean isDIF)
   {
      String current ;
      Enumeration e = parameters.elements();

      if(e!=null && e.hasMoreElements())
      {
        for(;e.hasMoreElements();)
        {
           current = (String) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) o.println( DIF_CONSTANTS.pfx_parameter+current+
                                   DIF_CONSTANTS.sfx_parameter); 
              else o.println( HLAML_CONSTANTS.pfx_parameter+current+
                              HLAML_CONSTANTS.sfx_parameter); 

           }// end of if

        }// end of for

      }// end of if

   }// end of printDIFParams

   /**
   */
   final void 
   printDIFClasses(PrintStream o, boolean isDIF)
   {
      InteractionDef current ;
      Enumeration e = subClasses.elements();

      if(e!=null && e.hasMoreElements())
      {
        for(;e.hasMoreElements();)
        {
           current = (InteractionDef) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) current.printDIF(o);
              else current.printXML(o);
           }

        }// end of for

      }// end of if

   }// end of printDIFClasses

   /**
   */
   public final void 
   printXML(PrintStream o)
   {
      o.println(HLAML_CONSTANTS.pfx_intclass);

      o.println(HLAML_CONSTANTS.pfx_name+iname+HLAML_CONSTANTS.sfx_name);

      o.println( HLAML_CONSTANTS.pfx_transport+tType.toString()+
                 HLAML_CONSTANTS.sfx_transport);

      o.println( HLAML_CONSTANTS.pfx_order+oType.toString()+
                 HLAML_CONSTANTS.sfx_order);

      printDIFParams(o,false);
      printDIFClasses(o,false);

      o.println(HLAML_CONSTANTS.sfx_intclass);

   }// end of printDIF

}// end of InteractionDef
