package owrti.xml;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.PrintStream;


/**
*
* @date   August 11, 1998 ( Last Updated August 11, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
ObjClassDef
{
   /** Object Class Name */
   String oname;

   /** Attributes */
   Hashtable attribs;

   /** Parent */
   ObjClassDef parent;

   /** Parent */
   Hashtable subClasses;

   /**
   */
   public ObjClassDef(String _oname)
   {
      this(_oname,null);

   }// end of CONSTRUCTOR

   /**
   */
   public ObjClassDef(String _oname,ObjClassDef _parent)
   {
      oname = _oname.trim();
      parent = _parent;
      subClasses = new Hashtable();
      attribs = new Hashtable();

   }// end of CONSTRUCTOR

   /**
   */
   public String getName() { return oname; }

   /**
   */
   public String getFullName() 
   { 
      String ret = oname; 
      if(parent!=null) ret = parent.getFullName()+"."+ret;
      return ret;

   }// end of getFullName()

   /**
   */
   public Enumeration getAttributes() { return attribs.elements(); }

   /**
   */
   public void addAttribute(AttributeDef att) 
   { 
     if(att!=null && !attribs.containsKey(att.getName()) ) 
        attribs.put(att.getName(),att);

   }// end of addAttribute()

   /**
   */
   public Enumeration getChildren() { return subClasses.elements(); }

   /**
   */
   public void addChildren(ObjClassDef subC) 
   { 
     if(subC!=null && !subClasses.containsKey(subC.getName()) ) 
        subClasses.put(subC.getName(),subC);

   }// end of addChildren()

   /**
   */
   public ObjClassDef getParent() { return parent; }

   /**
   */
   public final void
   printDIF(PrintStream o)
   {
      o.println(DIF_CONSTANTS.pfx_objclass+oname);
      printDIFAttribs(o,true);
      printDIFClasses(o,true);
      o.println(DIF_CONSTANTS.sfx_objclass);

   }// end of printDIF

   /**
   */
   final void 
   printDIFAttribs(PrintStream o, boolean isDIF)
   {
      AttributeDef current ;
      Enumeration e = attribs.elements();

      if(e!=null && e.hasMoreElements())
      {
        if(!isDIF) o.println(HLAML_CONSTANTS.pfx_attList); 
        for(;e.hasMoreElements();)
        {
           current = (AttributeDef) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) current.printDIF(o);
              else current.printXML(o);
           }

        }// end of for
        if(!isDIF) o.println(HLAML_CONSTANTS.sfx_attList); 

      }// end of if

   }// end of printDIFAttribs

   final void 
   printDIFClasses(PrintStream o, boolean isDIF)
   {
      ObjClassDef current ;
      Enumeration e = subClasses.elements();

      if(e!=null && e.hasMoreElements())
      {
        for(;e.hasMoreElements();)
        {
           current = (ObjClassDef) e.nextElement();
           if(current!=null) 
           {
              if(isDIF) current.printDIF(o);
              else current.printXML(o);
           }

        }// end of for

      }// end of if

   }// end of printDIFClasses

   /**
   */
   public final void
   printXML(PrintStream o)
   {
      o.println(HLAML_CONSTANTS.pfx_objclass);
      o.println(HLAML_CONSTANTS.pfx_name+oname+HLAML_CONSTANTS.sfx_name);
      printDIFAttribs(o,false);
      printDIFClasses(o,false);
      o.println(HLAML_CONSTANTS.sfx_objclass);

   }// end of printDIF

}// end of ObjClassDef
